@@ -17,30 +17,26 @@
 #include "misc.h"
 #include "settings.h"
 #include "grab.h"
-#include "button.h"
 
 static char *buttonNames[BI_COUNT];
 static IconNode *buttonIcons[BI_COUNT];
 
 static void DrawBorderHelper(const ClientNode *np);
-static void DrawBorderHandles(const ClientNode *np, GC gc);
+static void DrawBorderHandles(const ClientNode *np,
+                              Pixmap canvas, GC gc);
 static void DrawBorderButtons(const ClientNode *np,
                               Pixmap canvas, GC gc);
 static char DrawBorderIcon(BorderIconType t,
                            unsigned xoffset, unsigned yoffset,
-                           const VisualData *visual, Pixmap canvas);
+                           Pixmap canvas, long fg);
 static void DrawCloseButton(unsigned xoffset, unsigned yoffset,
-                            const VisualData *visual,
-                            Pixmap canvas, GC gc);
+                            Pixmap canvas, GC gc, long fg);
 static void DrawMaxIButton(unsigned xoffset, unsigned yoffset,
-                           const VisualData *visual,
-                           Pixmap canvas, GC gc);
+                           Pixmap canvas, GC gc, long fg);
 static void DrawMaxAButton(unsigned xoffset, unsigned yoffset,
-                           const VisualData *visual,
-                           Pixmap canvas, GC gc);
+                           Pixmap canvas, GC gc, long fg);
 static void DrawMinButton(unsigned xoffset, unsigned yoffset,
-                          const VisualData *visual,
-                          Pixmap canvas, GC gc);
+                          Pixmap canvas, GC gc, long fg);
 static unsigned GetButtonCount(const ClientNode *np);
 
 #ifdef USE_SHAPE
@@ -63,22 +59,35 @@ void StartupBorders(void)
       if(buttonNames[i]) {
          buttonIcons[i] = LoadNamedIcon(buttonNames[i], 1, 1);
          Release(buttonNames[i]);
+         buttonNames[i] = NULL;
       } else {
          buttonIcons[i] = NULL;
       }
    }
 
+   /* Always load a menu icon for windows without one. */
+   if(buttonIcons[BI_MENU] == NULL) {
+      buttonIcons[BI_MENU] = GetDefaultIcon();
+   }
 }
 
-/** Release server resources. */
-void ShutdownBorders(void)
+/** Destroy structures. */
+void DestroyBorders(void)
 {
+   unsigned i;
+   for(i = 0; i < BI_COUNT; i++)
+   {
+      if(buttonNames[i]) {
+         Release(buttonNames[i]);
+         buttonNames[i] = NULL;
+      }
+   }
 }
 
 /** Get the size of the icon to display on a window. */
 int GetBorderIconSize(void)
 {
-   if(settings.handles) {
+   if(settings.windowDecorations == DECO_MOTIF) {
       return settings.titleHeight - 4;
    } else {
       return settings.titleHeight - 6;
@@ -169,20 +178,17 @@ BorderActionType GetBorderActionType(const ClientNode *np, int x, int y)
    }
 
    /* Check south east/west and north east/west resizing. */
-   if(   np->width >= settings.titleHeight * 2
-      && np->height >= settings.titleHeight * 2) {
-      if(y > np->height + north - settings.titleHeight) {
-         if(x < settings.titleHeight) {
-            return (BA_RESIZE_S | BA_RESIZE_W | BA_RESIZE) & resizeMask;
-         } else if(x > np->width + west - settings.titleHeight) {
-            return (BA_RESIZE_S | BA_RESIZE_E | BA_RESIZE) & resizeMask;
-         }
-      } else if(y < settings.titleHeight) {
-         if(x < settings.titleHeight) {
-            return (BA_RESIZE_N | BA_RESIZE_W | BA_RESIZE) & resizeMask;
-         } else if(x > np->width + west - settings.titleHeight) {
-            return (BA_RESIZE_N | BA_RESIZE_E | BA_RESIZE) & resizeMask;
-         }
+   if(y > np->height + north - settings.titleHeight) {
+      if(x < settings.titleHeight) {
+         return (BA_RESIZE_S | BA_RESIZE_W | BA_RESIZE) & resizeMask;
+      } else if(x > np->width + west - settings.titleHeight) {
+         return (BA_RESIZE_S | BA_RESIZE_E | BA_RESIZE) & resizeMask;
+      }
+   } else if(y < settings.titleHeight) {
+      if(x < settings.titleHeight) {
+         return (BA_RESIZE_N | BA_RESIZE_W | BA_RESIZE) & resizeMask;
+      } else if(x > np->width + west - settings.titleHeight) {
+         return (BA_RESIZE_N | BA_RESIZE_E | BA_RESIZE) & resizeMask;
       }
    }
 
@@ -212,6 +218,12 @@ void ResetBorder(const ClientNode *np)
    int north, south, east, west;
    int width, height;
 
+   if(np->parent == None) {
+      JXMoveResizeWindow(display, np->window, np->x, np->y,
+                         np->width, np->height);
+      return;
+   }
+
    GrabServer();
 
    /* Determine the size of the window. */
@@ -232,7 +244,7 @@ void ResetBorder(const ClientNode *np)
                       width, height);
 
 #ifdef USE_SHAPE
-   if(settings.cornerRadius > 0) {
+   if(settings.cornerRadius > 0 || (np->state.status & STAT_SHAPED)) {
 
       /* First set the shape to the window border. */
       shapePixmap = JXCreatePixmap(display, np->parent, width, height, 1);
@@ -296,7 +308,7 @@ void ResetBorder(const ClientNode *np)
 }
 
 /** Draw a client border. */
-void DrawBorder(const ClientNode *np)
+void DrawBorder(ClientNode *np)
 {
 
    Assert(np);
@@ -316,8 +328,11 @@ void DrawBorder(const ClientNode *np)
       return;
    }
 
+   /* Create the frame if needed. */
+   ReparentClient(np);
+
    /* Return if there is no border. */
-   if(!(np->state.border & (BORDER_TITLE | BORDER_OUTLINE))) {
+   if(np->parent == None) {
       return;
    }
 
@@ -337,7 +352,6 @@ void DrawBorderHelper(const ClientNode *np)
 
    int north, south, east, west;
    unsigned int width, height;
-   int iconSize;
 
    unsigned int buttonCount;
    int titleWidth;
@@ -346,7 +360,6 @@ void DrawBorderHelper(const ClientNode *np)
 
    Assert(np);
 
-   iconSize = GetBorderIconSize();
    GetBorderSize(&np->state, &north, &south, &east, &west);
    width = np->width + east + west;
    height = np->height + north + south;
@@ -357,22 +370,21 @@ void DrawBorderHelper(const ClientNode *np)
       borderTextColor = COLOR_TITLE_ACTIVE_FG;
       titleColor1 = colors[COLOR_TITLE_ACTIVE_BG1];
       titleColor2 = colors[COLOR_TITLE_ACTIVE_BG2];
-      outlineColor = colors[COLOR_BORDER_ACTIVE_LINE];
+      outlineColor = colors[COLOR_TITLE_ACTIVE_DOWN];
 
    } else {
 
       borderTextColor = COLOR_TITLE_FG;
       titleColor1 = colors[COLOR_TITLE_BG1];
       titleColor2 = colors[COLOR_TITLE_BG2];
-      outlineColor = colors[COLOR_BORDER_LINE];
+      outlineColor = colors[COLOR_TITLE_DOWN];
 
    }
 
    /* Set parent background to reduce flicker. */
    JXSetWindowBackground(display, np->parent, titleColor2);
 
-   canvas = JXCreatePixmap(display, np->parent, width, north,
-                           np->visual.depth);
+   canvas = JXCreatePixmap(display, np->parent, width, north, rootDepth);
    gc = JXCreateGC(display, canvas, 0, NULL);
 
    /* Clear the window with the right color. */
@@ -383,32 +395,50 @@ void DrawBorderHelper(const ClientNode *np)
    buttonCount = GetButtonCount(np);
    titleWidth = width - east - west - 5;
    titleWidth -= settings.titleHeight * (buttonCount + 1);
-   titleWidth -= settings.handles ? (buttonCount + 1) : 0;
+   titleWidth -= settings.windowDecorations == DECO_MOTIF
+               ? (buttonCount + 1) : 0;
 
    /* Draw the top part (either a title or north border). */
    if((np->state.border & BORDER_TITLE) &&
       settings.titleHeight > settings.borderWidth) {
 
       const unsigned startx = west + 1;
-      const unsigned starty = settings.handles ? (south - 1) : 0;
+      const unsigned starty = settings.windowDecorations == DECO_MOTIF
+                            ? (south - 1) : 0;
 
       /* Draw a title bar. */
       DrawHorizontalGradient(canvas, gc, titleColor1, titleColor2,
                              0, 1, width, settings.titleHeight - 2);
 
       /* Draw the icon. */
-      if(np->icon && np->width >= settings.titleHeight) {
-         PutIcon(&np->visual, np->icon, canvas, colors[borderTextColor],
+#ifdef USE_ICONS
+      if(np->width >= settings.titleHeight) {
+         const int iconSize = GetBorderIconSize();
+         IconNode *icon = np->icon ? np->icon : buttonIcons[BI_MENU];
+         PutIcon(icon, canvas, colors[borderTextColor],
                  startx, starty + (settings.titleHeight - iconSize) / 2,
                  iconSize, iconSize);
       }
+#endif
 
       if(np->name && np->name[0] && titleWidth > 0) {
          const int sheight = GetStringHeight(FONT_BORDER);
-         const unsigned titlex = startx + settings.titleHeight
-                               + (settings.handles ? 4 : 0);
-         const unsigned titley = starty + (settings.titleHeight - sheight) / 2;
-         RenderString(&np->visual, canvas, FONT_BORDER, borderTextColor,
+         const int textWidth = GetStringWidth(FONT_BORDER, np->name);
+         unsigned titlex, titley;
+         int xoffset = 0;
+         switch (settings.titleTextAlignment) {
+         case ALIGN_CENTER:
+            xoffset = (titleWidth - textWidth) / 2;
+            break;
+         case ALIGN_RIGHT:
+            xoffset = (titleWidth - textWidth);
+            break;
+         }
+         xoffset = Max(xoffset, 0);
+         titlex = startx + settings.titleHeight + xoffset
+                + (settings.windowDecorations == DECO_MOTIF ? 4 : 0);
+         titley = starty + (settings.titleHeight - sheight) / 2;
+         RenderString(canvas, FONT_BORDER, borderTextColor,
                       titlex, titley, titleWidth, np->name);
       }
 
@@ -416,18 +446,25 @@ void DrawBorderHelper(const ClientNode *np)
 
    }
 
+   /* Copy the pixmap (for the title bar) to the window. */
 
-   /* Copy the title bar to the window. */
-   JXCopyArea(display, canvas, np->parent, gc, 0, 1,
-              width, north - 1, 0, 1);
+   /* Copy the pixmap for the title bar and clear the part of
+    * the window to be drawn directly. */
+   if(settings.windowDecorations == DECO_MOTIF) {
+      JXCopyArea(display, canvas, np->parent, gc, 2, 2,
+         width - 4, north - 2, 2, 2);
+      JXClearArea(display, np->parent,
+         2, north, width - 4, height - north - 2, False);
+   } else {
+      JXCopyArea(display, canvas, np->parent, gc, 1, 1,
+         width - 2, north - 1, 1, 1);
+      JXClearArea(display, np->parent,
+         1, north, width - 2, height - north - 1, False);
+   }
 
-   /* Window outline.
-    * These are drawn directly to the window.
-    */
-   JXClearArea(display, np->parent, 0, north,
-               width, height - north, False);
-   if(settings.handles) {
-      DrawBorderHandles(np, gc);
+   /* Window outline. */
+   if(settings.windowDecorations == DECO_MOTIF) {
+      DrawBorderHandles(np, np->parent, gc);
    } else {
       JXSetForeground(display, gc, outlineColor);
       if(np->state.status & STAT_SHADED) {
@@ -450,8 +487,8 @@ void DrawBorderHelper(const ClientNode *np)
          }
          if(!(np->state.maxFlags & (MAX_RIGHT | MAX_HORIZ))) {
             /* Right */
-            JXDrawLine(display, np->parent, gc,
-                       width - 1, 0, width - 1, height);
+            JXDrawLine(display, np->parent, gc, width - 1, 0,
+               width - 1, height);
          }
       } else {
          DrawRoundedRectangle(np->parent, gc, 0, 0, width - 1, height - 1,
@@ -465,7 +502,7 @@ void DrawBorderHelper(const ClientNode *np)
 }
 
 /** Draw window handles. */
-void DrawBorderHandles(const ClientNode *np, GC gc)
+void DrawBorderHandles(const ClientNode *np, Pixmap canvas, GC gc)
 {
    XSegment segments[8];
    long pixelUp, pixelDown;
@@ -553,7 +590,7 @@ void DrawBorderHandles(const ClientNode *np, GC gc)
 
    /* Draw pixel-up segments. */
    JXSetForeground(display, gc, pixelUp);
-   JXDrawSegments(display, np->parent, gc, segments, offset);
+   JXDrawSegments(display, canvas, gc, segments, offset);
    offset = 0;
 
    /* Bottom title border. */
@@ -620,7 +657,7 @@ void DrawBorderHandles(const ClientNode *np, GC gc)
 
    /* Draw pixel-down segments. */
    JXSetForeground(display, gc, pixelDown);
-   JXDrawSegments(display, np->parent, gc, segments, offset);
+   JXDrawSegments(display, canvas, gc, segments, offset);
    offset = 0;
 
    /* Draw marks */
@@ -670,7 +707,7 @@ void DrawBorderHandles(const ClientNode *np, GC gc)
 
       /* Draw pixel-down segments. */
       JXSetForeground(display, gc, pixelDown);
-      JXDrawSegments(display, np->parent, gc, segments, 8);
+      JXDrawSegments(display, canvas, gc, segments, 8);
 
       /* Upper left */
       segments[0].x1 = settings.titleHeight + settings.borderWidth;
@@ -714,7 +751,7 @@ void DrawBorderHandles(const ClientNode *np, GC gc)
 
       /* Draw pixel-up segments. */
       JXSetForeground(display, gc, pixelUp);
-      JXDrawSegments(display, np->parent, gc, segments, 8);
+      JXDrawSegments(display, canvas, gc, segments, 8);
 
    }
 }
@@ -736,7 +773,7 @@ unsigned GetButtonCount(const ClientNode *np)
    }
 
    buttonWidth = settings.titleHeight;
-   buttonWidth += settings.handles ? 1 : 0;
+   buttonWidth += settings.windowDecorations == DECO_MOTIF ? 1 : 0;
 
    GetBorderSize(&np->state, &north, &south, &east, &west);
 
@@ -796,29 +833,34 @@ void DrawBorderButtons(const ClientNode *np, Pixmap canvas, GC gc)
       pixelDown = colors[COLOR_TITLE_DOWN];
    }
 
+   if(settings.windowDecorations == DECO_MOTIF) {
+      JXSetForeground(display, gc, pixelDown);
+      JXDrawLine(display, canvas, gc,
+                      west + settings.titleHeight - 1,
+                      south,
+                      west + settings.titleHeight - 1,
+                      south + settings.titleHeight);
+      JXSetForeground(display, gc, pixelUp);
+      JXDrawLine(display, canvas, gc,
+                 west + settings.titleHeight,
+                 south,
+                 west + settings.titleHeight,
+                 south + settings.titleHeight);
+   }
+
    /* Close button. */
-   yoffset = settings.handles ? (south - 1) : 0;
+   yoffset = settings.windowDecorations == DECO_MOTIF ? (south - 1) : 0;
    if(np->state.border & BORDER_CLOSE) {
 
       JXSetForeground(display, gc, color);
-      DrawCloseButton(xoffset, yoffset, &np->visual, canvas, gc);
+      DrawCloseButton(xoffset, yoffset, canvas, gc, color);
 
-      if(settings.handles) {
+      if(settings.windowDecorations == DECO_MOTIF) {
          JXSetForeground(display, gc, pixelDown);
-         JXDrawLine(display, canvas, gc,
-                         west + settings.titleHeight - 1,
-                         south,
-                         west + settings.titleHeight - 1,
-                         south + settings.titleHeight);
          JXDrawLine(display, canvas, gc, xoffset - 1,
                     south, xoffset - 1,
                     south + settings.titleHeight);
          JXSetForeground(display, gc, pixelUp);
-         JXDrawLine(display, canvas, gc,
-                    east + settings.titleHeight,
-                    south,
-                    east + settings.titleHeight,
-                    south + settings.titleHeight);
          JXDrawLine(display, canvas, gc, xoffset,
                     south, xoffset, south + settings.titleHeight);
          xoffset -= 1;
@@ -835,27 +877,17 @@ void DrawBorderButtons(const ClientNode *np, Pixmap canvas, GC gc)
 
       JXSetForeground(display, gc, color);
       if(np->state.maxFlags) {
-         DrawMaxAButton(xoffset, yoffset, &np->visual, canvas, gc);
+         DrawMaxAButton(xoffset, yoffset, canvas, gc, color);
       } else {
-         DrawMaxIButton(xoffset, yoffset, &np->visual, canvas, gc);
+         DrawMaxIButton(xoffset, yoffset, canvas, gc, color);
       }
 
-      if(settings.handles) {
+      if(settings.windowDecorations == DECO_MOTIF) {
          JXSetForeground(display, gc, pixelDown);
-         JXDrawLine(display, canvas, gc,
-                         west + settings.titleHeight - 1,
-                         south,
-                         west + settings.titleHeight - 1,
-                         south + settings.titleHeight);
          JXDrawLine(display, canvas, gc, xoffset - 1,
                     south, xoffset - 1,
                     south + settings.titleHeight);
          JXSetForeground(display, gc, pixelUp);
-         JXDrawLine(display, canvas, gc,
-                    east + settings.titleHeight,
-                    south,
-                    east + settings.titleHeight,
-                    south + settings.titleHeight);
          JXDrawLine(display, canvas, gc, xoffset,
                     south, xoffset, south + settings.titleHeight);
          xoffset -= 1;
@@ -871,24 +903,14 @@ void DrawBorderButtons(const ClientNode *np, Pixmap canvas, GC gc)
    if(np->state.border & BORDER_MIN) {
 
       JXSetForeground(display, gc, color);
-      DrawMinButton(xoffset, yoffset, &np->visual, canvas, gc);
+      DrawMinButton(xoffset, yoffset, canvas, gc, color);
 
-      if(settings.handles) {
+      if(settings.windowDecorations == DECO_MOTIF) {
          JXSetForeground(display, gc, pixelDown);
-         JXDrawLine(display, canvas, gc,
-                         west + settings.titleHeight - 1,
-                         south,
-                         west + settings.titleHeight - 1,
-                         south + settings.titleHeight);
          JXDrawLine(display, canvas, gc, xoffset - 1,
                     south, xoffset - 1,
                     south + settings.titleHeight);
          JXSetForeground(display, gc, pixelUp);
-         JXDrawLine(display, canvas, gc,
-                    east + settings.titleHeight,
-                    south,
-                    east + settings.titleHeight,
-                    south + settings.titleHeight);
          JXDrawLine(display, canvas, gc, xoffset,
                     south, xoffset, south + settings.titleHeight);
          xoffset -= 1;
@@ -899,18 +921,11 @@ void DrawBorderButtons(const ClientNode *np, Pixmap canvas, GC gc)
 /** Attempt to draw a border icon. */
 char DrawBorderIcon(BorderIconType t,
                     unsigned xoffset, unsigned yoffset,
-                    const VisualData *visual, Pixmap canvas)
+                    Pixmap canvas, long fg)
 {
    if(buttonIcons[t]) {
-      ButtonNode button;
-      ResetButton(&button, canvas, visual);
-      button.x       = xoffset;
-      button.y       = yoffset;
-      button.width   = settings.titleHeight;
-      button.height  = settings.titleHeight;
-      button.icon    = buttonIcons[t];
-      button.fill    = 0;
-      DrawButton(&button);
+      PutIcon(buttonIcons[t], canvas, fg, xoffset + 2, yoffset + 2,
+              settings.titleHeight - 4, settings.titleHeight - 4);
       return 1;
    } else {
       return 0;
@@ -919,15 +934,14 @@ char DrawBorderIcon(BorderIconType t,
 
 /** Draw a close button. */
 void DrawCloseButton(unsigned xoffset, unsigned yoffset,
-                     const VisualData *visual,
-                     Pixmap canvas, GC gc)
+                     Pixmap canvas, GC gc, long fg)
 {
    XSegment segments[2];
    unsigned size;
    unsigned x1, y1;
    unsigned x2, y2;
 
-   if(DrawBorderIcon(BI_CLOSE, xoffset, yoffset, visual, canvas)) {
+   if(DrawBorderIcon(BI_CLOSE, xoffset, yoffset, canvas, fg)) {
       return;
    }
 
@@ -957,8 +971,7 @@ void DrawCloseButton(unsigned xoffset, unsigned yoffset,
 
 /** Draw an inactive maximize button. */
 void DrawMaxIButton(unsigned xoffset, unsigned yoffset,
-                    const VisualData *visual,
-                    Pixmap canvas, GC gc)
+                    Pixmap canvas, GC gc, long fg)
 {
 
    XSegment segments[5];
@@ -966,7 +979,7 @@ void DrawMaxIButton(unsigned xoffset, unsigned yoffset,
    unsigned int x1, y1;
    unsigned int x2, y2;
 
-   if(DrawBorderIcon(BI_MAX, xoffset, yoffset, visual, canvas)) {
+   if(DrawBorderIcon(BI_MAX, xoffset, yoffset, canvas, fg)) {
       return;
    }
 
@@ -1011,8 +1024,7 @@ void DrawMaxIButton(unsigned xoffset, unsigned yoffset,
 
 /** Draw an active maximize button. */
 void DrawMaxAButton(unsigned xoffset, unsigned yoffset,
-                    const VisualData *visual,
-                    Pixmap canvas, GC gc)
+                    Pixmap canvas, GC gc, long fg)
 {
    XSegment segments[8];
    unsigned size;
@@ -1020,7 +1032,7 @@ void DrawMaxAButton(unsigned xoffset, unsigned yoffset,
    unsigned x2, y2;
    unsigned x3, y3;
 
-   if(DrawBorderIcon(BI_MAX_ACTIVE, xoffset, yoffset, visual, canvas)) {
+   if(DrawBorderIcon(BI_MAX_ACTIVE, xoffset, yoffset, canvas, fg)) {
       return;
    }
 
@@ -1081,15 +1093,14 @@ void DrawMaxAButton(unsigned xoffset, unsigned yoffset,
 
 /** Draw a minimize button. */
 void DrawMinButton(unsigned xoffset, unsigned yoffset,
-                   const VisualData *visual,
-                   Pixmap canvas, GC gc)
+                   Pixmap canvas, GC gc, long fg)
 {
 
    unsigned int size;
    unsigned int x1, y1;
    unsigned int x2, y2;
 
-   if(DrawBorderIcon(BI_MIN, xoffset, yoffset, visual, canvas)) {
+   if(DrawBorderIcon(BI_MIN, xoffset, yoffset, canvas, fg)) {
       return;
    }
 
@@ -1150,13 +1161,15 @@ void GetBorderSize(const ClientState *state,
 
       if(state->border & BORDER_TITLE) {
          *north = settings.titleHeight;
-      } else {
+      } else if(settings.windowDecorations == DECO_MOTIF) {
          *north = 0;
+      } else {
+         *north = settings.borderWidth;
       }
       if(state->maxFlags & MAX_VERT) {
          *south = 0;
       } else {
-         if(settings.handles) {
+         if(settings.windowDecorations == DECO_MOTIF) {
             *north += settings.borderWidth;
             *south = settings.borderWidth;
          } else {
@@ -1317,6 +1330,9 @@ void FillRoundedRectangle(Drawable d, GC gc, int x, int y,
 /** Set the icon to use for a button. */
 void SetBorderIcon(BorderIconType t, const char *name)
 {
+   if(buttonNames[t]) {
+      Release(buttonNames[t]);
+   }
    buttonNames[t] = CopyString(name);
 }
  
