@@ -11,9 +11,9 @@
 #define BUTTON_H
 
 #include "font.h"
+#include "settings.h"
 
 struct IconNode;
-struct VisualData;
 
 /** Button types. */
 typedef unsigned char ButtonType;
@@ -25,11 +25,6 @@ typedef unsigned char ButtonType;
 #define BUTTON_TASK        5  /**< Item in the task list. */
 #define BUTTON_TASK_ACTIVE 6  /**< Active item in the task list. */
 
-/** Alignment of content in a button. */
-typedef unsigned char AlignmentType;
-#define ALIGN_LEFT   0  /**< Left align. */
-#define ALIGN_CENTER 1  /**< Center align. */
-
 /** Data used for drawing a button. */
 typedef struct {
 
@@ -39,7 +34,6 @@ typedef struct {
    char fill;                 /**< Determine if we should fill. */
    char border;               /**< Determine if we should draw a border. */
 
-   const struct VisualData *visual;  /**< Visual and depth to use. */
    Drawable drawable;         /**< The place to put the button. */
 
    int x, y;                  /**< The coordinates to render the button. */
@@ -60,8 +54,6 @@ void DrawButton(ButtonNode *bp);
  * @param d The drawable to use.
  * @param g The graphics context to use.
  */
-void ResetButton(ButtonNode *bp, Drawable d,
-                 const struct VisualData *visual);
+void ResetButton(ButtonNode *bp, Drawable d);
 
 #endif /* BUTTON_H */
-
