@@ -14,7 +14,6 @@
 #include "group.h"
 #include "tray.h"
 #include "confirm.h"
-#include "key.h"
 #include "cursor.h"
 #include "taskbar.h"
 #include "screen.h"
@@ -25,13 +24,13 @@
 #include "settings.h"
 #include "timing.h"
 #include "grab.h"
+#include "desktop.h"
 
 static ClientNode *activeClient;
 
 unsigned int clientCount;
 
 static void LoadFocus(void);
-static void ReparentClient(ClientNode *np, char notOwner);
 static void RestackTransients(const ClientNode *np);
 static void MinimizeTransients(ClientNode *np, char lower);
 static void RestoreTransients(ClientNode *np, char raise);
@@ -140,6 +139,7 @@ ClientNode *AddClientWindow(Window w, char alreadyMapped, char notOwner)
    memset(np, 0, sizeof(ClientNode));
 
    np->window = w;
+   np->parent = None;
    np->owner = None;
    np->state.desktop = currentDesktop;
 
@@ -147,8 +147,6 @@ ClientNode *AddClientWindow(Window w, char alreadyMapped, char notOwner)
    np->y = attr.y;
    np->width = attr.width;
    np->height = attr.height;
-   np->visual.depth = attr.depth;
-   np->visual.visual = attr.visual;
    np->cmap = attr.colormap;
    np->state.status = STAT_NONE;
    np->state.maxFlags = MAX_NONE;
@@ -183,25 +181,33 @@ ClientNode *AddClientWindow(Window w, char alreadyMapped, char notOwner)
    nodes[np->state.layer] = np;
 
    SetDefaultCursor(np->window);
-   ReparentClient(np, notOwner);
-   PlaceClient(np, alreadyMapped);
 
-   if(!((np->state.status & STAT_FULLSCREEN) || np->state.maxFlags)) {
-      int north, south, east, west;
-      GetBorderSize(&np->state, &north, &south, &east, &west);
-      JXMoveResizeWindow(display, np->parent, np->x - west, np->y - north,
-                         np->width + east + west, np->height + north + south);
-      JXMoveResizeWindow(display, np->window, west, north,
-                         np->width, np->height);
+   if(notOwner) {
+      XSetWindowAttributes sattr;
+      JXAddToSaveSet(display, np->window);
+      sattr.event_mask
+         = EnterWindowMask
+         | ColormapChangeMask
+         | PropertyChangeMask
+         | KeyReleaseMask
+         | StructureNotifyMask;
+      sattr.do_not_propagate_mask = ButtonPressMask
+                                  | ButtonReleaseMask
+                                  | PointerMotionMask
+                                  | KeyPressMask
+                                  | KeyReleaseMask;
+      JXChangeWindowAttributes(display, np->window,
+                               CWEventMask | CWDontPropagate, &sattr);
    }
+   JXGrabButton(display, AnyButton, AnyModifier, np->window, True,
+                ButtonPressMask, GrabModeSync, GrabModeAsync, None, None);
 
-   /* If one of these fails we are SOL, so who cares. */
+   PlaceClient(np, alreadyMapped);
+   ReparentClient(np);
    XSaveContext(display, np->window, clientContext, (void*)np);
-   XSaveContext(display, np->parent, frameContext, (void*)np);
 
    if(np->state.status & STAT_MAPPED) {
       JXMapWindow(display, np->window);
-      JXMapWindow(display, np->parent);
    }
 
    clientCount += 1;
@@ -212,6 +218,8 @@ ClientNode *AddClientWindow(Window w, char alreadyMapped, char notOwner)
 
    if(np->state.status & STAT_OPACITY) {
       SetOpacity(np, np->state.opacity, 1);
+   } else {
+      SetOpacity(np, settings.inactiveClientOpacity, 1);
    }
    if(np->state.status & STAT_STICKY) {
       SetCardinalAtom(np->window, ATOM_NET_WM_DESKTOP, ~0UL);
@@ -296,7 +304,9 @@ void MinimizeTransients(ClientNode *np, char lower)
    /* Unmap the window and update its state. */
    if(np->state.status & (STAT_MAPPED | STAT_SHADED)) {
       UnmapClient(np);
-      JXUnmapWindow(display, np->parent);
+      if(np->parent != None) {
+         JXUnmapWindow(display, np->parent);
+      }
    }
    np->state.status |= STAT_MINIMIZED;
 
@@ -397,10 +407,14 @@ void SetClientWithdrawn(ClientNode *np)
 
    if(np->state.status & STAT_MAPPED) {
       UnmapClient(np);
-      JXUnmapWindow(display, np->parent);
+      if(np->parent != None) {
+         JXUnmapWindow(display, np->parent);
+      }
    } else if(np->state.status & STAT_SHADED) {
       if(!(np->state.status & STAT_MINIMIZED)) {
-         JXUnmapWindow(display, np->parent);
+         if(np->parent != None) {
+            JXUnmapWindow(display, np->parent);
+         }
       }
    }
 
@@ -429,10 +443,14 @@ void RestoreTransients(ClientNode *np, char raise)
    /* Restore this window. */
    if(!(np->state.status & STAT_MAPPED)) {
       if(np->state.status & STAT_SHADED) {
-         JXMapWindow(display, np->parent);
+         if(np->parent != None) {
+            JXMapWindow(display, np->parent);
+         }
       } else {
          JXMapWindow(display, np->window);
-         JXMapWindow(display, np->parent);
+         if(np->parent != None) {
+            JXMapWindow(display, np->parent);
+         }
          np->state.status |= STAT_MAPPED;
       }
    }
@@ -459,6 +477,9 @@ void RestoreTransients(ClientNode *np, char raise)
 /** Restore a client window and its transients. */
 void RestoreClient(ClientNode *np, char raise)
 {
+   if((np->state.status & STAT_FIXED) && !(np->state.status & STAT_STICKY)) {
+      ChangeDesktop(np->state.desktop);
+   }
    RestoreTransients(np, raise);
    RequireRestack();
    RequireTaskUpdate();
@@ -614,37 +635,37 @@ void SetClientDesktop(ClientNode *np, unsigned int desktop)
 /** Hide a client without unmapping. This will not update transients. */
 void HideClient(ClientNode *np)
 {
-
-   Assert(np);
-
    if(activeClient == np) {
       activeClient = NULL;
    }
    np->state.status |= STAT_HIDDEN;
    if(np->state.status & (STAT_MAPPED | STAT_SHADED)) {
-      JXUnmapWindow(display, np->parent);
+      if(np->parent != None) {
+         JXUnmapWindow(display, np->parent);
+      } else {
+         JXUnmapWindow(display, np->window);
+      }
    }
-
 }
 
 /** Show a hidden client. This will not update transients. */
 void ShowClient(ClientNode *np)
 {
-
-   Assert(np);
-
    if(np->state.status & STAT_HIDDEN) {
       np->state.status &= ~STAT_HIDDEN;
       if(np->state.status & (STAT_MAPPED | STAT_SHADED)) {
          if(!(np->state.status & STAT_MINIMIZED)) {
-            JXMapWindow(display, np->parent);
+            if(np->parent != None) {
+               JXMapWindow(display, np->parent);
+            } else {
+               JXMapWindow(display, np->window);
+            }
             if(np->state.status & STAT_ACTIVE) {
                FocusClient(np);
             }
          }
       }
    }
-
 }
 
 /** Maximize a client window. */
@@ -797,13 +818,10 @@ void SetClientFullScreen(ClientNode *np, char fullScreen)
 /** Set the active client. */
 void FocusClient(ClientNode *np)
 {
-
-   Assert(np);
-
    if(np->state.status & STAT_HIDDEN) {
       return;
    }
-   if(!(np->state.status & STAT_CANFOCUS)) {
+   if(!(np->state.status & (STAT_CANFOCUS | STAT_TAKEFOCUS))) {
       return;
    }
 
@@ -833,8 +851,6 @@ void FocusClient(ClientNode *np)
       SetWindowAtom(rootWindow, ATOM_NET_ACTIVE_WINDOW, np->window);
       if(np->state.status & STAT_CANFOCUS) {
          JXSetInputFocus(display, np->window, RevertToParent, eventTime);
-      } else {
-         JXSetInputFocus(display, np->parent, RevertToParent, eventTime);
       }
       if(np->state.status & STAT_TAKEFOCUS) {
          SendClientMessage(np->window, ATOM_WM_PROTOCOLS, ATOM_WM_TAKE_FOCUS);
@@ -1093,11 +1109,19 @@ void RestackClients(void)
       fw = activeClient->window;
       for(np = nodes[activeClient->state.layer]; np; np = np->next) {
          if(np->owner == fw) {
-            stack[index] = np->parent;
+            if(np->parent != None) {
+               stack[index] = np->parent;
+            } else {
+               stack[index] = np->window;
+            }
             index += 1;
          }
       }
-      stack[index] = activeClient->parent;
+      if(activeClient->parent != None) {
+         stack[index] = activeClient->parent;
+      } else {
+         stack[index] = activeClient->window;
+      }
       index += 1;
    }
    layer = LAST_LAYER;
@@ -1109,7 +1133,11 @@ void RestackClients(void)
             if(fw != None && (np->window == fw || np->owner == fw)) {
                continue;
             }
-            stack[index] = np->parent;
+            if(np->parent != None) {
+               stack[index] = np->parent;
+            } else {
+               stack[index] = np->window;
+            }
             index += 1;
          }
       }
@@ -1166,7 +1194,6 @@ void RemoveClient(ClientNode *np)
 
    Assert(np);
    Assert(np->window != None);
-   Assert(np->parent != None);
 
    /* Remove this client from the client list */
    if(np->next) {
@@ -1181,7 +1208,9 @@ void RemoveClient(ClientNode *np)
    }
    clientCount -= 1;
    XDeleteContext(display, np->window, clientContext);
-   XDeleteContext(display, np->parent, frameContext);
+   if(np->parent != None) {
+      XDeleteContext(display, np->parent, frameContext);
+   }
 
    if(np->state.status & STAT_URGENT) {
       UnregisterCallback(SignalUrgent, np);
@@ -1293,99 +1322,85 @@ ClientNode *FindClientByParent(Window p)
 }
 
 /** Reparent a client window. */
-void ReparentClient(ClientNode *np, char notOwner)
+void ReparentClient(ClientNode *np)
 {
-
    XSetWindowAttributes attr;
+   XEvent event;
    int attrMask;
    int x, y, width, height;
    int north, south, east, west;
 
-   Assert(np);
+   if((np->state.border & (BORDER_TITLE | BORDER_OUTLINE)) == 0) {
 
-   if(notOwner) {
-      JXAddToSaveSet(display, np->window);
+      if(np->parent == None) {
+         return;
+      }
 
+      JXReparentWindow(display, np->window, rootWindow, np->x, np->y);
+      XDeleteContext(display, np->parent, frameContext);
+      JXDestroyWindow(display, np->parent);
+      np->parent = None;
+
+   } else {
+
+      if(np->parent != None) {
+         return;
+      }
+
+      attrMask = 0;
+
+      /* We can't use PointerMotionHint mask here since the exact location
+       * of the mouse on the frame is important. */
+      attrMask |= CWEventMask;
       attr.event_mask
-         = EnterWindowMask
-         | ColormapChangeMask
-         | PropertyChangeMask
-         | KeyReleaseMask
-         | StructureNotifyMask;
-      attr.do_not_propagate_mask = NoEventMask;
-      JXChangeWindowAttributes(display, np->window,
-                               CWEventMask | CWDontPropagate, &attr);
+         = ButtonPressMask
+         | ButtonReleaseMask
+         | ExposureMask
+         | PointerMotionMask
+         | SubstructureRedirectMask
+         | SubstructureNotifyMask
+         | EnterWindowMask
+         | LeaveWindowMask
+         | KeyPressMask
+         | KeyReleaseMask;
 
+      attrMask |= CWDontPropagate;
+      attr.do_not_propagate_mask = ButtonPressMask | ButtonReleaseMask;
+
+      attrMask |= CWBackPixel;
+      attr.background_pixel = colors[COLOR_TITLE_BG2];
+
+      attrMask |= CWBorderPixel;
+      attr.border_pixel = 0;
+
+      x = np->x;
+      y = np->y;
+      width = np->width;
+      height = np->height;
+      GetBorderSize(&np->state, &north, &south, &east, &west);
+      x -= west;
+      y -= north;
+      width += east + west;
+      height += north + south;
+
+      /* Create the frame window. */
+      np->parent = JXCreateWindow(display, rootWindow, x, y, width, height,
+                                  0, rootDepth, InputOutput,
+                                  rootVisual, attrMask, &attr);
+      XSaveContext(display, np->parent, frameContext, (void*)np);
+
+      JXSetWindowBorderWidth(display, np->window, 0);
+
+      /* Reparent the client window. */
+      JXReparentWindow(display, np->window, np->parent, west, north);
+
+      if(np->state.status & STAT_MAPPED) {
+         JXMapWindow(display, np->parent);
+      }
    }
-   JXGrabButton(display, AnyButton, AnyModifier, np->window, True,
-                ButtonPressMask, GrabModeSync, GrabModeAsync, None, None);
-   GrabKeys(np);
 
-   attrMask = 0;
-
-   /* We can't use PointerMotionHint mask here since the exact location
-    * of the mouse on the frame is important. */
-   attrMask |= CWEventMask;
-   attr.event_mask
-      = ButtonPressMask
-      | ButtonReleaseMask
-      | ExposureMask
-      | PointerMotionMask
-      | SubstructureRedirectMask
-      | SubstructureNotifyMask
-      | EnterWindowMask
-      | LeaveWindowMask
-      | KeyPressMask
-      | KeyReleaseMask;
-
-   attrMask |= CWDontPropagate;
-   attr.do_not_propagate_mask = ButtonPressMask | ButtonReleaseMask;
-
-   attrMask |= CWBackPixel;
-   attr.background_pixel = colors[COLOR_TITLE_BG2];
-
-   attrMask |= CWBorderPixel;
-   attr.border_pixel = 0;
-
-   attrMask |= CWColormap;
-   attr.colormap = np->cmap;
-
-   x = np->x;
-   y = np->y;
-   width = np->width;
-   height = np->height;
-   GetBorderSize(&np->state, &north, &south, &east, &west);
-   x -= west;
-   y -= north;
-   width += east + west;
-   height += north + south;
-
-   /* Create the frame window. */
-   np->parent = JXCreateWindow(display, rootWindow, x, y, width, height,
-                               0, np->visual.depth, InputOutput,
-                               np->visual.visual, attrMask, &attr);
- 
-   /* Update the window to get only the events we want. */
-   attrMask = CWDontPropagate;
-   attr.do_not_propagate_mask
-      = ButtonPressMask
-      | ButtonReleaseMask
-      | PointerMotionMask
-      | ButtonMotionMask
-      | KeyPressMask
-      | KeyReleaseMask;
-
-   /* Make sure client doesn't muck with these. */
-   attrMask |= CWBackingStore;
-   attr.backing_store = NotUseful;
-   attrMask |= CWWinGravity;
-   attr.win_gravity = NorthWestGravity;
-
-   JXChangeWindowAttributes(display, np->window, attrMask, &attr);
-   JXSetWindowBorderWidth(display, np->window, 0);
-
-   /* Reparent the client window. */
-   JXReparentWindow(display, np->window, np->parent, west, north);
+   JXSync(display, False);
+   JXCheckTypedWindowEvent(display, np->window, UnmapNotify, &event);
 
 }
 
