@@ -79,6 +79,10 @@ typedef unsigned int StatusFlags;
 #define STAT_NOPAGER    (1 << 21)   /**< Don't show in pager. */
 #define STAT_SHAPED     (1 << 22)   /**< This window is shaped. */
 #define STAT_FLASH      (1 << 23)   /**< Flashing for urgency. */
+#define STAT_DRAG       (1 << 24)   /**< Pass mouse events to JWM. */
+#define STAT_ILIST      (1 << 25)   /**< Ignore program-specified list. */
+#define STAT_IPAGER     (1 << 26)   /**< Ignore program-specified pager. */
+#define STAT_FIXED      (1 << 27)   /**< Keep on the specified desktop. */
 
 /** Maximization flags. */
 typedef unsigned char MaxFlags;
@@ -107,7 +111,6 @@ typedef struct AspectRatio {
 /** Struture to store information about a client window. */
 typedef struct ClientNode {
 
-   VisualData visual;         /**< Client visual and depth. */
    Window window;             /**< The client window. */
    Window parent;             /**< The frame window. */
 
@@ -319,6 +322,13 @@ void ShowClient(ClientNode *np);
  */
 void UpdateClientColormap(ClientNode *np);
 
+/** Reparent a client.
+ * This will create a window for a frame (or destroy it) depending on
+ * whether a client needs a frame.
+ * @param np The client.
+ */
+void ReparentClient(ClientNode *np);
+
 /** Send a configure event to a client.
  * This will send updated location and size information to a client.
  * @param np The client to get the event.
