@@ -193,7 +193,7 @@ char GrabMouseForChoose(void)
 {
    int result;
    unsigned int mask;
-   mask = ButtonPressMask | ButtonReleaseMask | PointerMotionMask;
+   mask = ButtonPressMask | ButtonReleaseMask;
    result = JXGrabPointer(display, rootWindow, False, mask,
                           GrabModeAsync, GrabModeAsync, None,
                           cursors[CURSOR_CHOOSE], CurrentTime);
@@ -208,7 +208,9 @@ char GrabMouseForChoose(void)
 /** Set the default cursor for a window. */
 void SetDefaultCursor(Window w)
 {
-   JXDefineCursor(display, w, cursors[CURSOR_DEFAULT]);
+   if(JLIKELY(w != None)) {
+      JXDefineCursor(display, w, cursors[CURSOR_DEFAULT]);
+   }
 }
 
 /** Move the mouse to the specified coordinates on a window. */
@@ -233,8 +235,6 @@ void SetMousePosition(int x, int y, Window w)
 /** Get the current mouse position. */
 void GetMousePosition(int *x, int *y, Window *w)
 {
-   Assert(x);
-   Assert(y);
    *x = mousex;
    *y = mousey;
    *w = mousew;
@@ -250,4 +250,3 @@ unsigned int GetMouseMask(void)
                   &mousex, &mousey, &winx, &winy, &mask);
    return mask;
 }
-
