@@ -27,16 +27,27 @@
 #ifndef DEBUG_H
 #define DEBUG_H
 
+#include "../config.h"
+
 #ifndef MAKE_DEPEND
 #   include <stdlib.h>
 #   ifdef HAVE_ALLOCA_H
 #      include <alloca.h>
+#   elif defined __GNUC__
+#      define alloca __builtin_alloca
+#   elif defined _AIX
+#      define alloca __alloca
+#   elif defined _MSC_VER
+#      include <malloc.h>
+#      define alloca _alloca
+#   elif defined HAVE_ALLOCA
+void *alloca(size_t);
 #   endif
 #endif /* MAKE_DEPEND */
 
 void Debug(const char *str, ...);
 
-#ifdef HAVE_ALLOCA_H
+#ifdef HAVE_ALLOCA
 
 #   define AllocateStack( x ) alloca( x )
 #   define ReleaseStack( x ) ((void)0)
