@@ -224,7 +224,7 @@ Menu *CreateDesktopMenu(unsigned int mask, void *context)
 
       item->action.type = MA_DESKTOP;
       item->action.context = context;
-      item->action.data.i = x;
+      item->action.value = x;
 
       item->name = Allocate(len + 3);
       item->name[0] = (mask & (1 << x)) ? '[' : ' ';
@@ -257,7 +257,7 @@ Menu *CreateSendtoMenu(MenuActionType mask, void *context)
 
       item->action.type = MA_SENDTO | mask;
       item->action.context = context;
-      item->action.data.i = x;
+      item->action.value = x;
 
       item->name = Allocate(len + 3);
       item->name[0] = (x == currentDesktop) ? '[' : ' ';
@@ -289,6 +289,10 @@ void ShowDesktop(void)
                   RestoreClient(np, 0);
                }
             } else {
+               if(np->state.status & STAT_ACTIVE) {
+                  JXSetInputFocus(display, rootWindow, RevertToParent,
+                                  CurrentTime);
+               }
                if(np->state.status & (STAT_MAPPED | STAT_SHADED)) {
                   MinimizeClient(np, 0);
                   np->state.status |= STAT_SDESKTOP;
