@@ -90,7 +90,6 @@ static void HandleShapeEvent(const XShapeEvent *event);
 /** Wait for an event and process it. */
 char WaitForEvent(XEvent *event)
 {
-
    struct timeval timeout;
    CallbackNode *cp;
    fd_set fds;
@@ -120,9 +119,11 @@ char WaitForEvent(XEvent *event)
       }
       if(task_update_pending) {
          UpdateTaskBar();
+         task_update_pending = 0;
       }
       if(pager_update_pending) {
          UpdatePager();
+         pager_update_pending = 0;
       }
 
       while(JXPending(display) == 0) {
@@ -239,7 +240,7 @@ char WaitForEvent(XEvent *event)
          handled = ProcessPopupEvent(event);
       }
 
-   } while(handled && !shouldExit);
+   } while(handled && JLIKELY(!shouldExit));
 
    return !handled;
 
@@ -248,7 +249,6 @@ char WaitForEvent(XEvent *event)
 /** Wake up components that need to run at certain times. */
 void Signal(void)
 {
-
    static TimeType last = ZERO_TIME;
 
    CallbackNode *cp;   
@@ -269,7 +269,6 @@ void Signal(void)
          (cp->callback)(&now, x, y, w, cp->data);
       }
    }
-
 }
 
 /** Process an event. */
@@ -305,6 +304,17 @@ void ProcessEvent(XEvent *event)
    }
 }
 
+/** Discard button events for the specified windows. */
+void DiscardButtonEvents()
+{
+   XEvent event;
+   JXSync(display, False);
+   while(JXCheckMaskEvent(display, ButtonPressMask | ButtonReleaseMask,
+			  &event)) {
+      UpdateTime(&event);
+   }
+}
+
 /** Discard motion events for the specified window. */
 void DiscardMotionEvents(XEvent *event, Window w)
 {
@@ -367,7 +377,7 @@ void HandleButtonEvent(const XButtonEvent *event)
       }
       DispatchBorderButtonEvent(event, np);
    } else if(event->window == rootWindow && event->type == ButtonPress) {
-      if(!ShowRootMenu(event->button, event->x, event->y)) {
+      if(!ShowRootMenu(event->button, event->x, event->y, 0)) {
          if(event->button == Button4) {
             LeftDesktop();
          } else if(event->button == Button5) {
@@ -378,18 +388,20 @@ void HandleButtonEvent(const XButtonEvent *event)
       const unsigned int mask = event->state & ~lockMask;
       np = FindClientByWindow(event->window);
       if(np) {
+         const char move_resize = (mask == Mod1Mask)
+            || (np->state.status & STAT_DRAG);
          switch(event->button) {
          case Button1:
          case Button2:
             FocusClient(np);
             RaiseClient(np);
-            if(mask == Mod1Mask) {
+            if(move_resize) {
                GetBorderSize(&np->state, &north, &south, &east, &west);
                MoveClient(np, event->x + west, event->y + north);
             }
             break;
          case Button3:
-            if(mask == Mod1Mask) {
+            if(move_resize) {
                GetBorderSize(&np->state, &north, &south, &east, &west);
                ResizeClient(np, BA_RESIZE | BA_RESIZE_E | BA_RESIZE_S,
                             event->x + west, event->y + north);
@@ -425,6 +437,8 @@ void HandleKeyPress(const XKeyEvent *event)
 {
    ClientNode *np;
    KeyType key;
+
+   SetMousePosition(event->x_root, event->y_root, event->window);
    key = GetKey(event);
    np = GetActiveClient();
    switch(key & 0xFF) {
@@ -507,11 +521,14 @@ void HandleKeyPress(const XKeyEvent *event)
       }
       break;
    case KEY_MAX:
+      ToggleMaximized(np, MAX_HORIZ | MAX_VERT);
+      break;
+   case KEY_RESTORE:
       if(np) {
          if(np->state.maxFlags) {
             MaximizeClient(np, MAX_NONE);
          } else {
-            MaximizeClient(np, MAX_HORIZ | MAX_VERT);
+            MinimizeClient(np, 1);
          }
       }
       break;
@@ -538,14 +555,15 @@ void HandleKeyPress(const XKeyEvent *event)
       break;
    case KEY_WIN:
       if(np) {
-         ShowWindowMenu(np, np->x, np->y);
+         RaiseClient(np);
+         ShowWindowMenu(np, np->x, np->y, 1);
       }
       break;
    case KEY_RESTART:
       Restart();
       break;
    case KEY_EXIT:
-      Exit();
+      Exit(1);
       break;
    case KEY_FULLSCREEN:
       if(np) {
@@ -558,22 +576,30 @@ void HandleKeyPress(const XKeyEvent *event)
       break;
    case KEY_SENDR:
       if(np) {
-         SetClientDesktop(np, GetRightDesktop(np->state.desktop));
+         const unsigned desktop = GetRightDesktop(np->state.desktop);
+         SetClientDesktop(np, desktop);
+         ChangeDesktop(desktop);
       }
       break;
    case KEY_SENDL:
       if(np) {
-         SetClientDesktop(np, GetLeftDesktop(np->state.desktop));
+         const unsigned desktop = GetLeftDesktop(np->state.desktop);
+         SetClientDesktop(np, desktop);
+         ChangeDesktop(desktop);
       }
       break;
    case KEY_SENDU:
       if(np) {
-         SetClientDesktop(np, GetAboveDesktop(np->state.desktop));
+         const unsigned desktop = GetAboveDesktop(np->state.desktop);
+         SetClientDesktop(np, desktop);
+         ChangeDesktop(desktop);
       }
       break;
    case KEY_SENDD:
       if(np) {
-         SetClientDesktop(np, GetBelowDesktop(np->state.desktop));
+         const unsigned desktop = GetBelowDesktop(np->state.desktop);
+         SetClientDesktop(np, desktop);
+         ChangeDesktop(desktop);
       }
       break;
    default:
@@ -586,11 +612,9 @@ void HandleKeyPress(const XKeyEvent *event)
 void HandleKeyRelease(const XKeyEvent *event)
 {
    KeyType key;
-   key = GetKey(event);
-   if(((key & 0xFF) != KEY_NEXTSTACK) &&
-      ((key & 0xFF) != KEY_NEXT) &&
-      ((key & 0xFF) != KEY_PREV) &&
-      ((key & 0xFF) != KEY_PREVSTACK)) {
+   key = GetKey(event) & 0xFF;
+   if(   key != KEY_NEXTSTACK && key != KEY_NEXT
+      && key != KEY_PREV      && key != KEY_PREVSTACK) {
       StopWindowWalk();
    }
 }
@@ -701,7 +725,11 @@ void HandleConfigureRequest(const XConfigureRequestEvent *event)
       } else {
          int north, south, east, west;
          GetBorderSize(&np->state, &north, &south, &east, &west);
-         JXMoveWindow(display, np->parent, np->x - west, np->y - north);
+         if(np->parent != None) {
+            JXMoveWindow(display, np->parent, np->x - west, np->y - north);
+         } else {
+            JXMoveWindow(display, np->window, np->x, np->y);
+         }
       }
 
       SendConfigureEvent(np);
@@ -816,6 +844,7 @@ char HandlePropertyNotify(const XPropertyEvent *event)
          if(np->state.status & STAT_URGENT) {
             RegisterCallback(URGENCY_DELAY, SignalUrgent, np);
          }
+         WriteState(np);
          break;
       case XA_WM_TRANSIENT_FOR:
          JXGetTransientForHint(display, np->window, &np->owner);
@@ -843,9 +872,12 @@ char HandlePropertyNotify(const XPropertyEvent *event)
             UpdateState(np);
             WriteState(np);
             ResetBorder(np);
+            changed = 1;
          } else if(event->atom == atoms[ATOM_NET_WM_WINDOW_OPACITY]) {
-            ReadWMOpacity(np);
-            SetOpacity(np, np->state.opacity, 1);
+            ReadWMOpacity(np->window, &np->state.opacity);
+            if(np->parent != None) {
+               SetOpacity(np, np->state.opacity, 1);
+            }
          }
          break;
       }
@@ -954,7 +986,7 @@ void HandleClientMessage(const XClientMessageEvent *event)
       if(event->message_type == atoms[ATOM_JWM_RESTART]) {
          Restart();
       } else if(event->message_type == atoms[ATOM_JWM_EXIT]) {
-         Exit();
+         Exit(0);
       } else if(event->message_type == atoms[ATOM_JWM_RELOAD]) {
          ReloadMenu();
       } else if(event->message_type == atoms[ATOM_NET_CURRENT_DESKTOP]) {
@@ -1078,9 +1110,14 @@ void HandleNetMoveResize(const XClientMessageEvent *event, ClientNode *np)
 void HandleNetWMMoveResize(const XClientMessageEvent *event, ClientNode *np)
 {
 
-   const long x = event->data.l[0] - np->x;
-   const long y = event->data.l[1] - np->y;
+   long x = event->data.l[0] - np->x;
+   long y = event->data.l[1] - np->y;
    const long direction = event->data.l[2];
+   int deltax, deltay;
+
+   GetGravityDelta(np, np->gravity, &deltax, &deltay);
+   x -= deltax;
+   y -= deltay;
 
    switch(direction) {
    case 0:  /* top-left */
@@ -1212,11 +1249,11 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
       if(actionMinimize) {
          RestoreClient(np, 0);
       }
-      if(actionNolist) {
+      if(actionNolist && !(np->state.status & STAT_ILIST)) {
          np->state.status &= ~STAT_NOLIST;
          RequireTaskUpdate();
       }
-      if(actionNopager) {
+      if(actionNopager && !(np->state.status & STAT_IPAGER)) {
          np->state.status &= ~STAT_NOPAGER;
          RequirePagerUpdate();
       }
@@ -1243,11 +1280,11 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
       if(actionMinimize) {
          MinimizeClient(np, 1);
       }
-      if(actionNolist) {
+      if(actionNolist && !(np->state.status & STAT_ILIST)) {
          np->state.status |= STAT_NOLIST;
          RequireTaskUpdate();
       }
-      if(actionNopager) {
+      if(actionNopager && !(np->state.status & STAT_IPAGER)) {
          np->state.status |= STAT_NOPAGER;
          RequirePagerUpdate();
       }
@@ -1299,11 +1336,11 @@ void HandleNetWMState(const XClientMessageEvent *event, ClientNode *np)
       }
       /* Note that we don't handle toggling of hidden per EWMH
        * recommendations. */
-      if(actionNolist) {
+      if(actionNolist && !(np->state.status & STAT_ILIST)) {
          np->state.status ^= STAT_NOLIST;
          RequireTaskUpdate();
       }
-      if(actionNopager) {
+      if(actionNopager && !(np->state.status & STAT_IPAGER)) {
          np->state.status ^= STAT_NOPAGER;
          RequirePagerUpdate();
       }
@@ -1405,7 +1442,9 @@ void HandleMapRequest(const XMapEvent *event)
          UpdateState(np);
          np->state.status |= STAT_MAPPED;
          XMapWindow(display, np->window);
-         XMapWindow(display, np->parent);
+         if(np->parent != None) {
+            XMapWindow(display, np->parent);
+         }
          if(!(np->state.status & STAT_STICKY)) {
             np->state.desktop = currentDesktop;
          }
@@ -1451,13 +1490,15 @@ void HandleUnmapNotify(const XUnmapEvent *event)
          UpdateTime(&e);
          RemoveClient(np);
       } else if((np->state.status & STAT_MAPPED) || event->send_event) {
-         np->state.status &= ~STAT_MAPPED;
-         JXUngrabButton(display, AnyButton, AnyModifier, np->window);
-         GravitateClient(np, 1);
-         JXReparentWindow(display, np->window, rootWindow, np->x, np->y);
-         WriteState(np);
-         JXRemoveFromSaveSet(display, np->window);
-         RemoveClient(np);
+         if(!(np->state.status & STAT_HIDDEN)) {
+            np->state.status &= ~STAT_MAPPED;
+            JXUngrabButton(display, AnyButton, AnyModifier, np->window);
+            GravitateClient(np, 1);
+            JXReparentWindow(display, np->window, rootWindow, np->x, np->y);
+            WriteState(np);
+            JXRemoveFromSaveSet(display, np->window);
+            RemoveClient(np);
+         }
       }
       UngrabServer();
 
@@ -1512,8 +1553,9 @@ void DispatchBorderButtonEvent(const XButtonEvent *event,
          if(event->button == Button1) {
             ResizeClient(np, action, event->x, event->y);
          } else if(event->button == Button3) {
-            ShowWindowMenu(np, np->x + event->x - bsize,
-                           np->y + event->y - settings.titleHeight - bsize);
+            const int x = np->x + event->x - bsize;
+            const int y = np->y + event->y - settings.titleHeight - bsize;
+            ShowWindowMenu(np, x, y, 0);
          }
       }
       break;
@@ -1539,8 +1581,9 @@ void DispatchBorderButtonEvent(const XButtonEvent *event,
             }
          }
       } else if(event->button == Button3) {
-         ShowWindowMenu(np, np->x + event->x - bsize,
-                        np->y + event->y - settings.titleHeight - bsize);
+         const int x = np->x + event->x - bsize;
+         const int y = np->y + event->y - settings.titleHeight - bsize;
+         ShowWindowMenu(np, x, y, 0);
       } else if(event->button == Button4) {
          ShadeClient(np);
       } else if(event->button == Button5) {
@@ -1553,8 +1596,9 @@ void DispatchBorderButtonEvent(const XButtonEvent *event,
       } else if(event->button == Button5) {
          UnshadeClient(np);
       } else if(event->type == ButtonPress) {
-         ShowWindowMenu(np, np->x + event->x - bsize,
-                        np->y + event->y - settings.titleHeight - bsize);
+         const int x = np->x + event->x - bsize;
+         const int y = np->y + event->y - settings.titleHeight - bsize;
+         ShowWindowMenu(np, x, y, 0);
       }
       break;
    case BA_CLOSE: /* Close button */
@@ -1602,7 +1646,8 @@ void DispatchBorderButtonEvent(const XButtonEvent *event,
 /** Update window state information. */
 void UpdateState(ClientNode *np)
 {
-   char alreadyMapped;
+   const char alreadyMapped = (np->state.status & STAT_MAPPED) ? 1 : 0;
+   const char active = (np->state.status & STAT_ACTIVE) ? 1 : 0;
 
    /* Remove from the layer list. */
    if(np->prev != NULL) {
@@ -1619,7 +1664,6 @@ void UpdateState(ClientNode *np)
    }
 
    /* Read the state (and new layer). */
-   alreadyMapped = (np->state.status & STAT_MAPPED) ? 1 : 0;
    if(np->state.status & STAT_URGENT) {
       UnregisterCallback(SignalUrgent, np);
    }
@@ -1643,6 +1687,10 @@ void UpdateState(ClientNode *np)
    }
    nodes[np->state.layer] = np;
 
+   if(active) {
+      FocusClient(np);
+   }
+
 }
 
 /** Update the last event time. */
