@@ -12,19 +12,15 @@
 
 #include "color.h"
 
-struct VisualData;
-
 /** Enumeration of different components that use fonts. */
 typedef unsigned char FontType;
 #define FONT_BORDER     0
 #define FONT_MENU       1
-#define FONT_TASK       2
-#define FONT_POPUP      3
-#define FONT_CLOCK      4
-#define FONT_TRAY       5
-#define FONT_TRAYBUTTON 6
-#define FONT_PAGER      7
-#define FONT_COUNT      8
+#define FONT_POPUP      2
+#define FONT_TRAY       3
+#define FONT_PAGER      4
+#define FONT_CLOCK      5
+#define FONT_COUNT      6
 
 void InitializeFonts(void);
 void StartupFonts(void);
@@ -38,7 +34,6 @@ void DestroyFonts(void);
 void SetFont(FontType type, const char *value);
 
 /** Render a string.
- * @param visual The visual and depth.
  * @param d The drawable on which to render the string.
  * @param font The font to use.
  * @param color The text color to use.
@@ -47,8 +42,7 @@ void SetFont(FontType type, const char *value);
  * @param width The maximum width allowed.
  * @param str The string to render.
  */
-void RenderString(const struct VisualData *visual, Drawable d,
-                  FontType font, ColorType color,
+void RenderString(Drawable d, FontType font, ColorType color,
                   int x, int y, int width, const char *str);
 
 /** Get the width of a string.
