@@ -215,8 +215,9 @@ void ApplyGroups(ClientNode *np)
 /** Apply a group to a client. */
 void ApplyGroup(const GroupType *gp, ClientNode *np)
 {
-
    OptionListType *lp;
+   char noPager = 0;
+   char noList = 0;
 
    Assert(gp);
    Assert(np);
@@ -227,9 +228,23 @@ void ApplyGroup(const GroupType *gp, ClientNode *np)
          break;
       case OPTION_NOLIST:
          np->state.status |= STAT_NOLIST;
+         noList = 1;
+         break;
+      case OPTION_ILIST:
+         np->state.status |= STAT_ILIST;
+         if(!noList) {
+            np->state.status &= ~STAT_NOLIST;
+         }
          break;
       case OPTION_NOPAGER:
          np->state.status |= STAT_NOPAGER;
+         noPager = 1;
+         break;
+      case OPTION_IPAGER:
+         np->state.status |= STAT_IPAGER;
+         if(!noPager) {
+            np->state.status &= ~STAT_NOPAGER;
+         }
          break;
       case OPTION_BORDER:
          np->state.border |= BORDER_OUTLINE;
@@ -322,6 +337,12 @@ void ApplyGroup(const GroupType *gp, ClientNode *np)
       case OPTION_NOFULLSCREEN:
          np->state.border &= ~BORDER_FULLSCREEN;
          break;
+      case OPTION_DRAG:
+         np->state.status |= STAT_DRAG;
+         break;
+      case OPTION_FIXED:
+         np->state.status |= STAT_FIXED;
+         break;
       default:
          Debug("invalid option: %d", lp->option);
          break;
