@@ -66,22 +66,22 @@ typedef struct {
 
 Atom atoms[ATOM_COUNT];
 
-const char jwmRestart[]    = "_JWM_RESTART";
-const char jwmExit[]       = "_JWM_EXIT";
-const char jwmReload[]     = "_JWM_RELOAD";
-const char opacityAtom[]   = "_NET_WM_WINDOW_OPACITY";
+const char jwmRestart[]       = "_JWM_RESTART";
+const char jwmExit[]          = "_JWM_EXIT";
+const char jwmReload[]        = "_JWM_RELOAD";
+const char managerProperty[]  = "MANAGER";
 
 static const AtomNode atomList[] = {
 
    { &atoms[ATOM_COMPOUND_TEXT],             "COMPOUND_TEXT"               },
    { &atoms[ATOM_UTF8_STRING],               "UTF8_STRING"                 },
    { &atoms[ATOM_XROOTPMAP_ID],              "_XROOTPMAP_ID"               },
+   { &atoms[ATOM_MANAGER],                   &managerProperty[0]           },
 
    { &atoms[ATOM_WM_STATE],                  "WM_STATE"                    },
    { &atoms[ATOM_WM_PROTOCOLS],              "WM_PROTOCOLS"                },
    { &atoms[ATOM_WM_DELETE_WINDOW],          "WM_DELETE_WINDOW"            },
    { &atoms[ATOM_WM_TAKE_FOCUS],             "WM_TAKE_FOCUS"               },
-   { &atoms[ATOM_WM_LOCALE_NAME],            "WM_LOCALE_NAME"              },
    { &atoms[ATOM_WM_CHANGE_STATE],           "WM_CHANGE_STATE"             },
    { &atoms[ATOM_WM_COLORMAP_WINDOWS],       "WM_COLORMAP_WINDOWS"         },
 
@@ -145,16 +145,18 @@ static const AtomNode atomList[] = {
    { &atoms[ATOM_NET_WM_WINDOW_TYPE_NORMAL], "_NET_WM_WINDOW_TYPE_NORMAL"  },
    { &atoms[ATOM_NET_WM_WINDOW_TYPE_MENU],   "_NET_WM_WINDOW_TYPE_MENU"    },
    { &atoms[ATOM_NET_WM_WINDOW_TYPE_NOTIFICATION],
-      "_NET_WM_WINDOW_TYPE_NOTIFICATION"},
+      "_NET_WM_WINDOW_TYPE_NOTIFICATION" },
    { &atoms[ATOM_NET_WM_WINDOW_TYPE_TOOLBAR], "_NET_WM_WINDOW_TYPE_TOOLBAR"},
    { &atoms[ATOM_NET_WM_WINDOW_TYPE_UTILITY], "_NET_WM_WINDOW_TYPE_UTILITY"},
    { &atoms[ATOM_NET_CLIENT_LIST],           "_NET_CLIENT_LIST"            },
    { &atoms[ATOM_NET_CLIENT_LIST_STACKING],  "_NET_CLIENT_LIST_STACKING"   },
    { &atoms[ATOM_NET_WM_STRUT_PARTIAL],      "_NET_WM_STRUT_PARTIAL"       },
    { &atoms[ATOM_NET_WM_STRUT],              "_NET_WM_STRUT"               },
-   { &atoms[ATOM_NET_WM_WINDOW_OPACITY],     &opacityAtom[0]               },
+   { &atoms[ATOM_NET_WM_WINDOW_OPACITY],     "_NET_WM_WINDOW_OPACITY"      },
    { &atoms[ATOM_NET_WM_MOVERESIZE],         "_NET_WM_MOVERESIZE"          },
    { &atoms[ATOM_NET_SYSTEM_TRAY_OPCODE],    "_NET_SYSTEM_TRAY_OPCODE"     },
+   { &atoms[ATOM_NET_SYSTEM_TRAY_ORIENTATION],
+      "_NET_SYSTEM_TRAY_ORIENTATION" },
 
    { &atoms[ATOM_MOTIF_WM_HINTS],            "_MOTIF_WM_HINTS"             },
 
@@ -224,8 +226,10 @@ void StartupHints(void)
    /* _NET_DESKTOP_NAMES */
    count = 0;
    for(x = 0; x < settings.desktopCount; x++) {
-      strcpy(data + count, GetDesktopName(x));
-      count += strlen(GetDesktopName(x)) + 1;
+      const char *name = GetDesktopName(x);
+      const unsigned len = strlen(name);
+      memcpy(&data[count], name, len + 1);
+      count += len + 1;
    }
    JXChangeProperty(display, rootWindow, atoms[ATOM_NET_DESKTOP_NAMES],
                     atoms[ATOM_UTF8_STRING], 8, PropModeReplace,
@@ -302,13 +306,9 @@ void ReadClientInfo(ClientNode *np, char alreadyMapped)
    np->state = ReadWindowState(np->window, alreadyMapped);
    if(np->minWidth == np->maxWidth && np->minHeight == np->maxHeight) {
       np->state.border &= ~BORDER_RESIZE;
+      np->state.border &= ~BORDER_MAX;
    }
 
-   /* Read WM protocols. */
-   ReadWMProtocols(np->window, &np->state);
-
-   ReadWMOpacity(np);
-
    /* Make sure this client is on at least as high of a layer
     * as its owner. */
    if(np->owner != None) {
@@ -323,11 +323,8 @@ void ReadClientInfo(ClientNode *np, char alreadyMapped)
 /** Write the window state hint for a client. */
 void WriteState(ClientNode *np)
 {
-
    unsigned long data[2];
 
-   Assert(np);
-
    if(np->state.status & STAT_MAPPED) {
       data[0] = NormalState;
    } else if(np->state.status & STAT_MINIMIZED) {
@@ -349,22 +346,22 @@ void WriteState(ClientNode *np)
 
    WriteNetState(np);
    WriteNetAllowed(np);
-
 }
 
 /** Set the opacity of a client. */
 void SetOpacity(ClientNode *np, unsigned int opacity, char force)
 {
-
+   Window w;
    if(np->state.opacity == opacity && !force) {
       return;
    }
 
+   w = np->parent != None ? np->parent : np->window;
    np->state.opacity = opacity;
    if(opacity == 0xFFFFFFFF) {
-      JXDeleteProperty(display, np->parent, atoms[ATOM_NET_WM_WINDOW_OPACITY]);
+      JXDeleteProperty(display, w, atoms[ATOM_NET_WM_WINDOW_OPACITY]);
    } else {
-      SetCardinalAtom(np->parent, ATOM_NET_WM_WINDOW_OPACITY, opacity);
+      SetCardinalAtom(w, ATOM_NET_WM_WINDOW_OPACITY, opacity);
    }
 }
 
@@ -565,9 +562,11 @@ ClientState ReadWindowState(Window win, char alreadyMapped)
    result.desktop = currentDesktop;
    result.opacity = UINT_MAX;
 
+   ReadWMProtocols(win, &result);
    ReadWMHints(win, &result, alreadyMapped);
    ReadWMState(win, &result);
    ReadMotifHints(win, &result);
+   ReadWMOpacity(win, &result.opacity);
 
    /* _NET_WM_DESKTOP */
    if(GetCardinalAtom(win, ATOM_NET_WM_DESKTOP, &card)) {
@@ -653,7 +652,7 @@ ClientState ReadWindowState(Window win, char alreadyMapped)
             break;
          } else if(  state[x] == atoms[ATOM_NET_WM_WINDOW_TYPE_DIALOG]) {
             result.border &= ~BORDER_MIN;
-            result.status |= STAT_NOLIST;
+            result.border &= ~BORDER_MAX;
             break;
          } else if(  state[x] == atoms[ATOM_NET_WM_WINDOW_TYPE_MENU]) {
             result.border       &= ~BORDER_MAX;
@@ -728,9 +727,9 @@ void ReadWMName(ClientNode *np)
    if(status != Success || realFormat == 0) {
       np->name = NULL;
    } else {
-      const size_t size = strlen((char*)name) + 1;
-      np->name = Allocate(size);
-      memcpy(np->name, name, size);
+      np->name = Allocate(count + 1);
+      memcpy(np->name, name, count);
+      np->name[count] = 0;
       JXFree(name);
       np->name = ConvertFromUTF8(np->name);
    }
@@ -742,14 +741,14 @@ void ReadWMName(ClientNode *np)
                                    atoms[ATOM_COMPOUND_TEXT],
                                    &realType, &realFormat, &count,
                                    &extra, &name);
-      if(status == Success && realFormat == 8) {
+      if(status == Success && realFormat != 0) {
          char **tlist;
          XTextProperty tprop;
          int tcount;
          tprop.value = name;
          tprop.encoding = atoms[ATOM_COMPOUND_TEXT];
          tprop.format = realFormat;
-         tprop.nitems = strlen((char*)name);
+         tprop.nitems = count;
          if(XmbTextPropertyToTextList(display, &tprop, &tlist, &tcount)
             == Success && tcount > 0) {
             const size_t len = strlen(tlist[0]) + 1;
@@ -942,22 +941,25 @@ void ReadWMState(Window win, ClientState *state)
    unsigned long extra;
    Atom realType;
    int realFormat;
-   unsigned int *temp;
+   unsigned long *temp;
 
+   count = 0;
    status = JXGetWindowProperty(display, win, atoms[ATOM_WM_STATE], 0, 2,
                                 False, atoms[ATOM_WM_STATE],
                                 &realType, &realFormat,
                                 &count, &extra, (unsigned char**)&temp);
-   if(JLIKELY(status == Success && realFormat != 0 && count == 2)) {
-      switch(temp[0]) {
-      case IconicState:
-         state->status |= STAT_MINIMIZED;
-         break;
-      case WithdrawnState:
-         state->status &= ~STAT_MAPPED;
-         break;
-      default:
-         break;
+   if(JLIKELY(status == Success && realFormat != 0)) {
+      if(JLIKELY(count == 2)) {
+         switch(temp[0]) {
+         case IconicState:
+            state->status |= STAT_MINIMIZED;
+            break;
+         case WithdrawnState:
+            state->status &= ~STAT_MAPPED;
+            break;
+         default:
+            break;
+         }
       }
       JXFree(temp);
    }
@@ -981,9 +983,6 @@ void ReadWMHints(Window win, ClientState *state, char alreadyMapped)
          case IconicState:
             state->status |= STAT_MINIMIZED;
             break;
-         case WithdrawnState:
-            state->status &= ~STAT_MAPPED;
-            break;
          default:
             break;
          }
@@ -1002,14 +1001,14 @@ void ReadWMHints(Window win, ClientState *state, char alreadyMapped)
 }
 
 /** Read _NET_WM_WINDOW_OPACITY. */
-void ReadWMOpacity(ClientNode *np) {
+void ReadWMOpacity(Window win, unsigned *opacity)
+{
    unsigned long card;
-   if(GetCardinalAtom(np->window, ATOM_NET_WM_WINDOW_OPACITY, &card)) {
-      np->state.status |= STAT_OPACITY;
+   if(GetCardinalAtom(win, ATOM_NET_WM_WINDOW_OPACITY, &card)) {
+      *opacity = card;
    } else {
-      card = UINT_MAX;
+      *opacity = UINT_MAX;
    }
-   np->state.opacity = (unsigned int)card;
 }
 
 /** Read _MOTIF_WM_HINTS */
@@ -1021,19 +1020,20 @@ void ReadMotifHints(Window win, ClientState *state)
    unsigned long itemCount, bytesLeft;
    unsigned char *data;
    int format;
+   int status;
 
    Assert(win != None);
    Assert(state);
 
-   if(JXGetWindowProperty(display, win, atoms[ATOM_MOTIF_WM_HINTS], 0L, 20L,
-                          False, atoms[ATOM_MOTIF_WM_HINTS], &type, &format,
-                          &itemCount, &bytesLeft, &data) != Success
-         || format == 0) {
+   status = JXGetWindowProperty(display, win, atoms[ATOM_MOTIF_WM_HINTS],
+                                0L, 20L, False, atoms[ATOM_MOTIF_WM_HINTS],
+                                &type, &format, &itemCount, &bytesLeft, &data);
+   if(status != Success || type == 0) {
       return;
    }
 
    mhints = (PropMwmHints*)data;
-   if(mhints) {
+   if(JLIKELY(mhints)) {
 
       if((mhints->flags & MWM_HINTS_FUNCTIONS)
          && !(mhints->functions & MWM_FUNC_ALL)) {
@@ -1085,12 +1085,13 @@ char GetCardinalAtom(Window window, AtomType atom, unsigned long *value)
    Assert(window != None);
    Assert(value);
 
+   count = 0;
    status = JXGetWindowProperty(display, window, atoms[atom], 0, 1, False,
                                 XA_CARDINAL, &realType, &realFormat,
                                 &count, &extra, &data);
    ret = 0;
    if(status == Success && realFormat != 0 && data) {
-      if(count == 1) {
+      if(JLIKELY(count == 1)) {
          *value = *(unsigned long*)data;
          ret = 1;
       }
@@ -1124,12 +1125,13 @@ char GetWindowAtom(Window window, AtomType atom, Window *value)
    Assert(window != None);
    Assert(value);
 
+   count = 0;
    status = JXGetWindowProperty(display, window, atoms[atom], 0, 1, False,
                                 XA_WINDOW, &realType, &realFormat,
                                 &count, &extra, &data);
    ret = 0;
    if(status == Success && realFormat != 0 && data) {
-      if(count == 1) {
+      if(JLIKELY(count == 1)) {
          *value = *(Window*)data;
          ret = 1;
       }
@@ -1156,3 +1158,10 @@ void SetPixmapAtom(Window window, AtomType atom, Pixmap value)
                     PropModeReplace, (unsigned char*)&value, 1);
 }
 
+/** Set an atom atom. */
+void SetAtomAtom(Window window, AtomType atom, AtomType value)
+{
+   Assert(window != None);
+   JXChangeProperty(display, window, atoms[atom], XA_ATOM, 32,
+                    PropModeReplace, (unsigned char*)&atoms[value], 1);
+}
