@@ -19,13 +19,13 @@ typedef enum {
    ATOM_COMPOUND_TEXT,
    ATOM_UTF8_STRING,
    ATOM_XROOTPMAP_ID,
+   ATOM_MANAGER,
 
    /* Standard atoms */
    ATOM_WM_STATE,
    ATOM_WM_PROTOCOLS,
    ATOM_WM_DELETE_WINDOW,
    ATOM_WM_TAKE_FOCUS,
-   ATOM_WM_LOCALE_NAME,
    ATOM_WM_CHANGE_STATE,
    ATOM_WM_COLORMAP_WINDOWS,
 
@@ -104,6 +104,7 @@ typedef enum {
    ATOM_NET_WM_MOVERESIZE,
 
    ATOM_NET_SYSTEM_TRAY_OPCODE,
+   ATOM_NET_SYSTEM_TRAY_ORIENTATION,
 
    /* MWM atoms */
    ATOM_MOTIF_WM_HINTS,
@@ -123,7 +124,7 @@ typedef enum {
 extern const char jwmRestart[];
 extern const char jwmExit[];
 extern const char jwmReload[];
-extern const char opacityAtom[];
+extern const char managerProperty[];
 
 #define FIRST_NET_ATOM ATOM_NET_SUPPORTED
 #define LAST_NET_ATOM  ATOM_NET_WM_STRUT
@@ -218,9 +219,10 @@ ClientState ReadWindowState(Window win, char alreadyMapped);
 void ReadWMHints(Window win, ClientState *state, char alreadyMapped);
 
 /** Read opacity.
- * @param np The client.
+ * @param win The window.
+ * @param opacity The opacity to update.
  */
-void ReadWMOpacity(struct ClientNode *np);
+void ReadWMOpacity(Window win, unsigned *opacity);
 
 /** Set the state of a client window.
  * @param np The client.
@@ -277,5 +279,12 @@ void SetWindowAtom(Window window, AtomType atom, unsigned long value);
  */
 void SetPixmapAtom(Window window, AtomType atom, Pixmap value);
 
+/** Set an atom atom.
+ * @param window The window.
+ * @param atom The atom to set.
+ * @param value The value.
+ */
+void SetAtomAtom(Window window, AtomType atom, AtomType value);
+
 #endif /* HINT_H */
 
