@@ -17,13 +17,12 @@
 #include "hint.h"
 #include "color.h"
 #include "settings.h"
+#include "border.h"
 
 IconNode emptyIcon;
 
 #ifdef USE_ICONS
 
-#include "x.xpm"
-
 /* Must be a power of two. */
 #define HASH_SIZE 128
 
@@ -33,6 +32,35 @@ typedef struct IconPathNode {
    struct IconPathNode *next;
 } IconPathNode;
 
+/* These extensions are appended to icon names during search. */
+const char *ICON_EXTENSIONS[] = {
+   "",
+#ifdef USE_PNG
+   ".png",
+   ".PNG",
+#endif
+#if defined(USE_CAIRO) && defined(USE_RSVG)
+   ".svg",
+   ".SVG",
+#endif
+#ifdef USE_XPM
+   ".xpm",
+   ".XPM",
+#endif
+#ifdef USE_JPEG
+   ".jpg",
+   ".JPG",
+   ".jpeg",
+   ".JPEG",
+#endif
+#ifdef USE_XBM
+   ".xbm",
+   ".XBM",
+#endif
+};
+static const unsigned EXTENSION_COUNT = ARRAY_LENGTH(ICON_EXTENSIONS);
+static const unsigned MAX_EXTENSION_LENGTH = 5;
+
 static IconNode **iconHash;
 static IconPathNode *iconPaths;
 static IconPathNode *iconPathsTail;
@@ -40,27 +68,18 @@ static GC iconGC;
 static char iconSizeSet = 0;
 
 static void DoDestroyIcon(int index, IconNode *icon);
-static void ReadNetWMIcon(ClientNode *np);
-static void ReadWMHintIcon(ClientNode *np);
-static IconNode *CreateIcon(void);
-static IconNode *GetDefaultIcon(void);
-static IconNode *CreateIconFromData(const char *name, char **data);
+static IconNode *ReadNetWMIcon(Window win);
+static IconNode *ReadWMHintIcon(Window win);
+static IconNode *CreateIcon(const ImageNode *image);
 static IconNode *CreateIconFromDrawable(Drawable d, Pixmap mask);
-static IconNode *CreateIconFromFile(const char *fileName,
-                                    char save, char preserveAspect);
 static IconNode *CreateIconFromBinary(const unsigned long *data,
                                       unsigned int length);
 static IconNode *LoadNamedIconHelper(const char *name, const char *path,
                                      char save, char preserveAspect);
 
-#if defined(USE_ICONS)
-static IconNode *LoadSuffixedIcon(const char *path, const char *name,
-                                  const char *suffix);
-#endif
-
 static ImageNode *GetBestImage(IconNode *icon, int rwidth, int rheight);
-static ScaledIconNode *GetScaledIcon(IconNode *icon, ImageNode *iconImage,
-                                     long fg, int rwidth, int rheight);
+static ScaledIconNode *GetScaledIcon(IconNode *icon, long fg,
+                                     int rwidth, int rheight);
 
 static void InsertIcon(IconNode *icon);
 static IconNode *FindIcon(const char *name);
@@ -92,8 +111,8 @@ void StartupIcons(void)
    gcValues.graphics_exposures = False;
    iconGC = JXCreateGC(display, rootWindow, gcMask, &gcValues);
 
-   iconSize.min_width = settings.titleHeight - 4;
-   iconSize.min_height = settings.titleHeight - 4;
+   iconSize.min_width = GetBorderIconSize();
+   iconSize.min_height = GetBorderIconSize();
    iconSize.max_width = iconSize.min_width;
    iconSize.max_height = iconSize.min_height;
    iconSize.width_inc = 1;
@@ -153,7 +172,7 @@ void AddIconPath(char *path)
 
    ip = Allocate(sizeof(IconPathNode));
    ip->path = Allocate(length + addSep + 1);
-   strcpy(ip->path, path);
+   memcpy(ip->path, path, length + 1);
    if(addSep) {
       ip->path[length] = '/';
       ip->path[length + 1] = 0;
@@ -171,10 +190,9 @@ void AddIconPath(char *path)
 }
 
 /** Draw an icon. */
-void PutIcon(const VisualData *visual, IconNode *icon, Drawable d, long fg,
+void PutIcon(IconNode *icon, Drawable d, long fg,
              int x, int y, int width, int height)
 {
-   ImageNode *imageNode;
    ScaledIconNode *node;
 
    Assert(icon);
@@ -184,17 +202,13 @@ void PutIcon(const VisualData *visual, IconNode *icon, Drawable d, long fg,
    }
 
    /* Scale the icon. */
-   imageNode = GetBestImage(icon, width, height);
-   node = GetScaledIcon(icon, imageNode, fg, width, height);
+   node = GetScaledIcon(icon, fg, width, height);
    if(node) {
 
-      const int ix = x + (width - node->width) / 2;
-      const int iy = y + (height - node->height) / 2;
-
       /* If we support xrender, use it. */
 #ifdef USE_XRENDER
-      if(haveRender) {
-         PutScaledRenderIcon(visual, imageNode, d, ix, iy);
+      if(icon->render) {
+         PutScaledRenderIcon(icon, node, d, x, y, width, height);
          return;
       }
 #endif
@@ -202,6 +216,9 @@ void PutIcon(const VisualData *visual, IconNode *icon, Drawable d, long fg,
       /* Draw the icon the old way. */
       if(node->image != None) {
 
+         const int ix = x + (width - node->width) / 2;
+         const int iy = y + (height - node->height) / 2;
+
          /* Set the clip mask. */
          if(node->mask != None) {
             JXSetClipOrigin(display, iconGC, ix, iy);
@@ -227,174 +244,181 @@ void PutIcon(const VisualData *visual, IconNode *icon, Drawable d, long fg,
 /** Load the icon for a client. */
 void LoadIcon(ClientNode *np)
 {
-
-   IconPathNode *ip;
-
-   Assert(np);
-
    /* If client already has an icon, destroy it first. */
    DestroyIcon(np->icon);
    np->icon = NULL;
 
    /* Attempt to read _NET_WM_ICON for an icon. */
-   ReadNetWMIcon(np);
+   np->icon = ReadNetWMIcon(np->window);
    if(np->icon) {
       return;
    }
-
-   /* Attempt to read an icon from XWMHints. */
-   ReadWMHintIcon(np);
-   if(np->icon) {
-      return;
-   }
-
-   /* Attempt to find an icon for this program in the icon directory */
-   if(np->instanceName) {
-      for(ip = iconPaths; ip; ip = ip->next) {
-
-#ifdef USE_PNG
-         np->icon = LoadSuffixedIcon(ip->path, np->instanceName, ".png");
-         if(np->icon) {
-            return;
-         }
-#endif
-
-#ifdef USE_XPM
-         np->icon = LoadSuffixedIcon(ip->path, np->instanceName, ".xpm");
-         if(np->icon) {
-            return;
-         }
-#endif
-
-#ifdef USE_JPEG
-         np->icon = LoadSuffixedIcon(ip->path, np->instanceName, ".jpg");
-         if(np->icon) {
-            return;
-         }
-#endif
-
-#ifdef USE_ICONS
-         np->icon = LoadSuffixedIcon(ip->path, np->instanceName, ".xbm");
-         if(np->icon) {
-            return;
-         }
-#endif
-
+   if(np->owner != None) {
+      np->icon = ReadNetWMIcon(np->owner);
+      if(np->icon) {
+         return;
       }
    }
 
-   /* Load the default icon */
-   np->icon = GetDefaultIcon();
-
-}
-
-/** Load an icon given a name, path, and suffix. */
-#if defined(USE_ICONS)
-IconNode *LoadSuffixedIcon(const char *path, const char *name,
-                           const char *suffix)
-{
-
-   IconNode *result;
-   ImageNode *image;
-   char *iconName;
-   unsigned int len;
-
-   Assert(path);
-   Assert(name);
-   Assert(suffix);
-
-   len = strlen(name) + strlen(path) + strlen(suffix);
-   iconName = Allocate(len + 1);
-   strcpy(iconName, path);
-   strcat(iconName, name);
-   strcat(iconName, suffix);
-
-   result = FindIcon(iconName);
-   if(result) {
-      Release(iconName);
-      return result;
+   /* Attempt to read an icon from XWMHints. */
+   np->icon = ReadWMHintIcon(np->window);
+   if(np->icon) {
+      return;
+   }
+   if(np->owner != None) {
+      np->icon = ReadNetWMIcon(np->owner);
+      if(np->icon) {
+         return;
+      }
    }
 
-   image = LoadImage(iconName);
-   if(image) {
-      result = CreateIcon();
-      result->name = iconName;
-      result->images = image;
-      InsertIcon(result);
-      return result;
-   } else {
-      Release(iconName);
-      return NULL;
+   /* Attempt to read an icon based on the window name. */
+   if(np->instanceName) {
+      np->icon = LoadNamedIcon(np->instanceName, 1, 1);
+      if(np->icon) {
+         return;
+      }
    }
-
 }
-#endif /* defined(USE_PNG) || defined(USE_XPM) || defined(USE_JPEG) */
 
 /** Load an icon from a file. */
 IconNode *LoadNamedIcon(const char *name, char save, char preserveAspect)
 {
 
-   IconPathNode *ip;
    IconNode *icon;
+   IconPathNode *ip;
 
    Assert(name);
 
-   if(name[0] == '/') {
-      return CreateIconFromFile(name, save, preserveAspect);
-   } else {
-      for(ip = iconPaths; ip; ip = ip->next) {
-         icon = LoadNamedIconHelper(name, ip->path, save, preserveAspect);
-         if(icon) {
-            return icon;
-         }
-      }
-      return NULL;
+   /* If no icon is specified, return an empty icon. */
+   if(name[0] == 0) {
+      return &emptyIcon;
    }
 
+   /* See if this icon has already been loaded. */
+   icon = FindIcon(name);
+   if(icon) {
+      return icon;
+   }
+
+   /* Check for an absolute file name. */
+   if(name[0] == '/') {
+      ImageNode *image = LoadImage(name, 0, 0, 1);
+      if(image) {
+         icon = CreateIcon(image);
+         icon->preserveAspect = preserveAspect;
+         icon->name = CopyString(name);
+         if(save) {
+            InsertIcon(icon);
+         }
+         DestroyImage(image);
+         return icon;
+      } else {
+         return &emptyIcon;
+      }
+   }
+
+   /* Try icon paths. */
+   for(ip = iconPaths; ip; ip = ip->next) {
+      icon = LoadNamedIconHelper(name, ip->path, save, preserveAspect);
+      if(icon) {
+         return icon;
+      }
+   }
+
+   /* The default icon. */
+   return NULL;
 }
 
 /** Helper for loading icons by name. */
 IconNode *LoadNamedIconHelper(const char *name, const char *path,
                               char save, char preserveAspect)
 {
-
-   IconNode *result;
+   ImageNode *image;
    char *temp;
+   const unsigned nameLength = strlen(name);
+   const unsigned pathLength = strlen(path);
+   unsigned i;
+   char hasExtension;
 
-   temp = AllocateStack(strlen(name) + strlen(path) + 1);
-   strcpy(temp, path);
-   strcat(temp, name);
-   result = CreateIconFromFile(temp, save, preserveAspect);
+   /* Full file name. */
+   temp = AllocateStack(nameLength + pathLength + MAX_EXTENSION_LENGTH + 1);
+   memcpy(&temp[0], path, pathLength);
+   memcpy(&temp[pathLength], name, nameLength + 1);
+
+   /* Determine if the extension is provided.
+    * We avoid extra file opens if so.
+    */
+   hasExtension = 0;
+   for(i = 1; i < EXTENSION_COUNT; i++) {
+      const unsigned offset = nameLength + pathLength;
+      const unsigned extLength = strlen(ICON_EXTENSIONS[i]);
+      if(JUNLIKELY(offset < extLength)) {
+         continue;
+      }
+      if(!strcmp(ICON_EXTENSIONS[i], &temp[offset])) {
+         hasExtension = 1;
+         break;
+      }
+   }
+
+   /* Attempt to load the image. */
+   image = NULL;
+   if(hasExtension) {
+      image = LoadImage(temp, 0, 0, 1);
+   } else {
+      for(i = 0; i < EXTENSION_COUNT; i++) {
+         const unsigned len = strlen(ICON_EXTENSIONS[i]);
+         memcpy(&temp[pathLength + nameLength], ICON_EXTENSIONS[i], len + 1);
+         image = LoadImage(temp, 0, 0, 1);
+         if(image) {
+            break;
+         }
+      }
+   }
    ReleaseStack(temp);
 
-   return result;
+   /* Create the icon if we were able to load the image. */
+   if(image) {
+      IconNode *result = CreateIcon(image);
+      result->preserveAspect = preserveAspect;
+      result->name = CopyString(temp);
+      if(save) {
+         InsertIcon(result);
+      }
+      DestroyImage(image);
+      return result;
+   }
 
+   return NULL;
 }
 
 /** Read the icon property from a client. */
-void ReadNetWMIcon(ClientNode *np)
+IconNode *ReadNetWMIcon(Window win)
 {
    static const long MAX_LENGTH = 1 << 20;
+   IconNode *icon = NULL;
    unsigned long count;
    int status;
    unsigned long extra;
    Atom realType;
    int realFormat;
    unsigned char *data;
-   status = JXGetWindowProperty(display, np->window, atoms[ATOM_NET_WM_ICON],
+   status = JXGetWindowProperty(display, win, atoms[ATOM_NET_WM_ICON],
                                 0, MAX_LENGTH, False, XA_CARDINAL,
                                 &realType, &realFormat, &count, &extra, &data);
    if(status == Success && realFormat != 0 && data) {
-      np->icon = CreateIconFromBinary((unsigned long*)data, count);
+      icon = CreateIconFromBinary((unsigned long*)data, count);
       JXFree(data);
    }
+   return icon;
 }
 
 /** Read the icon WMHint property from a client. */
-void ReadWMHintIcon(ClientNode *np)
+IconNode *ReadWMHintIcon(Window win)
 {
-   XWMHints *hints;
-   hints = JXGetWMHints(display, np->window);
+   IconNode *icon = NULL;
+   XWMHints *hints = JXGetWMHints(display, win);
    if(hints) {
       Drawable d = None;
       Pixmap mask = None;
@@ -405,27 +429,24 @@ void ReadWMHintIcon(ClientNode *np)
          d = hints->icon_pixmap;
       }
       if(d != None) {
-         np->icon = CreateIconFromDrawable(d, mask);
+         icon = CreateIconFromDrawable(d, mask);
       }
       JXFree(hints);
    }
-}
-
-/** Create the default icon. */
-IconNode *GetDefaultIcon(void)
-{
-   return CreateIconFromData("default", x_xpm);
+   return icon;
 }
 
 /** Create an icon from XPM image data. */
-IconNode *CreateIconFromData(const char *name, char **data)
+IconNode *GetDefaultIcon(void)
 {
-
+   static const char * const name = "default";
+   const unsigned width = 8;
+   const unsigned height = 8;
+   const unsigned border = 1;
    ImageNode *image;
    IconNode *result;
-
-   Assert(name);
-   Assert(data);
+   unsigned bytes;
+   unsigned x, y;
 
    /* Check if this icon has already been loaded */
    result = FindIcon(name);
@@ -433,17 +454,43 @@ IconNode *CreateIconFromData(const char *name, char **data)
       return result;
    }
 
-   image = LoadImageFromData(data);
-   if(image) {
-      result = CreateIcon();
-      result->name = CopyString(name);
-      result->images = image;
-      InsertIcon(result);
-      return result;
-   } else {
-      return NULL;
+   /* Allocate image data. */
+   bytes = (width * height + 7) / 8;
+   image = CreateImage(width, height, 1);
+   memset(image->data, 0, bytes);
+#ifdef USE_XRENDER
+   image->render = 0;
+#endif
+
+   /* Allocate the icon node. */
+   result = CreateIcon(image);
+   result->name = CopyString(name);
+   result->images = image;
+   InsertIcon(result);
+
+   /* Draw the icon. */
+   for(y = border; y < height - border; y++) {
+      const unsigned pixel_left = y * width + border;
+      const unsigned pixel_right = y * width + width - 1 - border;
+      const unsigned offset_left = pixel_left / 8;
+      const unsigned mask_left = 1 << (pixel_left % 8);
+      const unsigned offset_right = pixel_right / 8;
+      const unsigned mask_right = 1 << (pixel_right % 8);
+      image->data[offset_left] |= mask_left;
+      image->data[offset_right] |= mask_right;
+   }
+   for(x = border; x < width - border; x++) {
+      const unsigned pixel_top = x + border * width;
+      const unsigned pixel_bottom = x + width * (height - 1 - border);
+      const unsigned offset_top = pixel_top / 8;
+      const unsigned mask_top = 1 << (pixel_top % 8);
+      const unsigned offset_bottom = pixel_bottom / 8;
+      const unsigned mask_bottom = 1 << (pixel_bottom % 8);
+      image->data[offset_top] |= mask_top;
+      image->data[offset_bottom] |= mask_bottom;
    }
 
+   return result;
 }
 
 IconNode *CreateIconFromDrawable(Drawable d, Pixmap mask)
@@ -452,7 +499,7 @@ IconNode *CreateIconFromDrawable(Drawable d, Pixmap mask)
 
    image = LoadImageFromDrawable(d, mask);
    if(image) {
-      IconNode *result = CreateIcon();
+      IconNode *result = CreateIcon(image);
       result->images = image;
       return result;
    } else {
@@ -460,51 +507,24 @@ IconNode *CreateIconFromDrawable(Drawable d, Pixmap mask)
    }
 }
 
-/** Create an icon from the specified file. */
-IconNode *CreateIconFromFile(const char *fileName,
-                             char save, char preserveAspect)
-{
-
-   ImageNode *image;
-   IconNode *result;
-
-   if(!fileName) {
-      return NULL;
-   }
-
-   /* Check if this icon has already been loaded */
-   result = FindIcon(fileName);
-   if(result) {
-      return result;
-   }
-
-   image = LoadImage(fileName);
-   if(image) {
-      result = CreateIcon();
-      result->preserveAspect = preserveAspect;
-      result->images = image;
-      if(save) {
-         result->name = CopyString(fileName);
-         InsertIcon(result);
-      }
-      return result;
-   } else {
-      return NULL;
-   }
-
-}
-
 /** Get the best image for the requested size. */
 ImageNode *GetBestImage(IconNode *icon, int rwidth, int rheight)
 {
+   ImageNode *best;
+   ImageNode *ip;
+
+   /* If we don't have an image loaded, load one. */
+   if(icon->images == NULL) {
+      return LoadImage(icon->name, rwidth, rheight, icon->preserveAspect);
+   }
+
    /* Find the best image to use.
     * Select the smallest image to completely cover the
     * requested size.  If no image completely covers the
     * requested size, select the one that overlaps the most area.
     * If no size is specified, use the largest. */
-   ImageNode *best = icon->images;
-   ImageNode *ip = icon->images->next;
-   while(ip) {
+   best = icon->images;
+   for(ip = icon->images->next; ip; ip = ip->next) {
       const int best_area = best->width * best->height;
       const int other_area = ip->width * ip->height;
       int best_overlap;
@@ -531,37 +551,37 @@ ImageNode *GetBestImage(IconNode *icon, int rwidth, int rheight)
             best = ip;
          }
       }
-      ip = ip->next;
    }
    return best;
 }
 
 /** Get a scaled icon. */
-ScaledIconNode *GetScaledIcon(IconNode *icon, ImageNode *iconImage,
-                              long fg, int rwidth, int rheight)
+ScaledIconNode *GetScaledIcon(IconNode *icon, long fg,
+                              int rwidth, int rheight)
 {
 
    XColor color;
    XImage *image;
    XPoint *points;
+   ImageNode *imageNode;
    ScaledIconNode *np;
    GC maskGC;
    int x, y;
    int scalex, scaley;     /* Fixed point. */
    int srcx, srcy;         /* Fixed point. */
-   int ratio;              /* Fixed point. */
    int nwidth, nheight;
    unsigned char *data;
+   unsigned perLine;
 
    if(rwidth == 0) {
-      rwidth = iconImage->width;
+      rwidth = icon->width;
    }
    if(rheight == 0) {
-      rheight = iconImage->height;
+      rheight = icon->height;
    }
 
    if(icon->preserveAspect) {
-      ratio = (iconImage->width << 16) / iconImage->height;
+      const int ratio = (icon->width << 16) / icon->height;
       nwidth = Min(rwidth, (rheight * ratio) >> 16);
       nheight = Min(rheight, (nwidth << 16) / ratio);
       nwidth = (nheight * ratio) >> 16;
@@ -572,35 +592,41 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, ImageNode *iconImage,
    nwidth = Max(1, nwidth);
    nheight = Max(1, nheight);
 
-   /* Check if this size already exists.
-    * Note that XRender scales on the fly.
-    */
-   for(np = iconImage->nodes; np; np = np->next) {
+   /* Check if this size already exists. */
+   for(np = icon->nodes; np; np = np->next) {
+      if(!icon->bitmap || np->fg == fg) {
 #ifdef USE_XRENDER
-      if(np->imagePicture != None) {
-         np->width = nwidth;
-         np->height = nheight;
-         return np;
-      }
+         /* If we are using xrender and only have one image size
+          * available, we can simply scale the existing icon. */
+         if(icon->render) {
+            if(icon->images == NULL || icon->images->next == NULL) {
+               return np;
+            }
+         }
 #endif
-      if(np->width == nwidth && np->height == nheight) {
-         if(!iconImage->bitmap || np->fg == fg) {
+         if(np->width == nwidth && np->height == nheight) {
             return np;
          }
       }
    }
 
+   /* Need to load the image. */
+   imageNode = GetBestImage(icon, nwidth, nheight);
+   if(JUNLIKELY(!imageNode)) {
+      return NULL;
+   }
+
    /* See if we can use XRender to create the icon. */
 #ifdef USE_XRENDER
-   if(haveRender) {
-      np = CreateScaledRenderIcon(iconImage, fg);
+   if(icon->render) {
+      np = CreateScaledRenderIcon(imageNode, fg);
+      np->next = icon->nodes;
+      icon->nodes = np;
 
       /* Don't keep the image data around after creating the icon. */
-      Release(iconImage->data);
-      iconImage->data = NULL;
-
-      np->width = nwidth;
-      np->height = nheight;
+      if(icon->images == NULL) {
+         DestroyImage(imageNode);
+      }
 
       return np;
    }
@@ -611,11 +637,8 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, ImageNode *iconImage,
    np->fg = fg;
    np->width = nwidth;
    np->height = nheight;
-   np->next = iconImage->nodes;
-#ifdef USE_XRENDER
-   np->imagePicture = None;
-#endif
-   iconImage->nodes = np;
+   np->next = icon->nodes;
+   icon->nodes = np;
 
    /* Create a mask. */
    np->mask = JXCreatePixmap(display, rootWindow, nwidth, nheight, 1);
@@ -625,26 +648,31 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, ImageNode *iconImage,
    JXSetForeground(display, maskGC, 1);
 
    /* Create a temporary XImage for scaling. */
-   image = JXCreateImage(display, rootVisual.visual, rootVisual.depth,
+   image = JXCreateImage(display, rootVisual, rootDepth,
                          ZPixmap, 0, NULL, nwidth, nheight, 8, 0);
    image->data = Allocate(sizeof(unsigned long) * nwidth * nheight);
 
    /* Determine the scale factor. */
-   scalex = (iconImage->width << 16) / nwidth;
-   scaley = (iconImage->height << 16) / nheight;
+   scalex = (imageNode->width << 16) / nwidth;
+   scaley = (imageNode->height << 16) / nheight;
 
    points = Allocate(sizeof(XPoint) * nwidth);
-   data = iconImage->data;
+   data = imageNode->data;
+   if(imageNode->bitmap) {
+      perLine = (imageNode->width >> 3) + ((imageNode->width & 7) ? 1 : 0);
+   } else {
+      perLine = imageNode->width;
+   }
    srcy = 0;
    for(y = 0; y < nheight; y++) {
-      const int yindex = (srcy >> 16) * iconImage->width;
+      const int yindex = (srcy >> 16) * perLine;
       int pindex = 0;
       srcx = 0;
       for(x = 0; x < nwidth; x++) {
-         if(iconImage->bitmap) {
-            const int index = yindex + (srcx >> 16);
-            const int offset = index >> 3;
-            const int mask = 1 << (index & 7);
+         if(imageNode->bitmap) {
+            const int tx = srcx >> 16;
+            const int offset = yindex + (tx >> 3);
+            const int mask = 1 << (tx & 7);
             if(data[offset] & mask) {
                points[pindex].x = x;
                points[pindex].y = y;
@@ -652,7 +680,7 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, ImageNode *iconImage,
                pindex += 1;
             }
          } else {
-            const int yindex = (srcy >> 16) * iconImage->width;
+            const int yindex = (srcy >> 16) * imageNode->width;
             const int index = 4 * (yindex + (srcx >> 16));
             color.red = data[index + 1];
             color.red |= color.red << 8;
@@ -680,7 +708,7 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, ImageNode *iconImage,
  
    /* Create the color data pixmap. */
    np->image = JXCreatePixmap(display, rootWindow, nwidth, nheight,
-                              rootVisual.depth);
+                              rootDepth);
 
    /* Render the image to the color data pixmap. */
    JXPutImage(display, np->image, rootGC, image, 0, 0, 0, 0, nwidth, nheight);   
@@ -689,6 +717,10 @@ ScaledIconNode *GetScaledIcon(IconNode *icon, ImageNode *iconImage,
    image->data = NULL;
    JXDestroyImage(image);
 
+   if(icon->images == NULL) {
+      DestroyImage(imageNode);
+   }
+
    return np;
 
 }
@@ -706,11 +738,11 @@ IconNode *CreateIconFromBinary(const unsigned long *input,
 
    while(offset < length) {
 
-      const unsigned int width = input[offset + 0];
-      const unsigned int height = input[offset + 1];
+      const unsigned width = input[offset + 0];
+      const unsigned height = input[offset + 1];
       unsigned char *data;
       ImageNode *image;
-      unsigned int x, index;
+      unsigned x;
 
       if(JUNLIKELY(width * height + 2 > length - offset)) {
          Debug("invalid image size: %d x %d + 2 > %d",
@@ -721,27 +753,24 @@ IconNode *CreateIconFromBinary(const unsigned long *input,
          return result;
       }
 
-      if(result == NULL) {
-         result = CreateIcon();
-      }
-
       image = CreateImage(width, height, 0);
+      if(result == NULL) {
+         result = CreateIcon(image);
+      }
       image->next = result->images;
       result->images = image;
       data = image->data;
 
       /* Note: the data types here might be of different sizes. */
-      index = 0;
+      offset += 2;
       for(x = 0; x < width * height; x++) {
-         data[index++] = (input[offset + x + 2] >> 24) & 0xFF;
-         data[index++] = (input[offset + x + 2] >> 16) & 0xFF;
-         data[index++] = (input[offset + x + 2] >>  8) & 0xFF;
-         data[index++] = (input[offset + x + 2] >>  0) & 0xFF;
+         *data++ = (input[offset] >> 24) & 0xFF;
+         *data++ = (input[offset] >> 16) & 0xFF;
+         *data++ = (input[offset] >>  8) & 0xFF;
+         *data++ = (input[offset] >>  0) & 0xFF;
+         offset += 1;
       }
 
-      /* Move to the next size. */
-      offset += 2 + width * height;
-
       /* Don't insert this icon into the hash since it is transient. */
 
    }
@@ -750,52 +779,58 @@ IconNode *CreateIconFromBinary(const unsigned long *input,
 }
 
 /** Create an empty icon node. */
-IconNode *CreateIcon(void)
+IconNode *CreateIcon(const ImageNode *image)
 {
    IconNode *icon;
    icon = Allocate(sizeof(IconNode));
+   icon->nodes = NULL;
    icon->name = NULL;
    icon->images = NULL;
    icon->next = NULL;
    icon->prev = NULL;
+   icon->width = image->width;
+   icon->height = image->height;
+   icon->bitmap = image->bitmap;
+#ifdef USE_XRENDER
+   icon->render = image->render;
+#endif
    icon->preserveAspect = 1;
+   icon->transient = 1;
    return icon;
 }
 
 /** Helper method for destroy icons. */
 void DoDestroyIcon(int index, IconNode *icon)
 {
-   if(icon) {
-      ImageNode *image = icon->images;
-      while(image) {
-         ScaledIconNode *np = image->nodes;
-         while(np) {
-            ScaledIconNode *next_node = np->next;
-
+   if(icon && icon != &emptyIcon) {
+      while(icon->nodes) {
+         ScaledIconNode *np = icon->nodes;
 #ifdef USE_XRENDER
-            if(np->imagePicture != None) {
-               JXRenderFreePicture(display, np->imagePicture);
+         if(icon->render) {
+            if(np->image != None) {
+               JXRenderFreePicture(display, np->image);
             }
-            if(np->alphaPicture != None) {
-               JXRenderFreePicture(display, np->alphaPicture);
+            if(np->mask != None) {
+               JXRenderFreePicture(display, np->mask);
             }
+#else
+         if(0) {
 #endif
+         } else {
             if(np->image != None) {
                JXFreePixmap(display, np->image);
             }
             if(np->mask != None) {
                JXFreePixmap(display, np->mask);
             }
-
-            Release(np);
-            np = next_node;
          }
-         image = image->next;
+         icon->nodes = np->next;
+         Release(np);
       }
+      DestroyImage(icon->images);
       if(icon->name) {
          Release(icon->name);
       }
-      DestroyImage(icon->images);
 
       if(icon->prev) {
          icon->prev->next = icon->next;
@@ -812,7 +847,7 @@ void DoDestroyIcon(int index, IconNode *icon)
 /** Destroy an icon. */
 void DestroyIcon(IconNode *icon)
 {
-   if(icon && !icon->name) {
+   if(icon && icon->transient) {
       const unsigned int index = GetHash(icon->name);
       DoDestroyIcon(index, icon);
    }
@@ -830,6 +865,7 @@ void InsertIcon(IconNode *icon)
       iconHash[index]->prev = icon;
    }
    icon->next = iconHash[index];
+   icon->transient = 0;
    iconHash[index] = icon;
 }
 
@@ -863,4 +899,3 @@ unsigned int GetHash(const char *str)
 }
 
 #endif /* USE_ICONS */
-
