@@ -255,7 +255,7 @@ void ShowKeyMenu(const XKeyEvent *event)
       if(np->state == state && np->code == event->keycode) {
          const int button = GetRootMenuIndexFromString(np->command);
          if(JLIKELY(button >= 0)) {
-            ShowRootMenu(button, 0, 0);
+            ShowRootMenu(button, -1, -1, 1);
          }
          return;
       }
@@ -301,26 +301,13 @@ char ShouldGrab(KeyType key)
    case KEY_MAXRIGHT:
    case KEY_MAXV:
    case KEY_MAXH:
+   case KEY_RESTORE:
       return 1;
    default:
       return 0;
    }
 }
 
-/** Grab keys on a client window. */
-void GrabKeys(ClientNode *np)
-{
-
-   KeyNode *kp;
-
-   for(kp = bindings; kp; kp = kp->next) {
-      if(ShouldGrab(kp->key)) {
-         GrabKey(kp, np->window);
-      }
-   }
-
-}
-
 /** Get the modifier mask for a key. */
 unsigned int GetModifierMask(XModifierKeymap *modmap, KeySym key) {
 
