@@ -10,8 +10,6 @@
 #ifndef KEY_H
 #define KEY_H
 
-struct ClientNode;
-
 /** Enumeration of key binding types.
  * Note that we use the high bits to store additional information
  * for some key types (for example the desktop number).
@@ -58,6 +56,7 @@ typedef unsigned short KeyType;
 #define KEY_MAXRIGHT        38
 #define KEY_MAXV            39
 #define KEY_MAXH            40
+#define KEY_RESTORE         41
 
 void InitializeKeys(void);
 void StartupKeys(void);
@@ -72,11 +71,6 @@ extern unsigned int lockMask;
  */
 KeyType GetKey(const XKeyEvent *event);
 
-/** Grab keys on a client window.
- * @param np The client.
- */
-void GrabKeys(struct ClientNode *np);
-
 /** Insert a key binding.
  * @param key The key binding type.
  * @param modifiers The modifier mask.
