@@ -25,6 +25,7 @@ static const StringMappingType TOKEN_MAP[] = {
    { "ButtonClose",        TOK_BUTTONCLOSE      },
    { "ButtonMax",          TOK_BUTTONMAX        },
    { "ButtonMaxActive",    TOK_BUTTONMAXACTIVE  },
+   { "ButtonMenu",         TOK_BUTTONMENU       },
    { "ButtonMin",          TOK_BUTTONMIN        },
    { "Class",              TOK_CLASS            },
    { "Clock",              TOK_CLOCK            },
@@ -79,11 +80,9 @@ static const StringMappingType TOKEN_MAP[] = {
    { "Stick",              TOK_STICK            },
    { "Swallow",            TOK_SWALLOW          },
    { "TaskList",           TOK_TASKLIST         },
-   { "TaskListStyle",      TOK_TASKLISTSTYLE    },
    { "Text",               TOK_TEXT             },
    { "Tray",               TOK_TRAY             },
    { "TrayButton",         TOK_TRAYBUTTON       },
-   { "TrayButtonStyle",    TOK_TRAYBUTTONSTYLE  },
    { "TrayStyle",          TOK_TRAYSTYLE        },
    { "Width",              TOK_WIDTH            },
    { "WindowStyle",        TOK_WINDOWSTYLE      }
@@ -290,10 +289,11 @@ ReadDefault:
             if(temp) {
                if(current) {
                   if(current->value) {
+                     const unsigned value_len = strlen(current->value);
+                     const unsigned temp_len = strlen(temp);
                      current->value = Reallocate(current->value,
-                                                 strlen(current->value) +
-                                                 strlen(temp) + 1);
-                     strcat(current->value, temp);
+                                                 value_len + temp_len + 1);
+                     memcpy(&current->value[value_len], temp, temp_len + 1);
                      Release(temp);
                   } else {
                      current->value = temp;
@@ -338,6 +338,9 @@ int ParseEntity(const char *entity, char *ch, const char *file,
    } else if(!strncmp("&apos;", entity, 6)) {
       *ch = '\'';
       return 6;
+   } else if(!strncmp("&NewLine;", entity, 9)) {
+      *ch = '\n';
+      return 9;
    } else {
       unsigned int x;
       for(x = 0; entity[x]; x++) {
@@ -345,13 +348,22 @@ int ParseEntity(const char *entity, char *ch, const char *file,
             break;
          }
       }
-      temp = AllocateStack(x + 2);
-      strncpy(temp, entity, x + 1);
-      temp[x + 1] = 0;
-      Warning(_("%s[%d]: invalid entity: \"%.8s\""), file, line, temp);
-      ReleaseStack(temp);
-      *ch = '&';
-      return 1;
+      if(entity[1] == '#' && entity[x] == ';') {
+         if(entity[2] == 'x') {
+            *ch = (char)strtol(&entity[3], NULL, 16);
+         } else {
+            *ch = (char)strtol(&entity[2], NULL, 10);
+         }
+         return x + 1;
+      } else {
+         temp = AllocateStack(x + 2);
+         strncpy(temp, entity, x + 1);
+         temp[x + 1] = 0;
+         Warning(_("%s[%d]: invalid entity: \"%.8s\""), file, line, temp);
+         ReleaseStack(temp);
+         *ch = '&';
+         return 1;
+      }
    }
 }
 
@@ -533,8 +545,7 @@ TokenNode *CreateNode(TokenNode *current, const char *file,
    np->parent = current;
    np->next = NULL;
 
-   np->fileName = Allocate(strlen(file) + 1);
-   strcpy(np->fileName, file);
+   np->fileName = file;
    np->line = line;
    np->invalidName = NULL;
 
@@ -612,10 +623,6 @@ void ReleaseTokens(TokenNode *np)
          Release(np->invalidName);
       }
 
-      if(np->fileName) {
-         Release(np->fileName);
-      }
-
       Release(np);
       np = tp;
    }
