@@ -22,6 +22,9 @@
 #include "parse.h"
 #include "winmenu.h"
 #include "screen.h"
+#include "hint.h"
+#include "misc.h"
+#include "popup.h"
 
 #define BASE_ICON_OFFSET   3
 #define MENU_BORDER_SIZE   1
@@ -33,15 +36,17 @@ typedef unsigned char MenuSelectionType;
 
 static char ShowSubmenu(Menu *menu, Menu *parent,
                         RunMenuCommandType runner,
-                        int x, int y);
+                        int x, int y, char keyboard);
 
 static void PatchMenu(Menu *menu);
 static void UnpatchMenu(Menu *menu);
-static void CreateMenu(Menu *menu, int x, int y);
+static void CreateMenu(Menu *menu, int x, int y, char keyboard);
 static void HideMenu(Menu *menu);
 static void DrawMenu(Menu *menu);
 
 static char MenuLoop(Menu *menu, RunMenuCommandType runner);
+static void MenuCallback(const TimeType *now, int x, int y,
+                         Window w, void *data);
 static MenuSelectionType UpdateMotion(Menu *menu,
                                       RunMenuCommandType runner,
                                       XEvent *event);
@@ -118,7 +123,7 @@ void InitializeMenu(Menu *menu)
       }
    }
 
-   menu->height = 0;
+   menu->height = MENU_BORDER_SIZE;
    if(menu->label) {
       menu->height += menu->itemHeight;
    }
@@ -135,7 +140,7 @@ void InitializeMenu(Menu *menu)
    for(np = menu->items; np; np = np->next) {
       menu->offsets[index++] = menu->height;
       if(np->type == MENU_ITEM_SEPARATOR) {
-         menu->height += 5;
+         menu->height += 6;
       } else {
          menu->height += menu->itemHeight;
       }
@@ -154,16 +159,17 @@ void InitializeMenu(Menu *menu)
       }
    }
    menu->width += hasSubmenu + menu->textOffset;
-   menu->width += 7;
+   menu->width += 7 + 2 * MENU_BORDER_SIZE;
+   menu->height += MENU_BORDER_SIZE;
+   menu->mousex = -1;
+   menu->mousey = -1;
 
 }
 
 /** Show a menu. */
-void ShowMenu(Menu *menu, RunMenuCommandType runner, int x, int y)
+void ShowMenu(Menu *menu, RunMenuCommandType runner,
+              int x, int y, char keyboard)
 {
-
-   int mouseStatus, keyboardStatus;
-
    /* Don't show the menu if there isn't anything to show. */
    if(JUNLIKELY(!IsMenuValid(menu))) {
       return;
@@ -172,14 +178,25 @@ void ShowMenu(Menu *menu, RunMenuCommandType runner, int x, int y)
       return;
    }
 
-   mouseStatus = GrabMouse(rootWindow);
-   keyboardStatus = JXGrabKeyboard(display, rootWindow, False,
-                                   GrabModeAsync, GrabModeAsync, CurrentTime);
-   if(JUNLIKELY(!mouseStatus || keyboardStatus != GrabSuccess)) {
+   if(x < 0 && y < 0) {
+      Window w;
+      GetMousePosition(&x, &y, &w);
+      x -= menu->itemHeight / 2;
+      y -= menu->itemHeight / 2 + menu->offsets[0];
+   }
+
+   if(!GrabMouse(rootWindow)) {
+      return;
+   }
+   if(JXGrabKeyboard(display, rootWindow, False, GrabModeAsync,
+                     GrabModeAsync, CurrentTime) != GrabSuccess) {
+      JXUngrabPointer(display, CurrentTime);
       return;
    }
 
-   ShowSubmenu(menu, NULL, runner, x, y);
+   RegisterCallback(100, MenuCallback, menu);
+   ShowSubmenu(menu, NULL, runner, x, y, keyboard);
+   UnregisterCallback(MenuCallback, menu);
    UnpatchMenu(menu);
 
    JXUngrabKeyboard(display, CurrentTime);
@@ -192,6 +209,15 @@ void ShowMenu(Menu *menu, RunMenuCommandType runner, int x, int y)
 
 }
 
+/** Hide a menu. */
+void HideMenu(Menu *menu)
+{
+   Menu *mp;
+   for(mp = menu; mp; mp = mp->parent) {
+      JXUnmapWindow(display, mp->window);
+   }
+}
+
 /** Destroy a menu. */
 void DestroyMenu(Menu *menu)
 {
@@ -202,12 +228,15 @@ void DestroyMenu(Menu *menu)
          if(menu->items->name) {
             Release(menu->items->name);
          }
+         if(menu->items->tooltip) {
+            Release(menu->items->tooltip);
+         }
          switch(menu->items->action.type & MA_ACTION_MASK) {
          case MA_EXECUTE:
          case MA_EXIT:
          case MA_DYNAMIC:
-            if(menu->items->action.data.str) {
-               Release(menu->items->action.data.str);
+            if(menu->items->action.str) {
+               Release(menu->items->action.str);
             }
             break;
          default:
@@ -235,20 +264,21 @@ void DestroyMenu(Menu *menu)
 /** Show a submenu. */
 char ShowSubmenu(Menu *menu, Menu *parent,
                  RunMenuCommandType runner,
-                 int x, int y)
+                 int x, int y, char keyboard)
 {
 
    char status;
 
    PatchMenu(menu);
    menu->parent = parent;
-   CreateMenu(menu, x, y);
+   CreateMenu(menu, x, y, keyboard);
 
    menuShown += 1;
    status = MenuLoop(menu, runner);
    menuShown -= 1;
 
-   HideMenu(menu);
+   JXDestroyWindow(display, menu->window);
+   JXFreePixmap(display, menu->pixmap);
 
    return status;
 
@@ -275,7 +305,8 @@ void PatchMenu(Menu *menu)
          break;
       case MA_DYNAMIC:
          if(!item->submenu) {
-            submenu = ParseDynamicMenu(item->action.data.str);
+            submenu = ParseDynamicMenu(item->action.str);
+            submenu->itemHeight = item->action.value;
          }
          break;
       default:
@@ -386,8 +417,19 @@ char MenuLoop(Menu *menu, RunMenuCommandType runner)
             ip = GetMenuItem(menu, menu->currentIndex);
          }
          if(ip != NULL) {
-            ip->action.button = event.xbutton.button;
-            (runner)(&ip->action);
+            if(ip->type == MENU_ITEM_NORMAL) {
+               HideMenu(menu);
+               (runner)(&ip->action, event.xbutton.button);
+            } else {
+               const Menu *parent = ip->type == MENU_ITEM_SUBMENU
+                                  ? menu->parent : menu;
+               if(event.xbutton.x >= parent->x &&
+                  event.xbutton.x < parent->x + parent->width &&
+                  event.xbutton.y >= parent->y &&
+                  event.xbutton.y < parent->y + parent->height) {
+                  break;
+               }
+            }
          }
          return 1;
       default:
@@ -397,17 +439,67 @@ char MenuLoop(Menu *menu, RunMenuCommandType runner)
    }
 }
 
-/** Create and map a menu. */
-void CreateMenu(Menu *menu, int x, int y)
+/** Signal for showing popups. */
+void MenuCallback(const TimeType *now, int x, int y, Window w, void *data)
 {
+   Menu *menu = data;
+   MenuItem *item;
+   int i;
 
+   /* Check if the mouse moved (and reset if it did). */
+   if(   abs(menu->mousex - x) > settings.doubleClickDelta
+      || abs(menu->mousey - y) > settings.doubleClickDelta) {
+      menu->mousex = x;
+      menu->mousey = y;
+      menu->lastTime = *now;
+      return;
+   }
+
+   /* See if enough time has passed. */
+   const unsigned long diff = GetTimeDifference(now, &menu->lastTime);
+   if(diff < settings.popupDelay) {
+      return;
+   }
+
+   /* Locate the active menu item. */
+   while(menu) {
+      if(x > menu->x && x < menu->x + menu->width) {
+         if(y > menu->y && y < menu->y + menu->height) {
+            break;
+         }
+      }
+      if(menu->currentIndex < 0) {
+         return;
+      }
+      item = menu->items;
+      for(i = 0; i < menu->currentIndex; i++) {
+         item = item->next;
+      }
+      if(item->type != MENU_ITEM_SUBMENU) {
+         return;
+      }
+      menu = item->submenu;
+   }
+   if(menu->currentIndex < 0) {
+      return;
+   }
+   item = menu->items;
+   for(i = 0; i < menu->currentIndex; i++) {
+      item = item->next;
+   }
+   if(item->tooltip) {
+      ShowPopup(x, y, item->tooltip, POPUP_MENU);
+   }
+
+}
+
+/** Create and map a menu. */
+void CreateMenu(Menu *menu, int x, int y, char keyboard)
+{
    XSetWindowAttributes attr;
    unsigned long attrMask;
    int temp;
 
-   menu->lastIndex = -1;
-   menu->currentIndex = -1;
-
    if(menu->parent) {
       menu->screen = menu->parent->screen;
    } else {
@@ -444,16 +536,14 @@ void CreateMenu(Menu *menu, int x, int y)
    attrMask |= CWSaveUnder;
    attr.save_under = True;
 
-   attrMask |= CWBorderPixel;
-   attr.border_pixel = colors[COLOR_MENU_OUTLINE];
-
    menu->window = JXCreateWindow(display, rootWindow, x, y,
-                                 menu->width, menu->height,
-                                 MENU_BORDER_SIZE,
+                                 menu->width, menu->height, 0,
                                  CopyFromParent, InputOutput,
                                  CopyFromParent, attrMask, &attr);
+   SetAtomAtom(menu->window, ATOM_NET_WM_WINDOW_TYPE,
+               ATOM_NET_WM_WINDOW_TYPE_MENU);
    menu->pixmap = JXCreatePixmap(display, menu->window,
-                                 menu->width, menu->height, rootVisual.depth);
+                                 menu->width, menu->height, rootDepth);
 
    if(settings.menuOpacity < UINT_MAX) {
       SetCardinalAtom(menu->window, ATOM_NET_WM_WINDOW_OPACITY,
@@ -462,13 +552,16 @@ void CreateMenu(Menu *menu, int x, int y)
 
    JXMapRaised(display, menu->window); 
 
-}
+   if(keyboard && menu->itemCount != 0) {
+      const int y = menu->offsets[0] + menu->itemHeight / 2;
+      menu->lastIndex = 0;
+      menu->currentIndex = 0;
+      MoveMouse(menu->window, menu->itemHeight / 2, y);
+   } else {
+      menu->lastIndex = -1;
+      menu->currentIndex = -1;
+   }
 
-/** Hide a menu. */
-void HideMenu(Menu *menu)
-{
-   JXDestroyWindow(display, menu->window);
-   JXFreePixmap(display, menu->pixmap);
 }
 
 /** Draw a menu. */
@@ -482,6 +575,24 @@ void DrawMenu(Menu *menu)
    JXFillRectangle(display, menu->pixmap, rootGC, 0, 0,
                    menu->width, menu->height);
 
+   if(settings.menuDecorations == DECO_MOTIF) {
+      JXSetForeground(display, rootGC, colors[COLOR_MENU_UP]);
+      JXDrawLine(display, menu->pixmap, rootGC,
+                 0, 0, menu->width, 0);
+      JXDrawLine(display, menu->pixmap, rootGC,
+                 0, 0, 0, menu->height);
+
+      JXSetForeground(display, rootGC, colors[COLOR_MENU_DOWN]);
+      JXDrawLine(display, menu->pixmap, rootGC,
+                 0, menu->height - 1, menu->width, menu->height - 1);
+      JXDrawLine(display, menu->pixmap, rootGC,
+                 menu->width - 1, 0, menu->width - 1, menu->height);
+   } else {
+      JXSetForeground(display, rootGC, colors[COLOR_MENU_DOWN]);
+      JXDrawRectangle(display, menu->pixmap, rootGC,
+                      0, 0, menu->width - 1, menu->height - 1);
+   }
+
    if(menu->label) {
       DrawMenuItem(menu, NULL, -1);
    }
@@ -572,8 +683,8 @@ MenuSelectionType UpdateMotion(Menu *menu,
       case KEY_ENTER:
          ip = GetMenuItem(tp, tp->currentIndex);
          if(ip != NULL) {
-            ip->action.button = 0;
-            (runner)(&ip->action);
+            HideMenu(menu);
+            (runner)(&ip->action, 0);
          }
          return MENU_SUBSELECT;
       default:
@@ -649,9 +760,10 @@ MenuSelectionType UpdateMotion(Menu *menu,
    /* If the selected item is a submenu, show it. */
    ip = GetMenuItem(menu, menu->currentIndex);
    if(ip && IsMenuValid(ip->submenu)) {
-      if(ShowSubmenu(ip->submenu, menu, runner,
-                     menu->x + menu->width + MENU_BORDER_SIZE,
-                     menu->y + menu->offsets[menu->currentIndex])) {
+      const int x = menu->x + menu->width
+                  - (settings.menuDecorations == DECO_MOTIF ? 0 : 1);
+      const int y = menu->y + menu->offsets[menu->currentIndex];
+      if(ShowSubmenu(ip->submenu, menu, runner, x, y, 0)) {
 
          /* Item selected; destroy the menu tree. */
          return MENU_SUBSELECT;
@@ -699,10 +811,10 @@ void DrawMenuItem(Menu *menu, MenuItem *item, int index)
 
    if(!item) {
       if(index == -1 && menu->label) {
-         ResetButton(&button, menu->pixmap, &rootVisual);
-         button.x = 0;
-         button.y = 0;
-         button.width = menu->width - 1;
+         ResetButton(&button, menu->pixmap);
+         button.x = MENU_BORDER_SIZE;
+         button.y = MENU_BORDER_SIZE;
+         button.width = menu->width - MENU_BORDER_SIZE * 2;
          button.height = menu->itemHeight - 1;
          button.font = FONT_MENU;
          button.type = BUTTON_LABEL;
@@ -716,7 +828,7 @@ void DrawMenuItem(Menu *menu, MenuItem *item, int index)
    if(item->type != MENU_ITEM_SEPARATOR) {
       ColorType fg;
 
-      ResetButton(&button, menu->pixmap, &rootVisual);
+      ResetButton(&button, menu->pixmap);
       if(menu->currentIndex == index) {
          button.type = BUTTON_MENU_ACTIVE;
          fg = COLOR_MENU_ACTIVE_FG;
@@ -725,10 +837,10 @@ void DrawMenuItem(Menu *menu, MenuItem *item, int index)
          fg = COLOR_MENU_FG;
       }
 
-      button.x = 0;
+      button.x = MENU_BORDER_SIZE;
       button.y = menu->offsets[index];
       button.font = FONT_MENU;
-      button.width = menu->width;
+      button.width = menu->width - MENU_BORDER_SIZE * 2;
       button.height = menu->itemHeight;
       button.text = item->name;
       button.icon = item->icon;
@@ -753,10 +865,21 @@ void DrawMenuItem(Menu *menu, MenuItem *item, int index)
       }
 
    } else {
-      JXSetForeground(display, rootGC, colors[COLOR_MENU_ACTIVE_FG]);
-      JXDrawLine(display, menu->pixmap, rootGC, 4,
-                 menu->offsets[index] + 2, menu->width - 6,
-                 menu->offsets[index] + 2);
+      if(settings.menuDecorations == DECO_MOTIF) {
+         JXSetForeground(display, rootGC, colors[COLOR_MENU_DOWN]);
+         JXDrawLine(display, menu->pixmap, rootGC, 4,
+                    menu->offsets[index] + 2, menu->width - 6,
+                    menu->offsets[index] + 2);
+         JXSetForeground(display, rootGC, colors[COLOR_MENU_UP]);
+         JXDrawLine(display, menu->pixmap, rootGC, 4,
+                    menu->offsets[index] + 3, menu->width - 6,
+                    menu->offsets[index] + 3);
+      } else {
+         JXSetForeground(display, rootGC, colors[COLOR_MENU_FG]);
+         JXDrawLine(display, menu->pixmap, rootGC, 4,
+                    menu->offsets[index] + 2, menu->width - 6,
+                    menu->offsets[index] + 2);
+      }
    }
 
 }
@@ -860,8 +983,8 @@ void SetPosition(Menu *tp, int index)
 
    /* We need to do this twice so the event gets registered
     * on the submenu if one exists. */
-   MoveMouse(tp->window, 6, y);
-   MoveMouse(tp->window, 6, y);
+   MoveMouse(tp->window, tp->itemHeight / 2, y);
+   MoveMouse(tp->window, tp->itemHeight / 2, y);
 
 }
 
