@@ -10,6 +10,8 @@
 #ifndef MENU_H
 #define MENU_H
 
+#include "timing.h"
+
 struct ScreenType;
 
 /** Enumeration of menu action types. */
@@ -42,15 +44,13 @@ typedef unsigned char MenuActionType;
 /** Structure to represent a menu action for callbacks. */
 typedef struct MenuAction {
 
-   MenuActionType type;          /**< Type of action. */
-   void *context;
-   unsigned button;
+   void *context;                /**< Action context (client, etc.). */
 
-   /** Extra data for the action. */
-   union {
-      int i;
-      char *str;
-   } data;
+   /* Extra data for the action. */
+   char *str;
+   unsigned value;
+
+   MenuActionType type;          /**< Type of action. */
 
 } MenuAction;
 
@@ -65,6 +65,7 @@ typedef struct MenuItem {
 
    MenuItemType type;      /**< The menu item type. */
    char *name;             /**< Name to display (or NULL). */
+   char *tooltip;          /**< Tooltip to display (or NULL). */
    MenuAction action;      /**< Action to take if selected (or NULL). */
    char *iconName;         /**< Name of an icon to show (or NULL). */
    struct Menu *submenu;   /**< Submenu (or NULL). */
@@ -99,10 +100,12 @@ typedef struct Menu {
    int *offsets;           /**< y-offsets of menu items. */
    struct Menu *parent;    /**< The parent menu (or NULL). */
    const struct ScreenType *screen;
+   int mousex, mousey;
+   TimeType lastTime;
 
 } Menu;
 
-typedef void (*RunMenuCommandType)(MenuAction *action);
+typedef void (*RunMenuCommandType)(MenuAction *action, unsigned button);
 
 /** Create an empty menu item. */
 MenuItem *CreateMenuItem(MenuItemType type);
@@ -117,8 +120,10 @@ void InitializeMenu(Menu *menu);
  * @param runner Callback executed when an item is selected.
  * @param x The x-coordinate of the menu.
  * @param y The y-coordinate of the menu.
+ * @param keyboard Set if the request came from a key binding.
  */
-void ShowMenu(Menu *menu, RunMenuCommandType runner, int x, int y);
+void ShowMenu(Menu *menu, RunMenuCommandType runner,
+              int x, int y, char keyboard);
 
 /** Destroy a menu structure.
  * @param menu The menu to destroy.
