@@ -39,8 +39,7 @@ static char atTop;
 static ClientNode *currentClient;
 static TimeType moveTime;
 
-static void StopMove(ClientNode *np, int doMove,
-                     int oldx, int oldy, MaxFlags maxFlags);
+static void StopMove(ClientNode *np, int doMove, int oldx, int oldy);
 static void MoveController(int wasDestroyed);
 
 static void DoSnap(ClientNode *np);
@@ -99,7 +98,6 @@ char MoveClient(ClientNode *np, int startx, int starty)
    int doMove;
    int north, south, east, west;
    int height;
-   MaxFlags maxFlags;
 
    Assert(np);
 
@@ -110,7 +108,9 @@ char MoveClient(ClientNode *np, int startx, int starty)
       return 0;
    }
 
-   GrabMouseForMove();
+   if(!GrabMouseForMove()) {
+      return 0;
+   }
 
    RegisterCallback(0, SignalMove, NULL);
    np->controller = MoveController;
@@ -118,10 +118,9 @@ char MoveClient(ClientNode *np, int startx, int starty)
 
    oldx = np->x;
    oldy = np->y;
-   maxFlags = np->state.maxFlags;
 
    if(!(GetMouseMask() & (Button1Mask | Button2Mask))) {
-      StopMove(np, 0, oldx, oldy, maxFlags);
+      StopMove(np, 0, oldx, oldy);
       return 0;
    }
 
@@ -148,7 +147,7 @@ char MoveClient(ClientNode *np, int startx, int starty)
       case ButtonRelease:
          if(event.xbutton.button == Button1
             || event.xbutton.button == Button2) {
-            StopMove(np, doMove, oldx, oldy, maxFlags);
+            StopMove(np, doMove, oldx, oldy);
             return doMove;
          }
          break;
@@ -201,7 +200,11 @@ char MoveClient(ClientNode *np, int startx, int starty)
                MaximizeClient(np, MAX_NONE);
                startx = np->width / 2;
                starty = -north / 2;
-               MoveMouse(np->parent, startx, starty);
+               if(np->parent != None) {
+                  MoveMouse(np->parent, startx, starty);
+               } else {
+                  MoveMouse(np->window, startx, starty);
+               }
             }
 
             CreateMoveWindow(np);
@@ -219,8 +222,12 @@ char MoveClient(ClientNode *np, int startx, int starty)
                DrawOutline(np->x - west, np->y - north,
                            np->width + west + east, height);
             } else {
-               JXMoveWindow(display, np->parent, np->x - west,
-                            np->y - north);
+               if(np->parent != None) {
+                  JXMoveWindow(display, np->parent, np->x - west,
+                               np->y - north);
+               } else {
+                  JXMoveWindow(display, np->window, np->x, np->y);
+               }
                SendConfigureEvent(np);
             }
             UpdateMoveWindow(np);
@@ -243,7 +250,7 @@ char MoveClientKeyboard(ClientNode *np)
    int moved;
    int height;
    int north, south, east, west;
-   MaxFlags maxFlags;
+   Window win;
 
    Assert(np);
 
@@ -254,17 +261,20 @@ char MoveClientKeyboard(ClientNode *np)
       return 0;
    }
 
-   maxFlags = np->state.maxFlags;
    if(np->state.maxFlags != MAX_NONE) {
       MaximizeClient(np, MAX_NONE);
    }
 
-   if(JUNLIKELY(JXGrabKeyboard(display, np->parent, True, GrabModeAsync,
+   win = np->parent != None ? np->parent : np->window;
+   if(JUNLIKELY(JXGrabKeyboard(display, win, True, GrabModeAsync,
                                GrabModeAsync, CurrentTime))) {
       Debug("MoveClient: could not grab keyboard");
       return 0;
    }
-   GrabMouseForMove();
+   if(!GrabMouseForMove()) {
+      JXUngrabKeyboard(display, CurrentTime);
+      return 0;
+   }
 
    GetBorderSize(&np->state, &north, &south, &east, &west);
 
@@ -325,7 +335,7 @@ char MoveClientKeyboard(ClientNode *np)
             }
             break;
          default:
-            StopMove(np, 1, oldx, oldy, maxFlags);
+            StopMove(np, 1, oldx, oldy);
             return 1;
          }
 
@@ -345,7 +355,7 @@ char MoveClientKeyboard(ClientNode *np)
 
       } else if(event.type == ButtonRelease) {
 
-         StopMove(np, 1, oldx, oldy, maxFlags);
+         StopMove(np, 1, oldx, oldy);
          return 1;
 
       }
@@ -357,7 +367,7 @@ char MoveClientKeyboard(ClientNode *np)
             DrawOutline(np->x - west, np->y - west,
                         np->width + west + east, height + north + west);
          } else {
-            JXMoveWindow(display, np->parent, np->x - west, np->y - north);
+            JXMoveWindow(display, win, np->x - west, np->y - north);
             SendConfigureEvent(np);
          }
 
@@ -371,8 +381,7 @@ char MoveClientKeyboard(ClientNode *np)
 }
 
 /** Stop move. */
-void StopMove(ClientNode *np, int doMove,
-              int oldx, int oldy, MaxFlags maxFlags)
+void StopMove(ClientNode *np, int doMove, int oldx, int oldy)
 {
 
    int north, south, east, west;
@@ -388,28 +397,19 @@ void StopMove(ClientNode *np, int doMove,
    UnregisterCallback(SignalMove, NULL);
 
    if(!doMove) {
-
       np->x = oldx;
       np->y = oldy;
-
-      /* Restore maximized status. */
-      if(maxFlags) {
-         MaximizeClient(np, maxFlags);
-      }
       return;
-
    }
 
    GetBorderSize(&np->state, &north, &south, &east, &west);
 
-   JXMoveWindow(display, np->parent, np->x - west, np->y - north);
-   SendConfigureEvent(np);
-
-   /* Restore maximized status. */
-   if(maxFlags) {
-      MaximizeClient(np, maxFlags);
+   if(np->parent != None) {
+      JXMoveWindow(display, np->parent, np->x - west, np->y - north);
+   } else {
+      JXMoveWindow(display, np->window, np->x - west, np->y - north);
    }
-
+   SendConfigureEvent(np);
 }
 
 /** Snap to the screen and/or neighboring windows. */
