@@ -140,7 +140,7 @@ void Create(TrayComponentType *cp)
    Assert(cp->height > 0);
 
    cp->pixmap = JXCreatePixmap(display, rootWindow, cp->width,
-                               cp->height, rootVisual.depth);
+                               cp->height, rootDepth);
    pp->buffer = cp->pixmap;
 
 }
@@ -180,7 +180,7 @@ void SetSize(TrayComponentType *cp, int width, int height)
    if(pp->buffer != None) {
       JXFreePixmap(display, pp->buffer);
       pp->buffer = JXCreatePixmap(display, rootWindow, cp->width,
-                                  cp->height, rootVisual.depth);
+                                  cp->height, rootDepth);
       cp->pixmap = pp->buffer;
       DrawPager(pp);
    }
@@ -355,6 +355,11 @@ ClientFound:
       return;
    }
 
+   /* Start the move. */
+   if(!GrabMouseForMove()) {
+      return;
+   }
+
    /* If the client is maximized, unmaximize it. */
    maxFlags = np->state.maxFlags;
    if(np->state.maxFlags) {
@@ -363,9 +368,6 @@ ClientFound:
 
    GetBorderSize(&np->state, &north, &south, &east, &west);
 
-   /* Start the move. */
-   GrabMouseForMove();
-
    np->controller = PagerMoveController;
    shouldStopMove = 0;
 
@@ -538,7 +540,7 @@ void DrawPager(const PagerType *pp)
             if(textWidth < deskWidth) {
                xc = dx * (deskWidth + 1) + (deskWidth - textWidth) / 2;
                yc = dy * (deskHeight + 1) + (deskHeight - textHeight) / 2;
-               RenderString(&rootVisual, buffer, FONT_PAGER,
+               RenderString(buffer, FONT_PAGER,
                             COLOR_PAGER_TEXT, xc, yc, deskWidth, name);
             }
          }
@@ -553,7 +555,7 @@ void DrawPager(const PagerType *pp)
    }
 
    /* Draw the desktop dividers. */
-   JXSetForeground(display, rootGC, colors[COLOR_PAGER_FG]);
+   JXSetForeground(display, rootGC, colors[COLOR_PAGER_OUTLINE]);
    for(x = 1; x < settings.desktopHeight; x++) {
       JXDrawLine(display, buffer, rootGC,
                  0, (deskHeight + 1) * x - 1,
@@ -602,7 +604,7 @@ void SignalPager(const TimeType *now, int x, int y, Window w, void *data)
          if(desktop >= 0 && desktop < settings.desktopCount) {
             const char *desktopName = GetDesktopName(desktop);
             if(desktopName) {
-               ShowPopup(x, y, desktopName);
+               ShowPopup(x, y, desktopName, POPUP_PAGER);
             }
          }
 
