@@ -33,6 +33,11 @@
 #include "desktop.h"
 #include "border.h"
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/mman.h>
+#include <fcntl.h>
+
 /** Mapping of key names to key types.
  * Note that this mapping must be sorted.
  */
@@ -62,6 +67,7 @@ static const StringMappingType KEY_MAP[] = {
    { "rdesktop",              KEY_RDESKTOP      },
    { "resize",                KEY_RESIZE        },
    { "restart",               KEY_RESTART       },
+   { "restore",               KEY_RESTORE       },
    { "right",                 KEY_RIGHT         },
    { "select",                KEY_ENTER         },
    { "sendd",                 KEY_SENDD         },
@@ -85,9 +91,13 @@ static const StringMappingType OPTION_MAP[] = {
    { "border",             OPTION_BORDER        },
    { "centered",           OPTION_CENTERED      },
    { "constrain",          OPTION_CONSTRAIN     },
+   { "drag",               OPTION_DRAG          },
+   { "fixed",              OPTION_FIXED         },
    { "fullscreen",         OPTION_FULLSCREEN    },
    { "hmax",               OPTION_MAX_H         },
    { "iignore",            OPTION_IIGNORE       },
+   { "ilist",              OPTION_ILIST         },
+   { "ipager",             OPTION_IPAGER        },
    { "maximized",          OPTION_MAXIMIZED     },
    { "minimized",          OPTION_MINIMIZED     },
    { "noborder",           OPTION_NOBORDER      },
@@ -114,6 +124,7 @@ static const unsigned int OPTION_MAP_COUNT = ARRAY_LENGTH(OPTION_MAP);
 static const char *DEFAULT_TITLE = "JWM";
 static const char *LABEL_ATTRIBUTE = "label";
 static const char *ICON_ATTRIBUTE = "icon";
+static const char *TOOLTIP_ATTRIBUTE = "tooltip";
 static const char *CONFIRM_ATTRIBUTE = "confirm";
 static const char *LABELED_ATTRIBUTE = "labeled";
 static const char *ONROOT_ATTRIBUTE = "onroot";
@@ -127,6 +138,8 @@ static const char *TRUE_VALUE = "true";
 
 static char ParseFile(const char *fileName, int depth);
 static char *ReadFile(FILE *fd);
+static TokenNode *TokenizeFile(const char *fileName);
+static TokenNode *TokenizePipe(const char *command);
 
 /* Misc. */
 static void Parse(const TokenNode *start, int depth);
@@ -169,18 +182,14 @@ static void ParseGroupOption(const TokenNode *tp,
 /* Style. */
 static void ParseWindowStyle(const TokenNode *tp);
 static void ParseActiveWindowStyle(const TokenNode *tp);
-static void ParseTaskListStyle(const TokenNode *tp);
-static void ParseActiveTaskListStyle(const TokenNode *tp);
 static void ParseTrayStyle(const TokenNode *tp);
 static void ParseActiveTrayStyle(const TokenNode *tp);
 static void ParsePagerStyle(const TokenNode *tp);
 static void ParseActivePagerStyle(const TokenNode *tp);
+static void ParseClockStyle(const TokenNode *tp);
 static void ParseMenuStyle(const TokenNode *tp);
 static void ParseActiveMenuStyle(const TokenNode *tp);
 static void ParsePopupStyle(const TokenNode *tp);
-static void ParseClockStyle(const TokenNode *tp);
-static void ParseTrayButtonStyle(const TokenNode *tp);
-static void ParseActiveTrayButtonStyle(const TokenNode *tp);
 
 /* Feel. */
 static void ParseKey(const TokenNode *tp);
@@ -189,6 +198,8 @@ static void ParseMoveMode(const TokenNode *tp);
 static void ParseResizeMode(const TokenNode *tp);
 static void ParseFocusModel(const TokenNode *tp);
 
+static AlignmentType ParseTextAlignment(const TokenNode *tp);
+static DecorationsType ParseDecorations(const TokenNode *tp);
 static void ParseGradient(const char *value, ColorType a, ColorType b);
 static char *FindAttribute(AttributeNode *ap, const char *name);
 static int ParseTokenValue(const StringMappingType *mapping, int count,
@@ -208,7 +219,8 @@ void ParseConfig(const char *fileName)
 {
    if(!ParseFile(fileName, 0)) {
       if(JUNLIKELY(!ParseFile(SYSTEM_CONFIG, 0))) {
-         ParseError(NULL, "could not open %s or %s", fileName, SYSTEM_CONFIG);
+         ParseError(NULL, _("could not open %s or %s"),
+                    fileName, SYSTEM_CONFIG);
       }
    }
    ValidateTrayButtons();
@@ -221,32 +233,24 @@ void ParseConfig(const char *fileName)
  */
 char ParseFile(const char *fileName, int depth)
 {
-
    TokenNode *tokens;
-   FILE *fd;
-   char *buffer;
 
    depth += 1;
    if(JUNLIKELY(depth > MAX_INCLUDE_DEPTH)) {
-      ParseError(NULL, "include depth (%d) exceeded", MAX_INCLUDE_DEPTH);
+      ParseError(NULL, _("include depth (%d) exceeded"),
+                 MAX_INCLUDE_DEPTH);
       return 0;
    }
 
-   fd = fopen(fileName, "r");
-   if(!fd) {
+   tokens = TokenizeFile(fileName);
+   if(!tokens) {
       return 0;
    }
 
-   buffer = ReadFile(fd);
-   fclose(fd);
-
-   tokens = Tokenize(buffer, fileName);
-   Release(buffer);
    Parse(tokens, depth);
    ReleaseTokens(tokens);
 
    return 1;
-
 }
 
 /** Parse a token list. */
@@ -328,18 +332,12 @@ void Parse(const TokenNode *start, int depth)
             case TOK_STARTUPCOMMAND:
                AddStartupCommand(tp->value);
                break;
-            case TOK_TASKLISTSTYLE:
-               ParseTaskListStyle(tp);
-               break;
             case TOK_TRAY:
                ParseTray(tp);
                break;
             case TOK_TRAYSTYLE:
                ParseTrayStyle(tp);
                break;
-            case TOK_TRAYBUTTONSTYLE:
-               ParseTrayButtonStyle(tp);
-               break;
             case TOK_CLOCKSTYLE:
                ParseClockStyle(tp);
                break;
@@ -358,6 +356,9 @@ void Parse(const TokenNode *start, int depth)
             case TOK_BUTTONMIN:
                SetBorderIcon(BI_MIN, tp->value);
                break;
+            case TOK_BUTTONMENU:
+               SetBorderIcon(BI_MENU, tp->value);
+               break;
             default:
                InvalidTag(tp, TOK_JWM);
                break;
@@ -365,7 +366,7 @@ void Parse(const TokenNode *start, int depth)
          }
       }
    } else {
-      ParseError(start, "invalid start tag: %s", GetTokenName(start));
+      ParseError(start, _("invalid start tag: %s"), GetTokenName(start));
    }
 
 }
@@ -501,6 +502,33 @@ MenuItem *ParseMenuItem(const TokenNode *start, Menu *menu, MenuItem *last)
    menu->offsets = NULL;
    while(start) {
       switch(start->type) {
+      case TOK_DYNAMIC:
+
+         last = InsertMenuItem(last);
+         if(!menu->items) {
+            menu->items = last;
+         }
+
+         value = FindAttribute(start->attributes, LABEL_ATTRIBUTE);
+         last->name = CopyString(value);
+
+         value = FindAttribute(start->attributes, ICON_ATTRIBUTE);
+         last->iconName = CopyString(value);
+
+         value = FindAttribute(start->attributes, TOOLTIP_ATTRIBUTE);
+         last->tooltip = CopyString(value);
+
+         last->action.type = MA_DYNAMIC;
+         last->action.str = CopyString(start->value);
+
+         value = FindAttribute(start->attributes, HEIGHT_ATTRIBUTE);
+         if(value) {
+            last->action.value = ParseUnsigned(start, value);
+         } else {
+            last->action.value = menu->itemHeight;
+         }
+
+         break;
       case TOK_MENU:
 
          last = InsertMenuItem(last);
@@ -515,6 +543,9 @@ MenuItem *ParseMenuItem(const TokenNode *start, Menu *menu, MenuItem *last)
          value = FindAttribute(start->attributes, ICON_ATTRIBUTE);
          last->iconName = CopyString(value);
 
+         value = FindAttribute(start->attributes, TOOLTIP_ATTRIBUTE);
+         last->tooltip = CopyString(value);
+
          last->submenu = Allocate(sizeof(Menu));
          child = last->submenu;
 
@@ -554,11 +585,14 @@ MenuItem *ParseMenuItem(const TokenNode *start, Menu *menu, MenuItem *last)
             last->name = CopyString(start->value);
          }
 
+         value = FindAttribute(start->attributes, TOOLTIP_ATTRIBUTE);
+         last->tooltip = CopyString(value);
+
          value = FindAttribute(start->attributes, ICON_ATTRIBUTE);
          last->iconName = CopyString(value);
 
          last->action.type = MA_EXECUTE;
-         last->action.data.str = CopyString(start->value);
+         last->action.str = CopyString(start->value);
 
          break;
       case TOK_SEPARATOR:
@@ -583,7 +617,6 @@ MenuItem *ParseMenuItem(const TokenNode *start, Menu *menu, MenuItem *last)
       case TOK_KILL:
       case TOK_CLOSE:
       case TOK_SENDTO:
-      case TOK_DYNAMIC:
 
          last = InsertMenuItem(last);
          if(!menu->items) {
@@ -596,6 +629,9 @@ MenuItem *ParseMenuItem(const TokenNode *start, Menu *menu, MenuItem *last)
          }
          last->name = CopyString(value);
 
+         value = FindAttribute(start->attributes, TOOLTIP_ATTRIBUTE);
+         last->tooltip = CopyString(value);
+
          value = FindAttribute(start->attributes, ICON_ATTRIBUTE);
          last->iconName = CopyString(value);
 
@@ -630,10 +666,6 @@ MenuItem *ParseMenuItem(const TokenNode *start, Menu *menu, MenuItem *last)
          case TOK_SENDTO:
             last->action.type = MA_SENDTO_MENU;
             break;
-         case TOK_DYNAMIC:
-            last->action.type = MA_DYNAMIC;
-            last->action.data.str = CopyString(start->value);
-            break;
          default:
             break;
          }
@@ -646,24 +678,26 @@ MenuItem *ParseMenuItem(const TokenNode *start, Menu *menu, MenuItem *last)
             menu->items = last;
          }
 
-         value = FindAttribute(start->attributes, CONFIRM_ATTRIBUTE);
-         if(value && !strcmp(value, FALSE_VALUE)) {
-            settings.exitConfirmation = 0;
-         } else {
-            settings.exitConfirmation = 1;
-         }
-
          value = FindAttribute(start->attributes, LABEL_ATTRIBUTE);
          if(!value) {
             value = GetTokenName(start);
          }
          last->name = CopyString(value);
 
+         value = FindAttribute(start->attributes, TOOLTIP_ATTRIBUTE);
+         last->tooltip = CopyString(value);
+
          value = FindAttribute(start->attributes, ICON_ATTRIBUTE);
          last->iconName = CopyString(value);
 
          last->action.type = MA_EXIT;
-         last->action.data.str = CopyString(start->value);
+         last->action.str = CopyString(start->value);
+         value = FindAttribute(start->attributes, CONFIRM_ATTRIBUTE);
+         if(value && !strcmp(value, FALSE_VALUE)) {
+            last->action.value = 0;
+         } else {
+            last->action.value = 1;
+         }
 
          break;
       case TOK_RESTART:
@@ -679,6 +713,9 @@ MenuItem *ParseMenuItem(const TokenNode *start, Menu *menu, MenuItem *last)
          }
          last->name = CopyString(value);
 
+         value = FindAttribute(start->attributes, TOOLTIP_ATTRIBUTE);
+         last->tooltip = CopyString(value);
+
          value = FindAttribute(start->attributes, ICON_ATTRIBUTE);
          last->iconName = CopyString(value);
 
@@ -699,53 +736,17 @@ MenuItem *ParseMenuItem(const TokenNode *start, Menu *menu, MenuItem *last)
 /** Get tokens from a menu include (either dynamic or static). */
 TokenNode *ParseMenuIncludeHelper(const TokenNode *tp, const char *command)
 {
-   FILE *fd;
-   char *path;
-   char *buffer;
    TokenNode *start;
 
-   buffer = NULL;
    if(!strncmp(command, "exec:", 5)) {
-
-      path = Allocate(strlen(command) - 5 + 1);
-      strcpy(path, command + 5);
-      ExpandPath(&path);
-
-      fd = popen(path, "r");
-      if(JLIKELY(fd)) {
-         buffer = ReadFile(fd);
-         pclose(fd);
-      } else {
-         ParseError(tp, "could not execute included program: %s", path);
-      }
-
+      start = TokenizePipe(&command[5]);
    } else {
-
-      path = CopyString(command);
-      ExpandPath(&path);
-
-      fd = fopen(path, "r");
-      if(JLIKELY(fd)) {
-         buffer = ReadFile(fd);
-         fclose(fd);
-      } else {
-         ParseError(NULL, "could not open include: %s", path);
-      }
-
+      start = TokenizeFile(command);
    }
 
-   if(JUNLIKELY(!buffer)) {
-      Release(path);
-      return NULL;
-   }
-
-   start = Tokenize(buffer, path);
-   Release(buffer);
-   Release(path);
-
    if(JUNLIKELY(!start || start->type != TOK_JWM))
    {
-      ParseError(tp, "invalid include: %s", command);
+      ParseError(tp, _("invalid include: %s"), command);
       ReleaseTokens(start);
       return NULL;
    }
@@ -795,7 +796,7 @@ void ParseKey(const TokenNode *tp) {
 
    action = tp->value;
    if(JUNLIKELY(action == NULL)) {
-      ParseError(tp, "no action specified for Key");
+      ParseError(tp, _("no action specified for Key"));
       return;
    }
 
@@ -818,7 +819,7 @@ void ParseKey(const TokenNode *tp) {
    /* Insert the binding if it's valid. */
    if(JUNLIKELY(k == KEY_NONE)) {
 
-      ParseError(tp, "invalid Key action: \"%s\"", action);
+      ParseError(tp, _("invalid Key action: \"%s\""), action);
 
    } else {
 
@@ -828,29 +829,38 @@ void ParseKey(const TokenNode *tp) {
 
 }
 
+/** Parse text alignment. */
+AlignmentType ParseTextAlignment(const TokenNode *tp)
+{
+   static const StringMappingType mapping[] = {
+      {"left",    ALIGN_LEFT   },
+      {"center",  ALIGN_CENTER },
+      {"right",   ALIGN_RIGHT  }
+   };
+   const char *attr= FindAttribute(tp->attributes, "align");
+   if(attr) {
+      const int x = FindValue(mapping, ARRAY_LENGTH(mapping), attr);
+      if(JLIKELY(x >= 0)) {
+         return x;
+      } else {
+         Warning(_("invalid text alignment: \"%s\""), attr);
+      }
+   }
+
+   return ALIGN_LEFT;
+}
+
 /** Parse window style. */
 void ParseWindowStyle(const TokenNode *tp)
 {
    const TokenNode *np;
-   const char *decorations;
-
-   decorations = FindAttribute(tp->attributes, "decorations");
-   if(decorations) {
-      if(!strcmp(decorations, "motif")) {
-         settings.handles = 1;
-      } else if(!strcmp(decorations, "flat")) {
-         settings.handles = 0;
-      } else {
-         ParseError(tp, "invalid WindowStyle decorations: %s\n", decorations);
-      }
-   } else {
-      settings.handles = 0;
-   }
 
+   settings.windowDecorations = ParseDecorations(tp);
    for(np = tp->subnodeHead; np; np = np->next) {
       switch(np->type) {
       case TOK_FONT:
          SetFont(FONT_BORDER, np->value);
+         settings.titleTextAlignment = ParseTextAlignment(np);
          break;
       case TOK_WIDTH:
          settings.borderWidth = ParseUnsigned(np, np->value);
@@ -871,7 +881,7 @@ void ParseWindowStyle(const TokenNode *tp)
          ParseGradient(np->value, COLOR_TITLE_BG1, COLOR_TITLE_BG2);
          break;
       case TOK_OUTLINE:
-         SetColor(COLOR_BORDER_LINE, np->value);
+         ParseGradient(np->value, COLOR_TITLE_DOWN, COLOR_TITLE_UP);
          break;
       case TOK_OPACITY:
          settings.inactiveClientOpacity = ParseOpacity(tp, np->value);
@@ -901,7 +911,8 @@ void ParseActiveWindowStyle(const TokenNode *tp)
             COLOR_TITLE_ACTIVE_BG1, COLOR_TITLE_ACTIVE_BG2);
          break;
       case TOK_OUTLINE:
-         SetColor(COLOR_BORDER_ACTIVE_LINE, np->value);
+         ParseGradient(np->value, COLOR_TITLE_ACTIVE_DOWN,
+                       COLOR_TITLE_ACTIVE_UP);
          break;
       case TOK_OPACITY:
          settings.activeClientOpacity = ParseOpacity(np, np->value);
@@ -915,28 +926,25 @@ void ParseActiveWindowStyle(const TokenNode *tp)
 }
 
 /** Parse an include. */
-void ParseInclude(const TokenNode *tp, int depth) {
-
-   char *temp;
-
-   Assert(tp);
-
+void ParseInclude(const TokenNode *tp, int depth)
+{
    if(JUNLIKELY(!tp->value)) {
+      ParseError(tp, _("no include file specified"));
+      return;
+   }
 
-      ParseError(tp, "no include file specified");
-
+   if(!strncmp(tp->value, "exec:", 5)) {
+      TokenNode *tokens = TokenizePipe(&tp->value[5]);
+      if(JLIKELY(tokens)) {
+         Parse(tokens, 0);
+         ReleaseTokens(tokens);
+      } else {
+         ParseError(tp, _("could not process include: %s"), &tp->value[5]);
+      }
    } else {
-
-      temp = CopyString(tp->value);
-
-      ExpandPath(&temp);
-
-      if(JUNLIKELY(!ParseFile(temp, depth))) {
-         ParseError(tp, "could not open included file %s", temp);
+      if(JUNLIKELY(!ParseFile(tp->value, depth))) {
+         ParseError(tp, _("could not open included file: %s"), tp->value);
       }
-
-      Release(temp);
-
    }
 
 }
@@ -1013,70 +1021,23 @@ void ParseDesktopBackground(int desktop, const TokenNode *tp)
    SetBackground(desktop, type, tp->value);
 }
 
-/** Parse task list style. */
-void ParseTaskListStyle(const TokenNode *tp)
-{
-   static const StringMappingType mapping[] = {
-      { "left",   INSERT_LEFT    },
-      { "right",  INSERT_RIGHT   }
-   };
-   TokenNode *np;
-
-   settings.taskInsertMode = ParseAttribute(mapping, ARRAY_LENGTH(mapping),
-      tp, "insert", settings.taskInsertMode);
-
-   for(np = tp->subnodeHead; np; np = np->next) {
-      switch(np->type) {
-      case TOK_FONT:
-         SetFont(FONT_TASK, np->value);
-         break;
-      case TOK_ACTIVE:
-         ParseActiveTaskListStyle(np);
-         break;
-      case TOK_FOREGROUND:
-         SetColor(COLOR_TASK_FG, np->value);
-         break;
-      case TOK_BACKGROUND:
-         ParseGradient(np->value, COLOR_TASK_BG1, COLOR_TASK_BG2);
-         break;
-      default:
-         InvalidTag(np, TOK_TASKLISTSTYLE);
-         break;
-      }
-   }
-
-}
-
-/** Parse active task list style. */
-void ParseActiveTaskListStyle(const TokenNode *tp)
-{
-   TokenNode *np;
-   for(np = tp->subnodeHead; np; np = np->next) {
-      switch(np->type) {
-      case TOK_FOREGROUND:
-         SetColor(COLOR_TASK_ACTIVE_FG, np->value);
-         break;
-      case TOK_BACKGROUND:
-         ParseGradient(np->value, COLOR_TASK_ACTIVE_BG1, COLOR_TASK_ACTIVE_BG2);
-         break;
-      default:
-         InvalidTag(np, TOK_ACTIVE);
-         break;
-      }
-   }
-}
-
 /** Parse tray style. */
 void ParseTrayStyle(const TokenNode *tp)
 {
    const TokenNode *np;
    const char *temp;
 
+   settings.trayDecorations = ParseDecorations(tp);
    temp = FindAttribute(tp->attributes, "group");
    if(temp) {
       settings.groupTasks = !strcmp(temp, TRUE_VALUE);
    }
 
+   temp = FindAttribute(tp->attributes, "list");
+   if(temp) {
+      settings.listAllTasks = !strcmp(temp, "all");
+   }
+
    for(np = tp->subnodeHead; np; np = np->next) {
       switch(np->type) {
       case TOK_FONT:
@@ -1091,12 +1052,12 @@ void ParseTrayStyle(const TokenNode *tp)
       case TOK_FOREGROUND:
          SetColor(COLOR_TRAY_FG, np->value);
          break;
+      case TOK_OUTLINE:
+         ParseGradient(np->value, COLOR_TRAY_DOWN, COLOR_TRAY_UP);
+         break;
       case TOK_OPACITY:
          settings.trayOpacity = ParseOpacity(np, np->value);
          break;
-      case TOK_OUTLINE:
-         SetColor(COLOR_TRAY_OUTLINE, np->value);
-         break;
       default:
          InvalidTag(np, TOK_TRAYSTYLE);
          break;
@@ -1251,6 +1212,10 @@ void ParseTaskList(const TokenNode *tp, TrayType *tray)
       SetMaxTaskBarItemWidth(cp, temp);
    }
 
+   temp = FindAttribute(tp->attributes, "height");
+   if(temp) {
+      SetTaskBarHeight(cp, temp);
+   }
 }
 
 /** Parse a swallow tray component. */
@@ -1518,20 +1483,58 @@ void ParseActivePagerStyle(const TokenNode *tp)
    }
 }
 
+/** Parse clock style. */
+void ParseClockStyle(const TokenNode *tp)
+{
+   const TokenNode *np;
+   for(np = tp->subnodeHead; np; np = np->next) {
+      switch(np->type) {
+      case TOK_FOREGROUND:
+         SetColor(COLOR_CLOCK_FG, np->value);
+         break;
+      case TOK_BACKGROUND:
+         ParseGradient(np->value, COLOR_CLOCK_BG1, COLOR_CLOCK_BG2);
+         break;
+      case TOK_FONT:
+         SetFont(FONT_CLOCK, np->value);
+         break;
+      default:
+         InvalidTag(np, TOK_CLOCKSTYLE);
+         break;
+      }
+   }
+}
+
 /** Parse popup style. */
 void ParsePopupStyle(const TokenNode *tp)
 {
-
+   static const StringMappingType enable_mapping[] = {
+      { "button", POPUP_BUTTON   },
+      { "clock",  POPUP_CLOCK    },
+      { "false",  POPUP_NONE     },
+      { "menu",   POPUP_MENU     },
+      { "pager",  POPUP_PAGER    },
+      { "task",   POPUP_TASK     },
+      { "true",   POPUP_ALL      }
+   };
    const TokenNode *np;
    const char *str;
+   char *tok;
 
-   Assert(tp);
-
-   str = FindAttribute(tp->attributes, "enabled");
-   if(str && !strcmp(str, FALSE_VALUE)) {
-      settings.popupEnabled = 0;
-   } else {
-      settings.popupEnabled = 1;
+   tok = FindAttribute(tp->attributes, "enabled");
+   if(tok) {
+      settings.popupMask = POPUP_NONE;
+      tok = strtok(tok, ",");
+      while(tok) {
+         const int x = FindValue(enable_mapping,
+                                 ARRAY_LENGTH(enable_mapping), tok);
+         if(JLIKELY(x >= 0)) {
+            settings.popupMask |= x;
+         } else {
+            ParseError(tp, _("invalid value for 'enabled': \"%s\""), tok);
+         }
+         tok = strtok(NULL, ",");
+      }
    }
 
    str = FindAttribute(tp->attributes, "delay");
@@ -1544,15 +1547,15 @@ void ParsePopupStyle(const TokenNode *tp)
       case TOK_FONT:
          SetFont(FONT_POPUP, np->value);
          break;
-      case TOK_OUTLINE:
-         SetColor(COLOR_POPUP_OUTLINE, np->value);
-         break;
       case TOK_FOREGROUND:
          SetColor(COLOR_POPUP_FG, np->value);
          break;
       case TOK_BACKGROUND:
          SetColor(COLOR_POPUP_BG, np->value);
          break;
+      case TOK_OUTLINE:
+         SetColor(COLOR_POPUP_OUTLINE, np->value);
+         break;
       default:
          InvalidTag(np, TOK_POPUPSTYLE);
          break;
@@ -1564,11 +1567,9 @@ void ParsePopupStyle(const TokenNode *tp)
 /** Parse menu style. */
 void ParseMenuStyle(const TokenNode *tp)
 {
-
    const TokenNode *np;
 
-   Assert(tp);
-
+   settings.menuDecorations = ParseDecorations(tp);
    for(np = tp->subnodeHead; np; np = np->next) {
       switch(np->type) {
       case TOK_FONT:
@@ -1583,12 +1584,12 @@ void ParseMenuStyle(const TokenNode *tp)
       case TOK_ACTIVE:
          ParseActiveMenuStyle(np);
          break;
+      case TOK_OUTLINE:
+         ParseGradient(np->value, COLOR_MENU_DOWN, COLOR_MENU_UP);
+         break;
       case TOK_OPACITY:
          settings.menuOpacity = ParseOpacity(np, np->value);
          break;
-      case TOK_OUTLINE:
-         SetColor(COLOR_MENU_OUTLINE, np->value);
-         break;
       default:
          InvalidTag(np, TOK_MENUSTYLE);
          break;
@@ -1616,83 +1617,6 @@ void ParseActiveMenuStyle(const TokenNode *tp)
    }
 }
 
-/** Parse clock style. */
-void ParseClockStyle(const TokenNode *tp)
-{
-
-   const TokenNode *np;
-
-   Assert(tp);
-
-   for(np = tp->subnodeHead; np; np = np->next) {
-      switch(np->type) {
-      case TOK_FONT:
-         SetFont(FONT_CLOCK, np->value);
-         break;
-      case TOK_FOREGROUND:
-         SetColor(COLOR_CLOCK_FG, np->value);
-         break;
-      case TOK_BACKGROUND:
-         ParseGradient(np->value, COLOR_CLOCK_BG1, COLOR_CLOCK_BG2);
-         break;
-      default:
-         InvalidTag(np, TOK_CLOCKSTYLE);
-         break;
-      }
-   }
-
-}
-
-/** Parse tray button style. */
-void ParseTrayButtonStyle(const TokenNode *tp)
-{
-
-   const TokenNode *np;
-
-   Assert(tp);
-
-   for(np = tp->subnodeHead; np; np = np->next) {
-      switch(np->type) {
-      case TOK_FONT:
-         SetFont(FONT_TRAYBUTTON, np->value);
-         break;
-      case TOK_FOREGROUND:
-         SetColor(COLOR_TRAYBUTTON_FG, np->value);
-         break;
-      case TOK_BACKGROUND:
-         ParseGradient(np->value, COLOR_TRAYBUTTON_BG1, COLOR_TRAYBUTTON_BG2);
-         break;
-      case TOK_ACTIVE:
-         ParseActiveTrayButtonStyle(np);
-         break;
-      default:
-         InvalidTag(np, TOK_TRAYBUTTONSTYLE);
-         break;
-      }
-   }
-
-}
-
-/** Parse active tray button style. */
-void ParseActiveTrayButtonStyle(const TokenNode *tp)
-{
-   const TokenNode *np;
-   for(np = tp->subnodeHead; np; np = np->next) {
-      switch(np->type) {
-      case TOK_FOREGROUND:
-         SetColor(COLOR_TRAYBUTTON_ACTIVE_FG, np->value);
-         break;
-      case TOK_BACKGROUND:
-         ParseGradient(np->value, COLOR_TRAYBUTTON_ACTIVE_BG1,
-                       COLOR_TRAYBUTTON_ACTIVE_BG2);
-         break;
-      default:
-         InvalidTag(np, TOK_INVALID);
-         break;
-      }
-   }
-}
-
 /** Parse an option group. */
 void ParseGroup(const TokenNode *tp)
 {
@@ -1753,11 +1677,27 @@ void ParseGroupOption(const TokenNode *tp, struct GroupType *group,
       const unsigned int opacity = ParseOpacity(tp, option + 8);
       AddGroupOptionUnsigned(group, OPTION_OPACITY, opacity);
    } else {
-      ParseError(tp, "invalid Group Option: %s", option);
+      ParseError(tp, _("invalid Group Option: %s"), option);
    }
 
 }
 
+/** Parse decorations type. */
+DecorationsType ParseDecorations(const TokenNode *tp)
+{
+   const char *str = FindAttribute(tp->attributes, "decorations");
+   if(str) {
+      if(!strcmp(str, "motif")) {
+         return DECO_MOTIF;
+      } else if(!strcmp(str, "flat")) {
+         return DECO_FLAT;
+      } else {
+         ParseError(tp, _("invalid decorations: %s"), str);
+      }
+   }
+   return DECO_FLAT;
+}
+
 /** Parse a color which may be a gradient. */
 void ParseGradient(const char *value, ColorType a, ColorType b)
 {
@@ -1815,14 +1755,14 @@ int ParseTokenValue(const StringMappingType *mapping, int count,
                     const TokenNode *tp, int def)
 {
    if(JUNLIKELY(tp->value == NULL)) {
-      ParseError(tp, "%s is empty", GetTokenName(tp));
+      ParseError(tp, _("%s is empty"), GetTokenName(tp));
       return def;
    } else {
       const int x = FindValue(mapping, count, tp->value);
       if(JLIKELY(x >= 0)) {
          return x;
       } else {
-         ParseError(tp, "invalid %s: \"%s\"", GetTokenName(tp), tp->value);
+         ParseError(tp, _("invalid %s: \"%s\""), GetTokenName(tp), tp->value);
          return def;
       }
    }
@@ -1840,7 +1780,7 @@ int ParseAttribute(const StringMappingType *mapping, int count,
       if(JLIKELY(x >= 0)) {
          return x;
       } else {
-         ParseError(tp, "invalid value for %s: \"%s\"", attr, str);
+         ParseError(tp, _("invalid value for %s: \"%s\""), attr, str);
          return def;
       }
    }
@@ -1849,8 +1789,7 @@ int ParseAttribute(const StringMappingType *mapping, int count,
 /** Read a file. */
 char *ReadFile(FILE *fd)
 {
-
-   const int BLOCK_SIZE = 1024;  // Start at 1k.
+   const int BLOCK_SIZE = 1024;
 
    char *buffer;
    int len, max;
@@ -1865,7 +1804,7 @@ char *ReadFile(FILE *fd)
       if(len < max) {
          break;
       }
-      max *= 2;
+      max += BLOCK_SIZE;
       if(JUNLIKELY(max < 0)) {
          /* File is too big. */
          break;
@@ -1878,13 +1817,77 @@ char *ReadFile(FILE *fd)
    buffer[len] = 0;
 
    return buffer;
+}
 
+/** Tokenize a file by memory mapping it. */
+TokenNode *TokenizeFile(const char *fileName)
+{
+   struct stat sbuf;
+   TokenNode *tokens;
+   char *path;
+   char *buffer;
+
+   path = CopyString(fileName);
+   ExpandPath(&path);
+
+   int fd = open(path, O_RDONLY);
+   Release(path);
+
+   if(fd < 0) {
+      return NULL;
+   }
+   if(JUNLIKELY(fstat(fd, &sbuf) == -1)) {
+      close(fd);
+      return NULL;
+   }
+   buffer = mmap(NULL, sbuf.st_size, PROT_READ, MAP_SHARED, fd, 0);
+   if(JUNLIKELY(buffer == MAP_FAILED)) {
+      close(fd);
+      return NULL;
+   }
+   tokens = Tokenize(buffer, fileName);
+   munmap(buffer, sbuf.st_size);
+   close(fd);
+   return tokens;
+}
+
+/** Tokenize the output of a command. */
+TokenNode *TokenizePipe(const char *command)
+{
+   TokenNode *tokens;
+   FILE *fp;
+   char *path;
+   char *buffer;
+
+   path = CopyString(command);
+   ExpandPath(&path);
+
+   fp = popen(path, "r");
+   Release(path);
+
+   buffer = NULL;
+   if(JLIKELY(fp)) {
+      buffer = ReadFile(fp);
+      pclose(fp);
+   }
+   if(JUNLIKELY(!buffer)) {
+      return NULL;
+   }
+
+   tokens = Tokenize(buffer, command);
+   Release(buffer);
+   return tokens;
 }
 
 /** Parse an unsigned integer. */
 unsigned int ParseUnsigned(const TokenNode *tp, const char *str)
 {
-   const long value = strtol(str, NULL, 0);
+   long value;
+   if(JUNLIKELY(!str)) {
+      ParseError(tp, _("no value specified"));
+      return 0;
+   }
+   value = strtol(str, NULL, 0);
    if(JUNLIKELY(value < 0 || value > UINT_MAX)) {
       ParseError(tp, _("invalid setting: %s"), str);
       return 0;
@@ -1896,7 +1899,12 @@ unsigned int ParseUnsigned(const TokenNode *tp, const char *str)
 /** Parse opacity (a float between 0.0 and 1.0). */
 unsigned int ParseOpacity(const TokenNode *tp, const char *str)
 {
-   const float value = ParseFloat(str);
+   float value;
+   if(JUNLIKELY(!str)) {
+      ParseError(tp, _("no value specified"));
+      return UINT_MAX;
+   }
+   value = ParseFloat(str);
    if(JUNLIKELY(value <= 0.0 || value > 1.0)) {
       ParseError(tp, _("invalid opacity: %s"), str);
       return UINT_MAX;
@@ -1919,7 +1927,7 @@ WinLayerType ParseLayer(const TokenNode *tp, const char *str)
    if(JLIKELY(x >= 0)) {
       return x;
    }  else {
-      ParseError(tp, "invalid layer: %s", str);
+      ParseError(tp, _("invalid layer: %s"), str);
       return LAYER_NORMAL;
    }
 }
@@ -1950,15 +1958,16 @@ void ParseError(const TokenNode *tp, const char *str, ...)
 
    va_list ap;
 
-   static const char *FILE_MESSAGE = "%s[%u]";
+   static const char FILE_MESSAGE[] = "%s[%u]";
 
    char *msg;
 
    va_start(ap, str);
 
    if(tp) {
-      msg = Allocate(strlen(FILE_MESSAGE) + strlen(tp->fileName) + 1);
-      sprintf(msg, FILE_MESSAGE, tp->fileName, tp->line);
+      const size_t msg_len = sizeof(FILE_MESSAGE) + strlen(tp->fileName) + 1;
+      msg = Allocate(msg_len);
+      snprintf(msg, msg_len, FILE_MESSAGE, tp->fileName, tp->line);
    } else {
       msg = CopyString(_("configuration error"));
    }
