@@ -32,8 +32,8 @@ static char DoRemoveClientStrut(ClientNode *np);
 static void InsertStrut(const BoundingBox *box, ClientNode *np);
 static void CenterClient(const BoundingBox *box, ClientNode *np);
 static int IntComparator(const void *a, const void *b);
-static char TryTileClient(const BoundingBox *box, ClientNode *np,
-                          int x, int y);
+static int TryTileClient(const BoundingBox *box, ClientNode *np,
+                         int x, int y);
 static char TileClient(const BoundingBox *box, ClientNode *np);
 static void CascadeClient(const BoundingBox *box, ClientNode *np);
 
@@ -142,12 +142,13 @@ void ReadClientStrut(ClientNode *np)
     *   left_start_y, left_end_y, right_start_y, right_end_y,
     *   top_start_x, top_end_x, bottom_start_x, bottom_end_x
     */
+   count = 0;
    status = JXGetWindowProperty(display, np->window,
                                 atoms[ATOM_NET_WM_STRUT_PARTIAL],
                                 0, 12, False, XA_CARDINAL, &actualType,
                                 &actualFormat, &count, &bytesLeft, &value);
    if(status == Success && actualFormat != 0) {
-      if(count == 12) {
+      if(JLIKELY(count == 12)) {
 
          long leftStart, leftStop;
          long rightStart, rightStop;
@@ -208,11 +209,12 @@ void ReadClientStrut(ClientNode *np)
 
    /* Next try to read _NET_WM_STRUT */
    /* Format is: left_width, right_width, top_width, bottom_width */
+   count = 0;
    status = JXGetWindowProperty(display, np->window, atoms[ATOM_NET_WM_STRUT],
                                 0, 4, False, XA_CARDINAL, &actualType,
                                 &actualFormat, &count, &bytesLeft, &value);
    if(status == Success && actualFormat != 0) {
-      if(count == 4) {
+      if(JLIKELY(count == 4)) {
          lvalue = (long*)value;
          leftWidth = lvalue[0];
          rightWidth = lvalue[1];
@@ -350,8 +352,8 @@ void SubtractTrayBounds(const TrayType *tp, BoundingBox *box,
 
          src.x = tp->x;
          src.y = tp->y;
-         src.width = tp->width + 2 * TRAY_BORDER_SIZE;
-         src.height = tp->height + 2 * TRAY_BORDER_SIZE;
+         src.width = tp->width;
+         src.height = tp->height;
          if(src.x < 0) {
             src.width += src.x;
             src.x = 0;
@@ -384,8 +386,7 @@ void SubtractStrutBounds(BoundingBox *box, const ClientNode *np)
       if(np != NULL && sp->client == np) {
          continue;
       }
-      if(sp->client->state.desktop == currentDesktop
-         || (sp->client->state.status & STAT_STICKY)) {
+      if(IsClientOnCurrentDesktop(sp->client)) {
          last = *box;
          SubtractBounds(&sp->box, box);
          if(box->width * box->height <= 0) {
@@ -409,19 +410,20 @@ void CenterClient(const BoundingBox *box, ClientNode *np)
 /** Compare two integers. */
 int IntComparator(const void *a, const void *b)
 {
-   const int ia = *(const long*)a;
-   const int ib = *(const long*)b;
+   const int ia = *(const int*)a;
+   const int ib = *(const int*)b;
    return ia - ib;
 }
 
 /** Attempt to place the client at the specified coordinates. */
-char TryTileClient(const BoundingBox *box, ClientNode *np, int x, int y)
+int TryTileClient(const BoundingBox *box, ClientNode *np, int x, int y)
 {
    const ClientNode *tp;
    int layer;
    int north, south, east, west;
    int x1, x2, y1, y2;
    int ox1, ox2, oy1, oy2;
+   int overlap;
 
    /* Set the client position. */
    GetBorderSize(&np->state, &north, &south, &east, &west);
@@ -437,14 +439,13 @@ char TryTileClient(const BoundingBox *box, ClientNode *np, int x, int y)
    y2 = np->y + np->height + south;
 
    /* Loop over each client. */
+   overlap = 0;
    for(layer = np->state.layer; layer < LAYER_COUNT; layer++) {
       for(tp = nodes[layer]; tp; tp = tp->next) {
 
          /* Skip clients that aren't visible. */
-         if(tp->state.desktop != currentDesktop) {
-            if(!(tp->state.status & STAT_STICKY)) {
-               continue;
-            }
+         if(!IsClientOnCurrentDesktop(tp)) {
+            continue;
          }
          if(!(tp->state.status & STAT_MAPPED)) {
             continue;
@@ -467,14 +468,12 @@ char TryTileClient(const BoundingBox *box, ClientNode *np, int x, int y)
          if(y2 <= oy1 || y1 >= oy2) {
             continue;
          }
-         return 0;
-
+         overlap += (Min(ox2, x2) - Max(ox1, x1))
+                  * (Min(oy2, y2) - Max(oy1, y1));
       }
    }
 
-   /* No client overlaps this position. */
-   return 1;
-
+   return overlap;
 }
 
 /** Tiled placement. */
@@ -488,15 +487,15 @@ char TileClient(const BoundingBox *box, ClientNode *np)
    int count;
    int *xs;
    int *ys;
+   int leastOverlap;
+   int bestx, besty;
 
    /* Determine how much space to allocate. */
    count = 1;
    for(layer = np->state.layer; layer < LAYER_COUNT; layer++) {
       for(tp = nodes[layer]; tp; tp = tp->next) {
-         if(tp->state.desktop != currentDesktop) {
-            if(!(tp->state.status & STAT_STICKY)) {
-               continue;
-            }
+         if(!IsClientOnCurrentDesktop(tp)) {
+            continue;
          }
          if(!(tp->state.status & STAT_MAPPED)) {
             continue;
@@ -518,10 +517,8 @@ char TileClient(const BoundingBox *box, ClientNode *np)
    count = 1;
    for(layer = np->state.layer; layer < LAYER_COUNT; layer++) {
       for(tp = nodes[layer]; tp; tp = tp->next) {
-         if(tp->state.desktop != currentDesktop) {
-            if(!(tp->state.status & STAT_STICKY)) {
-               continue;
-            }
+         if(!IsClientOnCurrentDesktop(tp)) {
+            continue;
          }
          if(!(tp->state.status & STAT_MAPPED)) {
             continue;
@@ -543,12 +540,19 @@ char TileClient(const BoundingBox *box, ClientNode *np)
    qsort(ys, count, sizeof(int), IntComparator);
 
    /* Try all possible positions. */
+   leastOverlap = INT_MAX;
+   bestx = xs[0];
+   besty = ys[0];
    for(i = 0; i < count; i++) {
       for(j = 0; j < count; j++) {
-         if(TryTileClient(box, np, xs[i], ys[j])) {
-            ReleaseStack(xs);
-            ReleaseStack(ys);
-            return 1;
+         const int overlap = TryTileClient(box, np, xs[i], ys[j]);
+         if(overlap < leastOverlap) {
+            leastOverlap = overlap;
+            bestx = xs[i];
+            besty = ys[j];
+            if(overlap == 0) {
+               break;
+            }
          }
       }
    }
@@ -556,9 +560,18 @@ char TileClient(const BoundingBox *box, ClientNode *np)
    ReleaseStack(xs);
    ReleaseStack(ys);
 
-   /* Tiled placement failed. */
-   return 0;
-
+   if(leastOverlap < INT_MAX) {
+      /* Set the client position. */
+      GetBorderSize(&np->state, &north, &south, &east, &west);
+      np->x = bestx + west;
+      np->y = besty + north;
+      ConstrainSize(np);
+      ConstrainPosition(np);
+      return 1;
+   } else {
+      /* Tiled placement failed. */
+      return 0;
+   }
 }
 
 /** Cascade placement. */
@@ -822,19 +835,19 @@ void PlaceMaximizedClient(ClientNode *np, MaxFlags flags)
       np->x = box.x;
       np->width = box.width;
       if(!(np->state.status & STAT_IIGNORE)) {
-         np->width -= ((box.width - np->baseWidth) % np->xinc);
+         np->width -= ((np->width - np->baseWidth) % np->xinc);
       }
    } else if(flags & MAX_LEFT) {
       np->x = box.x;
       np->width = box.width / 2 - west;
       if(!(np->state.status & STAT_IIGNORE)) {
-         np->width -= ((box.width - np->baseWidth) % np->xinc);
+         np->width -= ((np->width - np->baseWidth) % np->xinc);
       }
    } else if(flags & MAX_RIGHT) {
       np->x = box.x + box.width / 2 + west;
       np->width = box.width / 2 - east;
       if(!(np->state.status & STAT_IIGNORE)) {
-         np->width -= ((box.width - np->baseWidth) % np->xinc);
+         np->width -= ((np->width - np->baseWidth) % np->xinc);
       }
    }
 
@@ -843,19 +856,19 @@ void PlaceMaximizedClient(ClientNode *np, MaxFlags flags)
       np->y = box.y + north;
       np->height = box.height - north;
       if(!(np->state.status & STAT_IIGNORE)) {
-         np->height -= ((box.height - np->baseHeight) % np->yinc);
+         np->height -= ((np->height - np->baseHeight) % np->yinc);
       }
    } else if(flags & MAX_TOP) {
       np->y = box.y + north;
       np->height = box.height / 2 - north - south;
       if(!(np->state.status & STAT_IIGNORE)) {
-         np->height -= ((box.height - np->baseHeight) % np->yinc);
+         np->height -= ((np->height - np->baseHeight) % np->yinc);
       }
    } else if(flags & MAX_BOTTOM) {
       np->y = box.y + box.height / 2 + north;
       np->height = box.height / 2 - north - south;
       if(!(np->state.status & STAT_IIGNORE)) {
-         np->height -= ((box.height - np->baseHeight) % np->yinc);
+         np->height -= ((np->height - np->baseHeight) % np->yinc);
       }
    }
 
