@@ -10,7 +10,8 @@
 #ifndef PLACE_H
 #define PLACE_H
 
-struct ClientNode;
+#include "client.h"
+
 struct ScreenType;
 struct TrayType;
 
@@ -32,31 +33,30 @@ void ShutdownPlacement(void);
 /** Remove struts associated with a client.
  * @param np The client.
  */
-void RemoveClientStrut(struct ClientNode *np);
+void RemoveClientStrut(ClientNode *np);
 
 /** Read struts associated with a client.
  * @param np The client.
  */
-void ReadClientStrut(struct ClientNode *np);
+void ReadClientStrut(ClientNode *np);
 
 /** Place a client on the screen.
  * @param np The client to place.
  * @param alreadyMapped 1 if already mapped, 0 if unmapped.
  */
-void PlaceClient(struct ClientNode *np, char alreadyMapped);
+void PlaceClient(ClientNode *np, char alreadyMapped);
 
 /** Place a maximized client on the screen.
  * @param np The client to place.
- * @param horiz Set if maximizing horizontally.
- * @param vert Set if maximizing vertically.
+ * @param flags The type of maximization to perform.
  */
-void PlaceMaximizedClient(struct ClientNode *np, char horiz, char vert);
+void PlaceMaximizedClient(ClientNode *np, MaxFlags flags);
 
 /** Move a client window for a border.
  * @param np The client.
  * @param negate 0 to gravitate for a border, 1 to gravitate for no border.
  */
-void GravitateClient(struct ClientNode *np, char negate);
+void GravitateClient(ClientNode *np, char negate);
 
 /** Get the x and y deltas for gravitating a client.
  * @param np The client.
@@ -64,18 +64,18 @@ void GravitateClient(struct ClientNode *np, char negate);
  * @param x Location to store the x delta.
  * @param y Location to store the y delta.
  */
-void GetGravityDelta(const struct ClientNode *np, int gravity, int *x, int *y);
+void GetGravityDelta(const ClientNode *np, int gravity, int *x, int *y);
 
 /** Constrain the size of a client.
  * @param np The client.
  * @return 1 if the size changed, 0 otherwise.
  */
-char ConstrainSize(struct ClientNode *np);
+char ConstrainSize(ClientNode *np);
 
 /** Constrain the position of a client.
  * @param np The  client.
  */
-void ConstrainPosition(struct ClientNode *np);
+void ConstrainPosition(ClientNode *np);
 
 /** Get the bounding box for the screen.
  * @param sp A pointer to the screen whose bounds to get.
