@@ -13,18 +13,20 @@
 struct IconNode;
 struct ImageNode;
 struct ScaledIconNode;
-struct VisualData;
 
 /** Put a scaled icon.
  * @param image The image to display.
+ * @param node The rendered image to display.
  * @param d The drawable on which to render the icon.
  * @param x The x-coordinate to place the icon.
  * @param y The y-coordinate to place the icon.
+ * @param width The width of the icon.
+ * @param height The height of the icon.
  * @return 1 if the icon was successfully rendered, 0 otherwise.
  */
-void PutScaledRenderIcon(const struct VisualData *visual,
-                         const struct ImageNode *image,
-                         Drawable d, int x, int y);
+void PutScaledRenderIcon(const struct IconNode *image,
+                         const struct ScaledIconNode *node,
+                         Drawable d, int x, int y, int width, int height);
 
 /** Create a scaled icon.
  * @param icon The icon.
