@@ -27,7 +27,7 @@ static Menu *rootMenu[ROOT_MENU_COUNT];
 
 static void ExitHandler(ClientNode *np);
 
-static void RunRootCommand(MenuAction *action);
+static void RunRootCommand(MenuAction *action, unsigned button);
 
 /** Initialize root menu data. */
 void InitializeRootMenu(void)
@@ -181,13 +181,13 @@ void GetRootMenuSize(int index, int *width, int *height)
 }
 
 /** Show a root menu. */
-char ShowRootMenu(int index, int x, int y)
+char ShowRootMenu(int index, int x, int y, char keyboard)
 {
 
    if(!rootMenu[index]) {
       return 0;
    }
-   ShowMenu(rootMenu[index], RunRootCommand, x, y);
+   ShowMenu(rootMenu[index], RunRootCommand, x, y, keyboard);
    return 1;
 
 }
@@ -206,9 +206,9 @@ void Restart(void)
 }
 
 /** Exit with optional confirmation. */
-void Exit(void)
+void Exit(char confirm)
 {
-   if(settings.exitConfirmation) {
+   if(confirm) {
       ShowConfirmDialog(NULL, ExitHandler,
                         _("Exit JWM"),
                         _("Are you sure?"),
@@ -233,12 +233,12 @@ void ReloadMenu(void)
 }
 
 /** Root menu callback. */
-void RunRootCommand(MenuAction *action)
+void RunRootCommand(MenuAction *action, unsigned button)
 {
 
    switch(action->type) {
    case MA_EXECUTE:
-      RunCommand(action->data.str);
+      RunCommand(action->str);
       break;
    case MA_RESTART:
       Restart();
@@ -247,11 +247,11 @@ void RunRootCommand(MenuAction *action)
       if(exitCommand) {
          Release(exitCommand);
       }
-      exitCommand = CopyString(action->data.str);
-      Exit();
+      exitCommand = CopyString(action->str);
+      Exit(action->value);
       break;
    case MA_DESKTOP:
-      ChangeDesktop(action->data.i);
+      ChangeDesktop(action->value);
       break;
 
    case MA_SENDTO:
