@@ -30,19 +30,21 @@ void InitializeSettings(void)
    settings.popupDelay = 600;
    settings.desktopDelay = 1000;
    settings.trayOpacity = UINT_MAX;
-   settings.popupEnabled = 1;
+   settings.popupMask = POPUP_ALL;
    settings.activeClientOpacity = UINT_MAX;
    settings.inactiveClientOpacity = (unsigned int)(0.75 * UINT_MAX);
    settings.borderWidth = 5;
    settings.titleHeight = 22;
+   settings.titleTextAlignment = ALIGN_LEFT;
    settings.desktopWidth = 4;
    settings.desktopHeight = 1;
    settings.menuOpacity = UINT_MAX;
-   settings.taskInsertMode = INSERT_RIGHT;
-   settings.exitConfirmation = 1;
+   settings.windowDecorations = DECO_FLAT;
+   settings.trayDecorations = DECO_FLAT;
+   settings.menuDecorations = DECO_FLAT;
    settings.cornerRadius = 4;
-   settings.handles = 0;
    settings.groupTasks = 0;
+   settings.listAllTasks = 0;
 }
 
 /** Make sure settings are reasonable. */
