@@ -38,10 +38,26 @@ typedef unsigned char FocusModelType;
 #define FOCUS_SLOPPY 0
 #define FOCUS_CLICK  1
 
-/** Taskbar insert modes. */
-typedef unsigned char InsertModeType;
-#define INSERT_LEFT  0
-#define INSERT_RIGHT 1
+/** Decorations. */
+typedef unsigned char DecorationsType;
+#define DECO_FLAT    0
+#define DECO_MOTIF   1
+
+/** Popup mask. */
+typedef unsigned char PopupMaskType;
+#define POPUP_NONE   0
+#define POPUP_TASK   1
+#define POPUP_PAGER  2
+#define POPUP_BUTTON 4
+#define POPUP_CLOCK  8
+#define POPUP_MENU   16
+#define POPUP_ALL    255
+
+/** Text alignment. */
+typedef unsigned char AlignmentType;
+#define ALIGN_LEFT      0
+#define ALIGN_CENTER    1
+#define ALIGN_RIGHT     2
 
 /** Settings. */
 typedef struct {
@@ -60,17 +76,19 @@ typedef struct {
    unsigned int menuOpacity;
    unsigned int desktopDelay;
    unsigned int cornerRadius;
+   AlignmentType titleTextAlignment;
    SnapModeType snapMode;
    MoveModeType moveMode;
    StatusWindowType moveStatusType;
    StatusWindowType resizeStatusType;
    FocusModelType focusModel;
    ResizeModeType resizeMode;
-   InsertModeType taskInsertMode;
-   char popupEnabled;
-   char exitConfirmation;
-   char handles;
+   DecorationsType windowDecorations;
+   DecorationsType trayDecorations;
+   DecorationsType menuDecorations;
+   PopupMaskType popupMask;
    char groupTasks;
+   char listAllTasks;
 } Settings;
 
 extern Settings settings;
@@ -86,4 +104,3 @@ void StartupSettings(void);
 void SetPathString(char **dest, const char *src);
 
 #endif
-
