@@ -25,12 +25,14 @@
 #include "settings.h"
 #include "event.h"
 #include "misc.h"
+#include "desktop.h"
 
 typedef struct TaskBarType {
 
    TrayComponentType *cp;
 
    int maxItemWidth;
+   int userHeight;
    int itemHeight;
    int itemWidth;
    LayoutType layout;
@@ -66,7 +68,7 @@ static char ShouldFocusEntry(const TaskEntry *tp);
 static TaskEntry *GetEntry(TaskBarType *bar, int x, int y);
 static void Render(const TaskBarType *bp);
 static void ShowClientList(TaskBarType *bar, TaskEntry *tp);
-static void RunTaskBarCommand(MenuAction *action);
+static void RunTaskBarCommand(MenuAction *action, unsigned button);
 
 static void SetSize(TrayComponentType *cp, int width, int height);
 static void Create(TrayComponentType *cp);
@@ -120,6 +122,9 @@ TrayComponentType *CreateTaskBar()
    tp->next = bars;
    bars = tp;
    tp->itemHeight = 0;
+   tp->itemWidth = 0;
+   tp->userHeight = 0;
+   tp->maxItemWidth = 0;
    tp->layout = LAYOUT_HORIZONTAL;
    tp->mousex = -settings.doubleClickDelta;
    tp->mousey = -settings.doubleClickDelta;
@@ -171,7 +176,7 @@ void Create(TrayComponentType *cp)
 {
    TaskBarType *tp = (TaskBarType*)cp->object;
    cp->pixmap = JXCreatePixmap(display, rootWindow, cp->width, cp->height,
-                               rootVisual.depth);
+                               rootDepth);
    tp->buffer = cp->pixmap;
    ClearTrayDrawable(cp);
 }
@@ -184,7 +189,7 @@ void Resize(TrayComponentType *cp)
       JXFreePixmap(display, tp->buffer);
    }
    cp->pixmap = JXCreatePixmap(display, rootWindow, cp->width, cp->height,
-                               rootVisual.depth);
+                               rootDepth);
    tp->buffer = cp->pixmap;
    ClearTrayDrawable(cp);
 }
@@ -195,7 +200,11 @@ void ComputeItemSize(TaskBarType *tp)
    TrayComponentType *cp = tp->cp;
    if(tp->layout == LAYOUT_VERTICAL) {
 
-      tp->itemHeight = GetStringHeight(FONT_TASK) + 12;
+      if(tp->userHeight > 0) {
+         tp->itemHeight = tp->userHeight;
+      } else {
+         tp->itemHeight = GetStringHeight(FONT_TRAY) + 12;
+      }
       tp->itemWidth = cp->width;
 
    } else {
@@ -228,38 +237,97 @@ void ProcessTaskButtonEvent(TrayComponentType *cp, int x, int y, int mask)
    TaskEntry *entry = GetEntry(bar, x, y);
 
    if(entry) {
+      ClientEntry *cp;
+      ClientNode *focused = NULL;
+      char onTop = 0;
       char hasActive = 0;
-      int layer;
 
       switch(mask) {
       case Button1:  /* Raise or minimize items in this group. */
-         for(layer = LAST_LAYER; layer >= FIRST_LAYER; layer--) {
-            ClientNode *np;
-            for(np = nodes[layer]; np; np = np->next) {
-               ClientEntry *cp;
-               if(np->state.status & STAT_MINIMIZED) {
-                  continue;
-               } else if(!ShouldFocus(np)) {
-                  continue;
-               }
-               for(cp = entry->clients; cp; cp = cp->next) {
-                  if(cp->client == np) {
-                     hasActive = 1;
-                     break;
+         for(cp = entry->clients; cp; cp = cp->next) {
+            int layer;
+            char foundTop = 0;
+            if(cp->client->state.status & STAT_MINIMIZED) {
+               continue;
+            } else if(!ShouldFocus(cp->client, 0)) {
+               continue;
+            }
+            for(layer = LAST_LAYER; layer >= FIRST_LAYER; layer--) {
+               ClientNode *np;
+               for(np = nodes[layer]; np; np = np->next) {
+                  if(np->state.status & STAT_MINIMIZED) {
+                     continue;
+                  } else if(!ShouldFocus(np, 0)) {
+                     continue;
                   }
+                  if(np == cp->client) {
+                     const char isActive = (np->state.status & STAT_ACTIVE)
+                                         && IsClientOnCurrentDesktop(np);
+                     onTop = onTop || !foundTop;
+                     if(isActive) {
+                        focused = np;
+                     }
+                     if(!(cp->client->state.status
+                           & (STAT_CANFOCUS | STAT_TAKEFOCUS))
+                        || isActive) {
+                        hasActive = 1;
+                     }
+                  }
+                  if(hasActive && onTop) {
+                     goto FoundActiveAndTop;
+                  }
+                  foundTop = 1;
                }
-               break;
             }
          }
-         if(hasActive) {
-            MinimizeGroup(entry);
+FoundActiveAndTop:
+         if(hasActive && onTop) {
+            ClientNode *nextClient = NULL;
+            int i;
+
+            /* Try to find a client on a different desktop. */
+            for(i = 0; i < settings.desktopCount - 1; i++) {
+               const int target = (currentDesktop + i + 1)
+                                % settings.desktopCount;
+               for(cp = entry->clients; cp; cp = cp->next) {
+                  ClientNode *np = cp->client;
+                  if(!ShouldFocus(np, 0)) {
+                     continue;
+                  } else if(np->state.status & STAT_STICKY) {
+                     continue;
+                  } else if(np->state.desktop == target) {
+                     if(!nextClient || np->state.status & STAT_ACTIVE) {
+                        nextClient = np;
+                     }
+                  }
+               }
+               if(nextClient) {
+                  break;
+               }
+            }
+            /* Focus the next client or minimize the current group. */
+            if(nextClient) {
+               ChangeDesktop(nextClient->state.desktop);
+               RestoreClient(nextClient, 1);
+            } else {
+               MinimizeGroup(entry);
+            }
          } else {
             FocusGroup(entry);
+            if(focused) {
+               FocusClient(focused);
+            }
          }
          break;
       case Button3:
          ShowClientList(bar, entry);
          break;
+      case Button4:
+         FocusPrevious();
+         break;
+      case Button5:
+         FocusNext();
+         break;
       default:
          break;
       }
@@ -272,7 +340,7 @@ void MinimizeGroup(const TaskEntry *tp)
 {
    ClientEntry *cp;
    for(cp = tp->clients; cp; cp = cp->next) {
-      if(ShouldFocus(cp->client)) {
+      if(ShouldFocus(cp->client, 1)) {
          MinimizeClient(cp->client, 0);
       }
    }
@@ -283,23 +351,55 @@ void FocusGroup(const TaskEntry *tp)
 {
    const char *className = tp->clients->client->className;
    ClientNode **toRestore;
+   const ClientEntry *cp;
    unsigned restoreCount;
    int i;
+   char shouldSwitch;
 
    /* If there is no class name, then there will only be one client. */
    if(!className || !settings.groupTasks) {
+      if(!(tp->clients->client->state.status & STAT_STICKY)) {
+         ChangeDesktop(tp->clients->client->state.desktop);
+      }
       RestoreClient(tp->clients->client, 1);
       FocusClient(tp->clients->client);
       return;
    }
 
+   /* If there is a client in the group on this desktop,
+    * then we remain on the same desktop. */
+   shouldSwitch = 1;
+   for(cp = tp->clients; cp; cp = cp->next) {
+      if(IsClientOnCurrentDesktop(cp->client)) {
+         shouldSwitch = 0;
+         break;
+      }
+   }
+
+   /* Switch to the desktop of the top-most client in the group. */
+   if(shouldSwitch) {
+      for(i = 0; i < LAYER_COUNT; i++) {
+         ClientNode *np;
+         for(np = nodes[i]; np; np = np->next) {
+            if(np->className && !strcmp(np->className, className)) {
+               if(ShouldFocus(np, 0)) {
+                  if(!(np->state.status & STAT_STICKY)) {
+                     ChangeDesktop(np->state.desktop);
+                  }
+                  break;
+               }
+            }
+         }
+      }
+   }
+
    /* Build up the list of clients to restore in correct order. */
    toRestore = AllocateStack(sizeof(ClientNode*) * clientCount);
    restoreCount = 0;
    for(i = 0; i < LAYER_COUNT; i++) {
       ClientNode *np;
       for(np = nodes[i]; np; np = np->next) {
-         if(!ShouldFocus(np)) {
+         if(!ShouldFocus(np, 1)) {
             continue;
          }
          if(np->className && !strcmp(np->className, className)) {
@@ -313,7 +413,7 @@ void FocusGroup(const TaskEntry *tp)
       RestoreClient(toRestore[i], 1);
    }
    for(i = 0; i < restoreCount; i++) {
-      if(toRestore[i]->state.status & STAT_CANFOCUS) {
+      if(toRestore[i]->state.status & (STAT_CANFOCUS | STAT_TAKEFOCUS)) {
          FocusClient(toRestore[i]);
          break;
       }
@@ -348,21 +448,21 @@ void ShowClientList(TaskBarType *bar, TaskEntry *tp)
       menu->items = NULL;
       menu->label = NULL;
 
-      item = CreateMenuItem(MENU_ITEM_SUBMENU);
+      item = CreateMenuItem(MENU_ITEM_NORMAL);
       item->name = CopyString(_("Close"));
       item->action.type = MA_CLOSE | MA_GROUP_MASK;
       item->action.context = tp;
       item->next = menu->items;
       menu->items = item;
 
-      item = CreateMenuItem(MENU_ITEM_SUBMENU);
+      item = CreateMenuItem(MENU_ITEM_NORMAL);
       item->name = CopyString(_("Minimize"));
       item->action.type = MA_MINIMIZE | MA_GROUP_MASK;
       item->action.context = tp;
       item->next = menu->items;
       menu->items = item;
 
-      item = CreateMenuItem(MENU_ITEM_SUBMENU);
+      item = CreateMenuItem(MENU_ITEM_NORMAL);
       item->name = CopyString(_("Restore"));
       item->action.type = MA_RESTORE | MA_GROUP_MASK;
       item->action.context = tp;
@@ -383,7 +483,7 @@ void ShowClientList(TaskBarType *bar, TaskEntry *tp)
 
       /* Load the clients into the menu. */
       for(cp = tp->clients; cp; cp = cp->next) {
-         if(!ShouldFocus(cp->client)) {
+         if(!ShouldFocus(cp->client, 0)) {
             continue;
          }
          item = CreateMenuItem(MENU_ITEM_NORMAL);
@@ -437,26 +537,26 @@ void ShowClientList(TaskBarType *bar, TaskEntry *tp)
       y = Max(y, sp->y);
    }
 
-   ShowMenu(menu, RunTaskBarCommand, x, y);
+   ShowMenu(menu, RunTaskBarCommand, x, y, 0);
 
    DestroyMenu(menu);
 
 }
 
 /** Run a menu action. */
-void RunTaskBarCommand(MenuAction *action)
+void RunTaskBarCommand(MenuAction *action, unsigned button)
 {
    ClientEntry *cp;
 
    if(action->type & MA_GROUP_MASK) {
       TaskEntry *tp = action->context;
       for(cp = tp->clients; cp; cp = cp->next) {
-         if(!ShouldFocus(cp->client)) {
+         if(!ShouldFocus(cp->client, 0)) {
             continue;
          }
          switch(action->type & ~MA_GROUP_MASK) {
          case MA_SENDTO:
-            SetClientDesktop(cp->client, action->data.i);
+            SetClientDesktop(cp->client, action->value);
             break;
          case MA_CLOSE:
             DeleteClient(cp->client);
@@ -472,11 +572,11 @@ void RunTaskBarCommand(MenuAction *action)
          }
       }
    } else if(action->type == MA_EXECUTE) {
-      if(action->button == Button3) {
+      if(button == Button3) {
          Window w;
          int x, y;
          GetMousePosition(&x, &y, &w);
-         ShowWindowMenu(action->context, x, y);
+         ShowWindowMenu(action->context, x, y, 0);
       } else {
          ClientNode *np = action->context;
          RestoreClient(np, 1);
@@ -484,7 +584,7 @@ void RunTaskBarCommand(MenuAction *action)
          MoveMouse(np->window, np->width / 2, np->height / 2);
       }
    } else {
-      RunWindowCommand(action);
+      RunWindowCommand(action, button);
    }
 }
 
@@ -506,25 +606,14 @@ void AddClientToTaskBar(ClientNode *np)
    if(tp == NULL) {
       tp = Allocate(sizeof(TaskEntry));
       tp->clients = NULL;
-      if(settings.taskInsertMode == INSERT_LEFT) {
-         tp->prev = NULL;
-         tp->next = taskEntries;
-         if(taskEntries) {
-            taskEntries->prev = tp;
-         } else {
-            taskEntriesTail = tp;
-         }
-         taskEntries = tp;
+      tp->next = NULL;
+      tp->prev = taskEntriesTail;
+      if(taskEntriesTail) {
+         taskEntriesTail->next = tp;
       } else {
-         tp->next = NULL;
-         tp->prev = taskEntriesTail;
-         if(taskEntriesTail) {
-            taskEntriesTail->next = tp;
-         } else {
-            taskEntries = tp;
-         }
-         taskEntriesTail = tp;
+         taskEntries = tp;
       }
+      taskEntriesTail = tp;
    }
 
    cp->next = tp->clients;
@@ -569,7 +658,7 @@ void RemoveClientFromTaskBar(ClientNode *np)
                }
                Release(tp);
             }
-            UpdateTaskBar();
+            RequireTaskUpdate();
             UpdateNetClientList();
             return;
          }
@@ -591,13 +680,18 @@ void UpdateTaskBar(void)
       if(bp->layout == LAYOUT_VERTICAL) {
          TaskEntry *tp;
          lastHeight = bp->cp->requestedHeight;
-         bp->itemHeight = GetStringHeight(FONT_TASK) + 12;
-         bp->cp->requestedHeight = 2;
+         if(bp->userHeight > 0) {
+            bp->itemHeight = bp->userHeight;
+         } else {
+            bp->itemHeight = GetStringHeight(FONT_TRAY) + 12;
+         }
+         bp->cp->requestedHeight = 0;
          for(tp = taskEntries; tp; tp = tp->next) {
             if(ShouldShowEntry(tp)) {
                bp->cp->requestedHeight += bp->itemHeight;
             }
          }
+         bp->cp->requestedHeight = Max(1, bp->cp->requestedHeight);
          if(lastHeight != bp->cp->requestedHeight) {
             ResizeTray(bp->cp->tray);
          }
@@ -621,11 +715,11 @@ void SignalTaskbar(const TimeType *now, int x, int y, Window w, void *data)
          ep = GetEntry(bp, x - bp->cp->screenx, y - bp->cp->screeny);
          if(settings.groupTasks) {
             if(ep && ep->clients->client->className) {
-               ShowPopup(x, y, ep->clients->client->className);
+               ShowPopup(x, y, ep->clients->client->className, POPUP_TASK);
             }
          } else {
             if(ep && ep->clients->client->name) {
-               ShowPopup(x, y, ep->clients->client->name);
+               ShowPopup(x, y, ep->clients->client->name, POPUP_TASK);
             }
          }
       }
@@ -637,6 +731,7 @@ void SignalTaskbar(const TimeType *now, int x, int y, Window w, void *data)
 void Render(const TaskBarType *bp)
 {
    TaskEntry *tp;
+   char *displayName;
    ButtonNode button;
    int x, y;
 
@@ -650,8 +745,9 @@ void Render(const TaskBarType *bp)
       return;
    }
 
-   ResetButton(&button, bp->cp->pixmap, &rootVisual);
-   button.font = FONT_TASK;
+   ResetButton(&button, bp->cp->pixmap);
+   button.border = settings.trayDecorations == DECO_MOTIF;
+   button.font = FONT_TRAY;
    button.height = bp->itemHeight;
    button.width = bp->itemWidth;
    button.text = NULL;
@@ -664,24 +760,50 @@ void Render(const TaskBarType *bp)
          continue;
       }
 
-      /* Check for an active or urgent window. */
+      /* Check for an active or urgent window and count clients. */
       ClientEntry *cp;
+      unsigned clientCount = 0;
       button.type = BUTTON_TASK;
       for(cp = tp->clients; cp; cp = cp->next) {
-         if(cp->client->state.status & (STAT_ACTIVE | STAT_FLASH)) {
-            button.type = BUTTON_TASK_ACTIVE;
-            break;
+         if(ShouldFocus(cp->client, 0)) {
+            const char flash = cp->client->state.status & STAT_FLASH;
+            const char active = (cp->client->state.status & STAT_ACTIVE)
+               && IsClientOnCurrentDesktop(cp->client);
+            if(flash || active) {
+               if(button.type == BUTTON_TASK) {
+                  button.type = BUTTON_TASK_ACTIVE;
+               } else {
+                  button.type = BUTTON_TASK;
+               }
+            }
+            clientCount += 1;
          }
       }
       button.x = x;
       button.y = y;
-      button.icon = tp->clients->client->icon;
+      if(tp->clients->client->icon == &emptyIcon) {
+         button.icon = NULL;
+      } else {
+         button.icon = tp->clients->client->icon;
+      }
+      displayName = NULL;
       if(tp->clients->client->className && settings.groupTasks) {
-         button.text = tp->clients->client->className;
+         if(clientCount != 1) {
+            const size_t len = strlen(tp->clients->client->className) + 16;
+            displayName = Allocate(len);
+            snprintf(displayName, len, "%s (%u)",
+                     tp->clients->client->className, clientCount);
+            button.text = displayName;
+         } else {
+            button.text = tp->clients->client->className;
+         }
       } else {
          button.text = tp->clients->client->name;
       }
       DrawButton(&button);
+      if(displayName) {
+         Release(displayName);
+      }
 
       if(bp->layout == LAYOUT_HORIZONTAL) {
          x += bp->itemWidth;
@@ -703,8 +825,8 @@ void FocusNext(void)
    for(tp = taskEntries; tp; tp = tp->next) {
       ClientEntry *cp;
       for(cp = tp->clients; cp; cp = cp->next) {
-         if(cp->client->state.status & STAT_CANFOCUS) {
-            if(ShouldFocus(cp->client)) {
+         if(cp->client->state.status & (STAT_CANFOCUS | STAT_TAKEFOCUS)) {
+            if(ShouldFocus(cp->client, 1)) {
                if(cp->client->state.status & STAT_ACTIVE) {
                   cp = cp->next;
                   goto ClientFound;
@@ -745,8 +867,8 @@ void FocusPrevious(void)
    for(tp = taskEntries; tp; tp = tp->next) {
       ClientEntry *cp;
       for(cp = tp->clients; cp; cp = cp->next) {
-         if(cp->client->state.status & STAT_CANFOCUS) {
-            if(ShouldFocus(cp->client)) {
+         if(cp->client->state.status & (STAT_CANFOCUS | STAT_TAKEFOCUS)) {
+            if(ShouldFocus(cp->client, 1)) {
                if(cp->client->state.status & STAT_ACTIVE) {
                   cp = cp->next;
                   goto ClientFound;
@@ -783,7 +905,7 @@ char ShouldShowEntry(const TaskEntry *tp)
 {
    const ClientEntry *cp;
    for(cp = tp->clients; cp; cp = cp->next) {
-      if(ShouldFocus(cp->client)) {
+      if(ShouldFocus(cp->client, 0)) {
          return 1;
       }
    }
@@ -795,8 +917,8 @@ char ShouldFocusEntry(const TaskEntry *tp)
 {
    const ClientEntry *cp;
    for(cp = tp->clients; cp; cp = cp->next) {
-      if(cp->client->state.status & STAT_CANFOCUS) {
-         if(ShouldFocus(cp->client)) {
+      if(cp->client->state.status & (STAT_CANFOCUS | STAT_TAKEFOCUS)) {
+         if(ShouldFocus(cp->client, 1)) {
             return 1;
          }
       }
@@ -848,6 +970,20 @@ void SetMaxTaskBarItemWidth(TrayComponentType *cp, const char *value)
    bp->maxItemWidth = temp;
 }
 
+/** Set the preferred height of the specified task bar. */
+void SetTaskBarHeight(TrayComponentType *cp, const char *value)
+{
+   TaskBarType *bp = (TaskBarType*)cp->object;
+   int temp;
+
+   temp = atoi(value);
+   if(JUNLIKELY(temp < 0)) {
+      Warning(_("invalid height for TaskList: %s"), value);
+      return;
+   }
+   bp->userHeight = temp;
+}
+
 /** Maintain the _NET_CLIENT_LIST[_STACKING] properties on the root. */
 void UpdateNetClientList(void)
 {
@@ -895,4 +1031,3 @@ void UpdateNetClientList(void)
    }
    
 }
-
