@@ -22,10 +22,13 @@
 #include "event.h"
 #include "client.h"
 #include "misc.h"
+#include "hint.h"
 
 #define DEFAULT_TRAY_WIDTH 32
 #define DEFAULT_TRAY_HEIGHT 32
 
+#define TRAY_BORDER_SIZE   1
+
 static TrayType *trays;
 static unsigned int trayCount;
 
@@ -61,9 +64,7 @@ void InitializeTray(void)
 /** Startup trays. */
 void StartupTray(void)
 {
-
    XSetWindowAttributes attr;
-   Atom atom;
    unsigned long attrMask;
    TrayType *tp;
    TrayComponentType *cp;
@@ -97,35 +98,30 @@ void StartupTray(void)
       attrMask |= CWBackPixel;
       attr.background_pixel = colors[COLOR_TRAY_BG2];
 
-      attrMask |= CWBorderPixel;
-      attr.border_pixel = colors[COLOR_TRAY_OUTLINE];
-
       Assert(tp->width > 0);
       Assert(tp->height > 0);
       tp->window = JXCreateWindow(display, rootWindow,
-                                  tp->x, tp->y, tp->width, tp->height,
-                                  TRAY_BORDER_SIZE,
-                                  rootVisual.depth, InputOutput,
-                                  rootVisual.visual, attrMask, &attr);
+                                  tp->x, tp->y, tp->width, tp->height, 0,
+                                  rootDepth, InputOutput,
+                                  rootVisual, attrMask, &attr);
+      SetAtomAtom(tp->window, ATOM_NET_WM_WINDOW_TYPE,
+                  ATOM_NET_WM_WINDOW_TYPE_DOCK);
 
       if(settings.trayOpacity < UINT_MAX) {
-         /* Can't use atoms yet as it hasn't been initialized. */
-         atom = JXInternAtom(display, opacityAtom, False);
-         JXChangeProperty(display, tp->window, atom, XA_CARDINAL, 32,
-                          PropModeReplace,
-                          (unsigned char*)&settings.trayOpacity, 1);
+         SetCardinalAtom(tp->window, ATOM_NET_WM_WINDOW_OPACITY,
+                         settings.trayOpacity);
       }
 
       SetDefaultCursor(tp->window);
 
       /* Create and layout items on the tray. */
-      xoffset = 0;
-      yoffset = 0;
+      xoffset = TRAY_BORDER_SIZE;
+      yoffset = TRAY_BORDER_SIZE;
       for(cp = tp->components; cp; cp = cp->next) {
 
          if(cp->Create) {
             if(tp->layout == LAYOUT_HORIZONTAL) {
-               height = tp->height;
+               height = tp->height - TRAY_BORDER_SIZE * 2;
                width = cp->width;
                if(width == 0) {
                   width = variableSize;
@@ -135,7 +131,7 @@ void StartupTray(void)
                   }
                }
             } else {
-               width = tp->width;
+               width = tp->width - TRAY_BORDER_SIZE * 2;
                height = cp->height;
                if(height == 0) {
                   height = variableSize;
@@ -176,13 +172,11 @@ void StartupTray(void)
 
    RequirePagerUpdate();
    RequireTaskUpdate();
-
 }
 
 /** Shutdown trays. */
 void ShutdownTray(void)
 {
-
    TrayType *tp;
    TrayComponentType *cp;
 
@@ -194,13 +188,11 @@ void ShutdownTray(void)
       }
       JXDestroyWindow(display, tp->window);
    }
-
 }
 
 /** Destroy tray data. */
 void DestroyTray(void)
 {
-
    TrayType *tp;
    TrayComponentType *cp;
 
@@ -216,13 +208,11 @@ void DestroyTray(void)
 
       trays = tp;
    }
-
 }
 
 /** Create an empty tray. */
 TrayType *CreateTray(void)
 {
-
    TrayType *tp;
 
    tp = Allocate(sizeof(TrayType));
@@ -254,13 +244,11 @@ TrayType *CreateTray(void)
    RegisterCallback(100, SignalTray, tp);
 
    return tp;
-
 }
 
 /** Create an empty tray component. */
 TrayComponentType *CreateTrayComponent(void)
 {
-
    TrayComponentType *cp;
 
    cp = Allocate(sizeof(TrayComponentType));
@@ -293,18 +281,12 @@ TrayComponentType *CreateTrayComponent(void)
    cp->next = NULL;
 
    return cp;
-
 }
 
 /** Add a tray component to a tray. */
 void AddTrayComponent(TrayType *tp, TrayComponentType *cp)
 {
-
-   Assert(tp);
-   Assert(cp);
-
    cp->tray = tp;
-
    if(tp->componentsTail) {
       tp->componentsTail->next = cp;
    } else {
@@ -312,13 +294,11 @@ void AddTrayComponent(TrayType *tp, TrayComponentType *cp)
    }
    tp->componentsTail = cp;
    cp->next = NULL;
-
 }
 
 /** Compute the max component width. */
 int ComputeMaxWidth(TrayType *tp)
 {
-
    TrayComponentType *cp;
    int result;
    int temp;
@@ -334,29 +314,25 @@ int ComputeMaxWidth(TrayType *tp)
    }
 
    return result;
-
 }
 
 /** Compute the total width of a tray. */
 int ComputeTotalWidth(TrayType *tp)
 {
-
    TrayComponentType *cp;
    int result;
 
-   result = 0;
+   result = 2 * TRAY_BORDER_SIZE;
    for(cp = tp->components; cp; cp = cp->next) {
       result += cp->width;
    }
 
    return result;
-
 }
 
 /** Compute the max component height. */
 int ComputeMaxHeight(TrayType *tp)
 {
-
    TrayComponentType *cp;
    int result;
    int temp;
@@ -372,29 +348,25 @@ int ComputeMaxHeight(TrayType *tp)
    }
 
    return result;
-
 }
 
 /** Compute the total height of a tray. */
 int ComputeTotalHeight(TrayType *tp)
 {
-
    TrayComponentType *cp;
    int result;
 
-   result = 0;
+   result = 2 * TRAY_BORDER_SIZE;
    for(cp = tp->components; cp; cp = cp->next) {
       result += cp->height;
    }
 
    return result;
-
 }
 
 /** Check if the tray fills the screen horizontally. */
 char CheckHorizontalFill(TrayType *tp)
 {
-
    TrayComponentType *cp;
 
    for(cp = tp->components; cp; cp = cp->next) {
@@ -404,13 +376,11 @@ char CheckHorizontalFill(TrayType *tp)
    }
 
    return 0;
-
 }
 
 /** Check if the tray fills the screen vertically. */
 char CheckVerticalFill(TrayType *tp)
 {
-
    TrayComponentType *cp;
 
    for(cp = tp->components; cp; cp = cp->next) {
@@ -420,13 +390,11 @@ char CheckVerticalFill(TrayType *tp)
    }
 
    return 0;
-
 }
 
 /** Compute the size of a tray. */
 void ComputeTraySize(TrayType *tp)
 {
-
    TrayComponentType *cp;
    const ScreenType *sp;
    int x, y;
@@ -435,7 +403,7 @@ void ComputeTraySize(TrayType *tp)
    if(tp->layout == LAYOUT_HORIZONTAL) {
 
       if(tp->height == 0) {
-         tp->height = ComputeMaxHeight(tp);
+         tp->height = ComputeMaxHeight(tp) + TRAY_BORDER_SIZE * 2;
       }
       if(tp->height == 0) {
          tp->height = DEFAULT_TRAY_HEIGHT;
@@ -444,7 +412,7 @@ void ComputeTraySize(TrayType *tp)
    } else {
 
       if(tp->width == 0) {
-         tp->width = ComputeMaxWidth(tp);
+         tp->width = ComputeMaxWidth(tp) + TRAY_BORDER_SIZE * 2;
       }
       if(tp->width == 0) {
          tp->width = DEFAULT_TRAY_WIDTH;
@@ -456,9 +424,9 @@ void ComputeTraySize(TrayType *tp)
    for(cp = tp->components; cp; cp = cp->next) {
       if(cp->SetSize) {
          if(tp->layout == LAYOUT_HORIZONTAL) {
-            (cp->SetSize)(cp, 0, tp->height);
+            (cp->SetSize)(cp, 0, tp->height - TRAY_BORDER_SIZE * 2);
          } else {
-            (cp->SetSize)(cp, tp->width, 0);
+            (cp->SetSize)(cp, tp->width - TRAY_BORDER_SIZE * 2, 0);
          }
       }
    }
@@ -534,48 +502,42 @@ void ComputeTraySize(TrayType *tp)
    /* Compute the tray location. */
    switch(tp->valign) {
    case TALIGN_TOP:
-      tp->y = sp->y - TRAY_BORDER_SIZE;
+      tp->y = sp->y;
       break;
    case TALIGN_BOTTOM:
-      tp->y = sp->y + sp->height - tp->height + TRAY_BORDER_SIZE;
+      tp->y = sp->y + sp->height - tp->height;
       break;
    case TALIGN_CENTER:
       tp->y = sp->y + (sp->height - tp->height) / 2;
       break;
    default:
       if(tp->y < 0) {
-         tp->y = sp->y + sp->height - tp->height + TRAY_BORDER_SIZE;
-      } else {
-         tp->y -= TRAY_BORDER_SIZE;
+         tp->y = sp->y + sp->height - tp->height;
       }
       break;
    }
 
    switch(tp->halign) {
    case TALIGN_LEFT:
-      tp->x = sp->x - TRAY_BORDER_SIZE;
+      tp->x = sp->x;
       break;
    case TALIGN_RIGHT:
-      tp->x = sp->x + sp->width - tp->width + TRAY_BORDER_SIZE;
+      tp->x = sp->x + sp->width - tp->width;
       break;
    case TALIGN_CENTER:
       tp->x = sp->x + (sp->width - tp->width) / 2;
       break;
    default:
       if(tp->x < 0) {
-         tp->x = sp->x + sp->width - tp->width + TRAY_BORDER_SIZE;
-      } else {
-         tp->x -= TRAY_BORDER_SIZE;
+         tp->x = sp->x + sp->width - tp->width;
       }
       break;
    }
-
 }
 
 /** Display a tray (for autohide). */
 void ShowTray(TrayType *tp)
 {
-
    Window win1, win2;
    int winx, winy;
    unsigned int mask;
@@ -591,13 +553,11 @@ void ShowTray(TrayType *tp)
       SetMousePosition(mousex, mousey, win2);
 
    }
-
 }
 
 /** Show all trays. */
 void ShowAllTrays(void)
 {
-
    TrayType *tp;
 
    if(shouldExit) {
@@ -607,13 +567,11 @@ void ShowAllTrays(void)
    for(tp = trays; tp; tp = tp->next) {
       ShowTray(tp);
    }
-
 }
 
 /** Hide a tray (for autohide). */
 void HideTray(TrayType *tp)
 {
-
    const ScreenType *sp;
    int x, y;
 
@@ -628,20 +586,20 @@ void HideTray(TrayType *tp)
    sp = GetCurrentScreen(tp->x, tp->y);
    switch(tp->autoHide) {
    case THIDE_LEFT:
-      x = sp->y - tp->width - TRAY_BORDER_SIZE;
+      x = sp->y - tp->width + 1;
       y = tp->y;
       break;
    case THIDE_RIGHT:
-      x = sp->y + sp->width - TRAY_BORDER_SIZE;
+      x = sp->y + sp->width - 1;
       y = tp->y;
       break;
    case THIDE_TOP:
       x = tp->x;
-      y = sp->y - tp->height - TRAY_BORDER_SIZE;
+      y = sp->y - tp->height + 1;
       break;
    case THIDE_BOTTOM:
       x = tp->x;
-      y = sp->y + sp->height - TRAY_BORDER_SIZE;
+      y = sp->y + sp->height - 1;
       break;
    default:
       Assert(0);
@@ -651,13 +609,11 @@ void HideTray(TrayType *tp)
    /* Move and redraw. */
    JXMoveWindow(display, tp->window, x, y);
    DrawSpecificTray(tp);
-
 }
 
 /** Process a tray event. */
 char ProcessTrayEvent(const XEvent *event)
 {
-
    TrayType *tp;
 
    for(tp = trays; tp; tp = tp->next) {
@@ -685,7 +641,6 @@ char ProcessTrayEvent(const XEvent *event)
    }
 
    return 0;
-
 }
 
 /** Signal the tray (needed for autohide). */
@@ -715,31 +670,29 @@ void HandleTrayEnterNotify(TrayType *tp, const XCrossingEvent *event)
 /** Get the tray component under the given coordinates. */
 TrayComponentType *GetTrayComponent(TrayType *tp, int x, int y)
 {
-
    TrayComponentType *cp;
    int xoffset, yoffset;
 
    xoffset = 0;
    yoffset = 0;
    for(cp = tp->components; cp; cp = cp->next) {
-      int startx = xoffset;
-      int starty = yoffset;
-      int width = cp->width;
-      int height = cp->height;
+      const int startx = xoffset;
+      const int starty = yoffset;
+      const int width = cp->width;
+      const int height = cp->height;
       if(x >= startx && x < startx + width) {
          if(y >= starty && y < starty + height) {
             return cp;
          }
       }
       if(tp->layout == LAYOUT_HORIZONTAL) {
-         xoffset += cp->width;
+         xoffset += width;
       } else {
-         yoffset += cp->height;
+         yoffset += height;
       }
    }
 
    return NULL;
-
 }
 
 /** Handle a button press on a tray. */
@@ -750,6 +703,7 @@ void HandleTrayButtonPress(TrayType *tp, const XButtonEvent *event)
       const int x = event->x - cp->x;
       const int y = event->y - cp->y;
       const int mask = event->button;
+      DiscardButtonEvents();
       (cp->ProcessButtonPress)(cp, x, y, mask);
    }
 }
@@ -757,7 +711,6 @@ void HandleTrayButtonPress(TrayType *tp, const XButtonEvent *event)
 /** Handle a button release on a tray. */
 void HandleTrayButtonRelease(TrayType *tp, const XButtonEvent *event)
 {
-
    TrayComponentType *cp;
 
    // First inform any components that have a grab.
@@ -780,13 +733,11 @@ void HandleTrayButtonRelease(TrayType *tp, const XButtonEvent *event)
       const int mask = event->button;
       (cp->ProcessButtonRelease)(cp, x, y, mask);
    }
-
 }
 
 /** Handle a motion notify event. */
 void HandleTrayMotionNotify(TrayType *tp, const XMotionEvent *event)
 {
-
    TrayComponentType *cp = GetTrayComponent(tp, event->x, event->y);
    if(cp && cp->ProcessMotionEvent) {
       const int x = event->x - cp->x;
@@ -794,13 +745,11 @@ void HandleTrayMotionNotify(TrayType *tp, const XMotionEvent *event)
       const int mask = event->state;
       (cp->ProcessMotionEvent)(cp, x, y, mask);
    }
-
 }
 
 /** Draw all trays. */
 void DrawTray(void)
 {
-
    TrayType *tp;
 
    if(shouldExit) {
@@ -810,19 +759,32 @@ void DrawTray(void)
    for(tp = trays; tp; tp = tp->next) {
       DrawSpecificTray(tp);
    }
-
 }
 
 /** Draw a specific tray. */
 void DrawSpecificTray(const TrayType *tp)
 {
-
    TrayComponentType *cp;
 
    for(cp = tp->components; cp; cp = cp->next) {
       UpdateSpecificTray(tp, cp);
    }
 
+   if(settings.trayDecorations == DECO_MOTIF) {
+      JXSetForeground(display, rootGC, colors[COLOR_TRAY_UP]);
+      JXDrawLine(display, tp->window, rootGC, 0, 0, tp->width - 1, 0);
+      JXDrawLine(display, tp->window, rootGC, 0, tp->height - 1, 0, 0);
+
+      JXSetForeground(display, rootGC, colors[COLOR_TRAY_DOWN]);
+      JXDrawLine(display, tp->window, rootGC, 0, tp->height - 1,
+                 tp->width - 1, tp->height - 1);
+      JXDrawLine(display, tp->window, rootGC, tp->width - 1, 0,
+                 tp->width - 1, tp->height - 1);
+   } else {
+      JXSetForeground(display, rootGC, colors[COLOR_TRAY_DOWN]);
+      JXDrawRectangle(display, tp->window, rootGC, 0, 0,
+                      tp->width - 1, tp->height - 1);
+   }
 }
 
 /** Raise tray windows. */
@@ -849,42 +811,55 @@ void LowerTrays(void)
 /** Update a specific component on a tray. */
 void UpdateSpecificTray(const TrayType *tp, const TrayComponentType *cp)
 {
-
    if(JUNLIKELY(shouldExit)) {
       return;
    }
 
    /* If the tray is hidden, draw only the background. */
-   if(!tp->hidden && cp->pixmap != None) {
+   if(cp->pixmap != None) {
       JXCopyArea(display, cp->pixmap, tp->window, rootGC, 0, 0,
                  cp->width, cp->height, cp->x, cp->y);
    }
-
 }
 
 /** Layout tray components on a tray. */
 void LayoutTray(TrayType *tp, int *variableSize, int *variableRemainder)
 {
-
    TrayComponentType *cp;
    unsigned int variableCount;
    int width, height;
    int temp;
 
-   tp->width = tp->requestedWidth;
-   tp->height = tp->requestedHeight;
+   if(tp->requestedWidth >= 0) {
+      tp->width = tp->requestedWidth;
+   } else {
+      tp->width = rootWidth + tp->requestedWidth - tp->x;
+   }
+   if(tp->requestedHeight >= 0) {
+      tp->height = tp->requestedHeight;
+   } else {
+      tp->height = rootHeight + tp->requestedHeight - tp->y;
+   }
 
    for(cp = tp->components; cp; cp = cp->next) {
-      cp->width = cp->requestedWidth;
-      cp->height = cp->requestedHeight;
+      if(cp->requestedWidth != 0) {
+         cp->width = cp->requestedWidth;
+      } else {
+         cp->width = 0;
+      }
+      if(cp->requestedHeight != 0) {
+         cp->height = cp->requestedHeight;
+      } else {
+         cp->height = 0;
+      }
    }
 
    ComputeTraySize(tp);
 
    /* Get the remaining size after setting fixed size components. */
    /* Also, keep track of the number of variable size components. */
-   width = tp->width;
-   height = tp->height;
+   width = tp->width - TRAY_BORDER_SIZE * 2;
+   height = tp->height - TRAY_BORDER_SIZE * 2;
    variableCount = 0;
    for(cp = tp->components; cp; cp = cp->next) {
       if(tp->layout == LAYOUT_HORIZONTAL) {
@@ -932,13 +907,11 @@ void LayoutTray(TrayType *tp, int *variableSize, int *variableRemainder)
 
    tp->width = Max(1, tp->width);
    tp->height = Max(1, tp->height);
-
 }
 
 /** Resize a tray. */
 void ResizeTray(TrayType *tp)
 {
-
    TrayComponentType *cp;
    int variableSize;
    int variableRemainder;
@@ -950,8 +923,8 @@ void ResizeTray(TrayType *tp)
    LayoutTray(tp, &variableSize, &variableRemainder);
 
    /* Reposition items on the tray. */
-   xoffset = 0;
-   yoffset = 0;
+   xoffset = TRAY_BORDER_SIZE;
+   yoffset = TRAY_BORDER_SIZE;
    for(cp = tp->components; cp; cp = cp->next) {
 
       cp->x = xoffset;
@@ -961,7 +934,7 @@ void ResizeTray(TrayType *tp)
 
       if(cp->Resize) {
          if(tp->layout == LAYOUT_HORIZONTAL) {
-            height = tp->height;
+            height = tp->height - TRAY_BORDER_SIZE * 2;
             width = cp->width;
             if(width == 0) {
                width = variableSize;
@@ -971,7 +944,7 @@ void ResizeTray(TrayType *tp)
                }
             }
          } else {
-            width = tp->width;
+            width = tp->width - TRAY_BORDER_SIZE * 2;
             height = cp->height;
             if(height == 0) {
                height = variableSize;
@@ -1006,7 +979,6 @@ void ResizeTray(TrayType *tp)
    if(tp->hidden) {
       HideTray(tp);
    }
-
 }
 
 /** Draw the tray background on a drawable. */
@@ -1061,48 +1033,19 @@ void SetTrayY(TrayType *tp, const char *str)
 /** Set the width of a tray. */
 void SetTrayWidth(TrayType *tp, const char *str)
 {
-
-   int width;
-
-   Assert(tp);
-   Assert(str);
-
-   width = atoi(str);
-
-   if(JUNLIKELY(width < 0)) {
-      Warning(_("invalid tray width: %d"), width);
-   } else {
-      tp->requestedWidth = width;
-   }
-
+   tp->requestedWidth = atoi(str);
 }
 
 /** Set the height of a tray. */
 void SetTrayHeight(TrayType *tp, const char *str)
 {
-
-   int height;
-
-   Assert(tp);
-   Assert(str);
-
-   height = atoi(str);
-
-   if(JUNLIKELY(height < 0)) {
-      Warning(_("invalid tray height: %d"), height);
-   } else {
-      tp->requestedHeight = height;
-   }
-
+   tp->requestedHeight = atoi(str);
 }
 
 
 /** Set the tray orientation. */
 void SetTrayLayout(TrayType *tp, const char *str)
 {
-
-   Assert(tp);
-
    if(!str) {
 
       /* Compute based on requested size. */
@@ -1131,7 +1074,6 @@ void SetTrayLayout(TrayType *tp, const char *str)
    } else {
       tp->layout = LAYOUT_HORIZONTAL;
    }
-
 }
 
 /** Set the layer for a tray. */
